=== modified file 'bzr'
--- bzr	2011-07-15 08:25:00 +0000
+++ bzr	2011-08-12 09:49:24 +0000
@@ -23,7 +23,7 @@
 import warnings
 
 # update this on each release
-_script_version = (2, 4, 0)
+_script_version = (2, 5, 0)
 
 try:
     version_info = sys.version_info

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2011-07-15 08:25:00 +0000
+++ bzrlib/__init__.py	2011-08-12 09:49:24 +0000
@@ -52,7 +52,7 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (2, 4, 0, 'dev', 6)
+version_info = (2, 5, 0, 'dev', 1)
 
 # API compatibility version
 api_minimum_version = (2, 4, 0)

=== modified file 'bzrlib/atomicfile.py'
--- bzrlib/atomicfile.py	2011-06-28 21:47:24 +0000
+++ bzrlib/atomicfile.py	2011-07-15 09:04:54 +0000
@@ -80,13 +80,6 @@
             if stat.S_IMODE(st.st_mode) != new_mode:
                 os.chmod(self.tmpfilename, new_mode)
 
-    def _get_closed(self):
-        symbol_versioning.warn('AtomicFile.closed deprecated in bzr 0.10',
-                               DeprecationWarning, stacklevel=2)
-        return self._fd is None
-
-    closed = property(_get_closed)
-
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__,
                            self.realfilename)

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2011-07-18 14:43:35 +0000
+++ bzrlib/branch.py	2011-08-12 09:49:24 +0000
@@ -66,11 +66,6 @@
 from bzrlib.trace import mutter, mutter_callsite, note, is_quiet
 
 
-BZR_BRANCH_FORMAT_4 = "Bazaar-NG branch, format 0.0.4\n"
-BZR_BRANCH_FORMAT_5 = "Bazaar-NG branch, format 5\n"
-BZR_BRANCH_FORMAT_6 = "Bazaar Branch Format 6 (bzr 0.15)\n"
-
-
 class Branch(controldir.ControlComponent):
     """Branch holding a history of revisions.
 
@@ -1545,10 +1540,15 @@
         # For bzr native formats must_fetch is just the tip, and if_present_fetch
         # are the tags.
         must_fetch = set([self.last_revision()])
-        try:
-            if_present_fetch = set(self.tags.get_reverse_tag_dict())
-        except errors.TagsNotSupported:
-            if_present_fetch = set()
+        if_present_fetch = set()
+        c = self.get_config()
+        include_tags = c.get_user_option_as_bool('branch.fetch_tags',
+                                                 default=False)
+        if include_tags:
+            try:
+                if_present_fetch = set(self.tags.get_reverse_tag_dict())
+            except errors.TagsNotSupported:
+                pass
         must_fetch.discard(_mod_revision.NULL_REVISION)
         if_present_fetch.discard(_mod_revision.NULL_REVISION)
         return must_fetch, if_present_fetch
@@ -3165,7 +3165,7 @@
 
 
 class Converter7to8(object):
-    """Perform an in-place upgrade of format 6 to format 7"""
+    """Perform an in-place upgrade of format 7 to format 8"""
 
     def convert(self, branch):
         format = BzrBranchFormat8()

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2011-07-06 13:30:21 +0000
+++ bzrlib/builtins.py	2011-07-21 07:08:05 +0000
@@ -2317,8 +2317,11 @@
 
     :Other filtering:
 
-      The --message option can be used for finding revisions that match a
-      regular expression in a commit message.
+      The --match option can be used for finding revisions that match a
+      regular expression in a commit message, committer, author or bug.
+      Specifying the option several times will match any of the supplied
+      expressions. --match-author, --match-bugs, --match-committer and
+      --match-message can be used to only match a specific field.
 
     :Tips & tricks:
 
@@ -2384,10 +2387,10 @@
                    argname='N',
                    type=_parse_levels),
             Option('message',
-                   short_name='m',
                    help='Show revisions whose message matches this '
                         'regular expression.',
-                   type=str),
+                   type=str,
+                   hidden=True),
             Option('limit',
                    short_name='l',
                    help='Limit the output to the first N revisions.',
@@ -2404,6 +2407,27 @@
                    ),
             Option('signatures',
                    help='Show digital signature validity'),
+            ListOption('match',
+                short_name='m',
+                help='Show revisions whose properties match this '
+                'expression.',
+                type=str),
+            ListOption('match-message',
+                   help='Show revisions whose message matches this '
+                   'expression.',
+                type=str),
+            ListOption('match-committer',
+                   help='Show revisions whose committer matches this '
+                   'expression.',
+                type=str),
+            ListOption('match-author',
+                   help='Show revisions whose authors match this '
+                   'expression.',
+                type=str),
+            ListOption('match-bugs',
+                   help='Show revisions whose bugs match this '
+                   'expression.',
+                type=str)
             ]
     encoding_type = 'replace'
 
@@ -2423,6 +2447,11 @@
             authors=None,
             exclude_common_ancestry=False,
             signatures=False,
+            match=None,
+            match_message=None,
+            match_committer=None,
+            match_author=None,
+            match_bugs=None,
             ):
         from bzrlib.log import (
             Logger,
@@ -2531,6 +2560,18 @@
         match_using_deltas = (len(file_ids) != 1 or filter_by_dir
             or delta_type or partial_history)
 
+        match_dict = {}
+        if match:
+            match_dict[''] = match
+        if match_message:
+            match_dict['message'] = match_message
+        if match_committer:
+            match_dict['committer'] = match_committer
+        if match_author:
+            match_dict['author'] = match_author
+        if match_bugs:
+            match_dict['bugs'] = match_bugs
+            
         # Build the LogRequest and execute it
         if len(file_ids) == 0:
             file_ids = None
@@ -2539,7 +2580,7 @@
             start_revision=rev1, end_revision=rev2, limit=limit,
             message_search=message, delta_type=delta_type,
             diff_type=diff_type, _match_using_deltas=match_using_deltas,
-            exclude_common_ancestry=exclude_common_ancestry,
+            exclude_common_ancestry=exclude_common_ancestry, match=match_dict,
             signature=signatures
             )
         Logger(b, rqst).show(lf)
@@ -3032,6 +3073,10 @@
 
         old_file_id = rev_tree.path2id(relpath)
 
+        # TODO: Split out this code to something that generically finds the
+        # best id for a path across one or more trees; it's like
+        # find_ids_across_trees but restricted to find just one. -- mbp
+        # 20110705.
         if name_from_revision:
             # Try in revision if requested
             if old_file_id is None:
@@ -3039,41 +3084,26 @@
                     "%r is not present in revision %s" % (
                         filename, rev_tree.get_revision_id()))
             else:
-                content = rev_tree.get_file_text(old_file_id)
+                actual_file_id = old_file_id
         else:
             cur_file_id = tree.path2id(relpath)
-            found = False
-            if cur_file_id is not None:
-                # Then try with the actual file id
-                try:
-                    content = rev_tree.get_file_text(cur_file_id)
-                    found = True
-                except errors.NoSuchId:
-                    # The actual file id didn't exist at that time
-                    pass
-            if not found and old_file_id is not None:
-                # Finally try with the old file id
-                content = rev_tree.get_file_text(old_file_id)
-                found = True
-            if not found:
-                # Can't be found anywhere
+            if cur_file_id is not None and rev_tree.has_id(cur_file_id):
+                actual_file_id = cur_file_id
+            elif old_file_id is not None:
+                actual_file_id = old_file_id
+            else:
                 raise errors.BzrCommandError(
                     "%r is not present in revision %s" % (
                         filename, rev_tree.get_revision_id()))
         if filtered:
-            from bzrlib.filters import (
-                ContentFilterContext,
-                filtered_output_bytes,
-                )
-            filters = rev_tree._content_filter_stack(relpath)
-            chunks = content.splitlines(True)
-            content = filtered_output_bytes(chunks, filters,
-                ContentFilterContext(relpath, rev_tree))
-            self.cleanup_now()
-            self.outf.writelines(content)
+            from bzrlib.filter_tree import ContentFilterTree
+            filter_tree = ContentFilterTree(rev_tree,
+                rev_tree._content_filter_stack)
+            content = filter_tree.get_file_text(actual_file_id)
         else:
-            self.cleanup_now()
-            self.outf.write(content)
+            content = rev_tree.get_file_text(actual_file_id)
+        self.cleanup_now()
+        self.outf.write(content)
 
 
 class cmd_local_time_offset(Command):

=== modified file 'bzrlib/bundle/__init__.py'
--- bzrlib/bundle/__init__.py	2011-01-26 19:34:58 +0000
+++ bzrlib/bundle/__init__.py	2011-07-23 16:33:38 +0000
@@ -54,7 +54,7 @@
                                        exclude_trailing_slash=False)
         if not filename:
             raise errors.NotABundle('A directory cannot be a bundle')
-        return _mod_transport.get_transport(url)
+        return _mod_transport.get_transport_from_url(url)
 
     try:
         bytef, transport = _mod_transport.do_catching_redirections(

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2011-05-31 21:08:52 +0000
+++ bzrlib/bzrdir.py	2011-08-08 16:28:55 +0000
@@ -2287,11 +2287,8 @@
     help='Same as 2a.')
 
 # The current format that is made on 'bzr init'.
-format_name = config.GlobalConfig().get_user_option('default_format')
-if format_name is None:
-    controldir.format_registry.set_default('2a')
-else:
-    controldir.format_registry.set_default(format_name)
+format_name = config.GlobalStack().get('default_format')
+controldir.format_registry.set_default(format_name)
 
 # XXX 2010-08-20 JRV: There is still a lot of code relying on
 # bzrlib.bzrdir.format_registry existing. When BzrDir.create/BzrDir.open/etc

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2011-06-27 15:36:58 +0000
+++ bzrlib/commands.py	2011-07-15 14:27:20 +0000
@@ -164,16 +164,6 @@
     return cmd[4:].replace('_','-')
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def _builtin_commands():
-    """Return a dict of {name: cmd_class} for builtin commands.
-
-    :deprecated: Use the builtin_command_registry registry instead
-    """
-    # return dict(name: cmd_class)
-    return dict(builtin_command_registry.items())
-
-
 def _register_builtin_commands():
     if builtin_command_registry.keys():
         # only load once
@@ -442,19 +432,6 @@
         """
         self._operation.cleanup_now()
 
-    @deprecated_method(deprecated_in((2, 1, 0)))
-    def _maybe_expand_globs(self, file_list):
-        """Glob expand file_list if the platform does not do that itself.
-
-        Not used anymore, now that the bzr command-line parser globs on
-        Windows.
-
-        :return: A possibly empty list of unicode paths.
-
-        Introduced in bzrlib 0.18.
-        """
-        return file_list
-
     def _usage(self):
         """Return single-line grammar for this command.
 
@@ -723,11 +700,6 @@
                 del self._operation
         self.run = run
 
-    @deprecated_method(deprecated_in((2, 2, 0)))
-    def run_direct(self, *args, **kwargs):
-        """Deprecated thunk from bzrlib 2.1."""
-        return self.run(*args, **kwargs)
-
     def run(self):
         """Actually run the command.
 
@@ -978,7 +950,8 @@
 
 def apply_lsprofiled(filename, the_callable, *args, **kwargs):
     from bzrlib.lsprof import profile
-    ret, stats = profile(exception_to_return_code, the_callable, *args, **kwargs)
+    ret, stats = profile(exception_to_return_code, the_callable,
+                         *args, **kwargs)
     stats.sort()
     if filename is None:
         stats.pprint()
@@ -988,11 +961,6 @@
     return ret
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def shlex_split_unicode(unsplit):
-    return cmdline.split(unsplit)
-
-
 def get_alias(cmd, config=None):
     """Return an expanded alias, or None if no alias exists.
 

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2011-07-25 07:51:35 +0000
+++ bzrlib/config.py	2011-08-12 09:49:24 +0000
@@ -31,6 +31,7 @@
 log_format=name-of-format
 validate_signatures_in_log=true|false(default)
 acceptable_keys=pattern1,pattern2
+gpg_signing_key=amy@example.com
 
 in locations.conf, you specify the url of a branch and options for it.
 Wildcards may be used - * and ? as normal in shell completion. Options
@@ -171,9 +172,7 @@
 # FIXME: Until we can guarantee that each config file is loaded once and
 # only once for a given bzrlib session, we don't want to re-read the file every
 # time we query for an option so we cache the value (bad ! watch out for tests
-# needing to restore the proper value).This shouldn't be part of 2.4.0 final,
-# yell at mgz^W vila and the RM if this is still present at that time
-# -- vila 20110219
+# needing to restore the proper value). -- vila 20110219
 _expand_default_value = None
 def _get_expand_default_value():
     global _expand_default_value
@@ -536,6 +535,14 @@
             return True
         return False
 
+    def gpg_signing_key(self):
+        """GPG user-id to sign commits"""
+        key = self.get_user_option('gpg_signing_key')
+        if key == "default" or key == None:
+            return self.user_email()
+        else:
+            return key
+
     def get_alias(self, value):
         return self._get_alias(value)
 
@@ -1504,14 +1511,16 @@
             raise errors.BzrError('You must have one of BZR_HOME, APPDATA,'
                                   ' or HOME set')
         return osutils.pathjoin(base, 'bazaar', '2.0')
-    elif sys.platform == 'darwin':
+    else:
+        if base is not None:
+            base = base.decode(osutils._fs_enc)
+    if sys.platform == 'darwin':
         if base is None:
             # this takes into account $HOME
             base = os.path.expanduser("~")
         return osutils.pathjoin(base, '.bazaar')
     else:
         if base is None:
-
             xdg_dir = os.environ.get('XDG_CONFIG_HOME', None)
             if xdg_dir is None:
                 xdg_dir = osutils.pathjoin(os.path.expanduser("~"), ".config")
@@ -1520,7 +1529,6 @@
                 trace.mutter(
                     "Using configuration in XDG directory %s." % xdg_dir)
                 return xdg_dir
-
             base = os.path.expanduser("~")
         return osutils.pathjoin(base, ".bazaar")
 
@@ -2265,31 +2273,90 @@
     value, in which config files it can be stored, etc (TBC).
     """
 
-    def __init__(self, name, default=None):
+    def __init__(self, name, default=None, help=None):
         self.name = name
         self.default = default
+        self.help = help
 
     def get_default(self):
         return self.default
 
 
-# Options registry
-
-option_registry = registry.Registry()
-
-
-option_registry.register(
-    'editor', Option('editor'),
-    help='The command called to launch an editor to enter a message.')
-
-option_registry.register(
-    'dirstate.fdatasync', Option('dirstate.fdatasync', default=True),
-    help='Flush dirstate changes onto physical disk?')
-
-option_registry.register(
-    'repository.fdatasync',
-    Option('repository.fdatasync', default=True),
-    help='Flush repository changes onto physical disk?')
+class OptionRegistry(registry.Registry):
+    """Register config options by their name.
+
+    This overrides ``registry.Registry`` to simplify registration by acquiring
+    some information from the option object itself.
+    """
+
+    def register(self, option):
+        """Register a new option to its name.
+
+        :param option: The option to register. Its name is used as the key.
+        """
+        super(OptionRegistry, self).register(option.name, option,
+                                             help=option.help)
+
+    def register_lazy(self, key, module_name, member_name):
+        """Register a new option to be loaded on request.
+
+        :param key: This is the key to use to request the option later. Since
+            the registration is lazy, it should be provided and match the
+            option name.
+
+        :param module_name: The python path to the module. Such as 'os.path'.
+
+        :param member_name: The member of the module to return.  If empty or
+                None, get() will return the module itself.
+        """
+        super(OptionRegistry, self).register_lazy(key,
+                                                  module_name, member_name)
+
+    def get_help(self, key=None):
+        """Get the help text associated with the given key"""
+        option = self.get(key)
+        the_help = option.help
+        if callable(the_help):
+            return the_help(self, key)
+        return the_help
+
+
+option_registry = OptionRegistry()
+
+
+# Registered options in lexicographical order
+
+option_registry.register(
+    Option('dirstate.fdatasync', default=True,
+           help='''
+Flush dirstate changes onto physical disk?
+
+If true (default), working tree metadata changes are flushed through the
+OS buffers to physical disk.  This is somewhat slower, but means data
+should not be lost if the machine crashes.  See also repository.fdatasync.
+'''))
+option_registry.register(
+    Option('default_format', default='2a',
+           help='Format used when creating branches.'))
+option_registry.register(
+    Option('editor',
+           help='The command called to launch an editor to enter a message.'))
+option_registry.register(
+    Option('language',
+           help='Language to translate messages into.'))
+option_registry.register(
+    Option('output_encoding',
+           help= 'Unicode encoding for output'
+           ' (terminal encoding if not specified).'))
+option_registry.register(
+    Option('repository.fdatasync', default=True,
+           help='''\
+Flush repository changes onto physical disk?
+
+If true (default), repository changes are flushed through the OS buffers
+to physical disk.  This is somewhat slower, but means data should not be
+lost if the machine crashes.  See also dirstate.fdatasync.
+'''))
 
 
 class Section(object):
@@ -2562,16 +2629,16 @@
 class GlobalStore(LockableIniFileStore):
 
     def __init__(self, possible_transports=None):
-        t = transport.get_transport(config_dir(),
-                                    possible_transports=possible_transports)
+        t = transport.get_transport_from_path(
+            config_dir(), possible_transports=possible_transports)
         super(GlobalStore, self).__init__(t, 'bazaar.conf')
 
 
 class LocationStore(LockableIniFileStore):
 
     def __init__(self, possible_transports=None):
-        t = transport.get_transport(config_dir(),
-                                    possible_transports=possible_transports)
+        t = transport.get_transport_from_path(
+            config_dir(), possible_transports=possible_transports)
         super(LocationStore, self).__init__(t, 'locations.conf')
 
 

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2011-05-19 10:51:37 +0000
+++ bzrlib/diff.py	2011-07-15 15:31:01 +0000
@@ -44,10 +44,6 @@
 from bzrlib.registry import (
     Registry,
     )
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    deprecated_in,
-    )
 from bzrlib.trace import mutter, note, warning
 
 
@@ -288,39 +284,6 @@
                         new_abspath, e)
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def get_trees_and_branches_to_diff(path_list, revision_specs, old_url, new_url,
-                                   apply_view=True):
-    """Get the trees and specific files to diff given a list of paths.
-
-    This method works out the trees to be diff'ed and the files of
-    interest within those trees.
-
-    :param path_list:
-        the list of arguments passed to the diff command
-    :param revision_specs:
-        Zero, one or two RevisionSpecs from the diff command line,
-        saying what revisions to compare.
-    :param old_url:
-        The url of the old branch or tree. If None, the tree to use is
-        taken from the first path, if any, or the current working tree.
-    :param new_url:
-        The url of the new branch or tree. If None, the tree to use is
-        taken from the first path, if any, or the current working tree.
-    :param apply_view:
-        if True and a view is set, apply the view or check that the paths
-        are within it
-    :returns:
-        a tuple of (old_tree, new_tree, old_branch, new_branch,
-        specific_files, extra_trees) where extra_trees is a sequence of
-        additional trees to search in for file-ids.  The trees and branches
-        are not locked.
-    """
-    op = cleanup.OperationWithCleanups(get_trees_and_branches_to_diff_locked)
-    return op.run_simple(path_list, revision_specs, old_url, new_url,
-            op.add_cleanup, apply_view=apply_view)
-    
-
 def get_trees_and_branches_to_diff_locked(
     path_list, revision_specs, old_url, new_url, add_cleanup, apply_view=True):
     """Get the trees and specific files to diff given a list of paths.
@@ -436,7 +399,8 @@
     extra_trees = None
     if working_tree is not None and working_tree not in (old_tree, new_tree):
         extra_trees = (working_tree,)
-    return old_tree, new_tree, old_branch, new_branch, specific_files, extra_trees
+    return (old_tree, new_tree, old_branch, new_branch,
+            specific_files, extra_trees)
 
 
 def _get_tree_to_diff(spec, tree=None, branch=None, basis_is_default=True):

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2011-08-01 06:24:48 +0000
+++ bzrlib/errors.py	2011-08-02 01:10:27 +0000
@@ -621,7 +621,7 @@
 
     _fmt = 'Unsupported protocol for url "%(path)s"%(extra)s'
 
-    def __init__(self, url, extra):
+    def __init__(self, url, extra=""):
         PathError.__init__(self, url, extra=extra)
 
 

=== modified file 'bzrlib/export/__init__.py'
--- bzrlib/export/__init__.py	2011-06-13 16:34:53 +0000
+++ bzrlib/export/__init__.py	2011-07-08 23:38:46 +0000
@@ -19,6 +19,8 @@
 
 import os
 import time
+import warnings
+
 from bzrlib import (
     errors,
     pyutils,
@@ -58,10 +60,10 @@
 
     When requesting a specific type of export, load the respective path.
     """
-    def _loader(tree, dest, root, subdir, filtered, force_mtime, fileobj):
+    def _loader(tree, dest, root, subdir, force_mtime, fileobj):
         func = pyutils.get_named_object(module, funcname)
-        return func(tree, dest, root, subdir, filtered=filtered,
-                    force_mtime=force_mtime, fileobj=fileobj)
+        return func(tree, dest, root, subdir, force_mtime=force_mtime,
+            fileobj=fileobj)
 
     register_exporter(scheme, extensions, _loader)
 
@@ -91,7 +93,8 @@
         a directory to start exporting from.
 
     :param filtered: If True, content filtering is applied to the exported
-        files.
+        files.  Deprecated in favour of passing a ContentFilterTree
+        as the source.
 
     :param per_file_timestamps: Whether to use the timestamp stored in the tree
         rather than now(). This will do a revision lookup for every file so
@@ -122,13 +125,20 @@
 
     trace.mutter('export version %r', tree)
 
+    if filtered:
+        from bzrlib.filter_tree import ContentFilterTree
+        warnings.warn(
+            "passing filtered=True to export is deprecated in bzr 2.4",
+            stacklevel=2)
+        tree = ContentFilterTree(tree, tree._content_filter_stack)
+        # We don't want things re-filtered by the specific exporter.
+        filtered = False
+
+    tree.lock_read()
     try:
-        tree.lock_read()
-
-        for _ in _exporters[format](tree, dest, root, subdir,
-                                    filtered=filtered,
-                                    force_mtime=force_mtime, fileobj=fileobj):
-
+        for _ in _exporters[format](
+            tree, dest, root, subdir,
+            force_mtime=force_mtime, fileobj=fileobj):
             yield
     finally:
         tree.unlock()
@@ -153,7 +163,7 @@
         the entire tree, and anything else should specify the relative path to
         a directory to start exporting from.
     :param filtered: If True, content filtering is applied to the
-                     files exported.
+        files exported.  Deprecated in favor of passing an ContentFilterTree.
     :param per_file_timestamps: Whether to use the timestamp stored in the
         tree rather than now(). This will do a revision lookup
         for every file so will be significantly slower.

=== modified file 'bzrlib/export/dir_exporter.py'
--- bzrlib/export/dir_exporter.py	2011-06-28 13:55:39 +0000
+++ bzrlib/export/dir_exporter.py	2011-07-04 21:55:35 +0000
@@ -21,13 +21,9 @@
 
 from bzrlib import errors, osutils
 from bzrlib.export import _export_iter_entries
-from bzrlib.filters import (
-    ContentFilterContext,
-    filtered_output_bytes,
-    )
-
-
-def dir_exporter_generator(tree, dest, root, subdir=None, filtered=False,
+
+
+def dir_exporter_generator(tree, dest, root, subdir=None,
                            force_mtime=None, fileobj=None):
     """Return a generator that exports this tree to a new directory.
 
@@ -79,10 +75,6 @@
     # the directories
     flags = os.O_CREAT | os.O_TRUNC | os.O_WRONLY | getattr(os, 'O_BINARY', 0)
     for (relpath, executable), chunks in tree.iter_files_bytes(to_fetch):
-        if filtered:
-            filters = tree._content_filter_stack(relpath)
-            context = ContentFilterContext(relpath, tree, ie)
-            chunks = filtered_output_bytes(chunks, filters, context)
         fullpath = osutils.pathjoin(dest, relpath)
         # We set the mode and let the umask sort out the file info
         mode = 0666

=== modified file 'bzrlib/export/tar_exporter.py'
--- bzrlib/export/tar_exporter.py	2011-06-28 13:54:56 +0000
+++ bzrlib/export/tar_exporter.py	2011-07-21 07:08:05 +0000
@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Export a Tree to a non-versioned directory."""
+"""Export a tree to a tarball."""
 
 import os
 import StringIO
@@ -26,14 +26,9 @@
     osutils,
     )
 from bzrlib.export import _export_iter_entries
-from bzrlib.filters import (
-    ContentFilterContext,
-    filtered_output_bytes,
-    )
-
-
-def prepare_tarball_item(tree, root, final_path, entry, filtered=False,
-                         force_mtime=None):
+
+
+def prepare_tarball_item(tree, root, final_path, entry, force_mtime=None):
     """Prepare a tarball item for exporting
 
     :param tree: Tree to export
@@ -42,8 +37,6 @@
 
     :param entry: Entry to export
 
-    :param filtered: Whether to apply filters
-
     :param force_mtime: Option mtime to force, instead of using tree
         timestamps.
 
@@ -61,17 +54,13 @@
             item.mode = 0755
         else:
             item.mode = 0644
-        if filtered:
-            chunks = tree.get_file_lines(entry.file_id)
-            filters = tree._content_filter_stack(final_path)
-            context = ContentFilterContext(final_path, tree, entry)
-            contents = filtered_output_bytes(chunks, filters, context)
-            content = ''.join(contents)
-            item.size = len(content)
-            fileobj = StringIO.StringIO(content)
-        else:
-            item.size = tree.get_file_size(entry.file_id)
-            fileobj = tree.get_file(entry.file_id)
+        # This brings the whole file into memory, but that's almost needed for
+        # the tarfile contract, which wants the size of the file up front.  We
+        # want to make sure it doesn't change, and we need to read it in one
+        # go for content filtering.
+        content = tree.get_file_text(entry.file_id)
+        item.size = len(content)
+        fileobj = StringIO.StringIO(content)
     elif entry.kind == "directory":
         item.type = tarfile.DIRTYPE
         item.name += '/'
@@ -90,8 +79,7 @@
     return (item, fileobj)
 
 
-def export_tarball_generator(tree, ball, root, subdir=None, filtered=False,
-                   force_mtime=None):
+def export_tarball_generator(tree, ball, root, subdir=None, force_mtime=None):
     """Export tree contents to a tarball.
 
     :returns: A generator that will repeatedly produce None as each file is
@@ -103,8 +91,6 @@
     :param ball: Tarball to export to; it will be closed when writing is
         complete.
 
-    :param filtered: Whether to apply filters
-
     :param subdir: Sub directory to export
 
     :param force_mtime: Option mtime to force, instead of using tree
@@ -113,15 +99,15 @@
     try:
         for final_path, entry in _export_iter_entries(tree, subdir):
             (item, fileobj) = prepare_tarball_item(
-                tree, root, final_path, entry, filtered, force_mtime)
+                tree, root, final_path, entry, force_mtime)
             ball.addfile(item, fileobj)
             yield
     finally:
         ball.close()
 
 
-def tgz_exporter_generator(tree, dest, root, subdir, filtered=False,
-                           force_mtime=None, fileobj=None):
+def tgz_exporter_generator(tree, dest, root, subdir, force_mtime=None,
+    fileobj=None):
     """Export this tree to a new tar file.
 
     `dest` will be created holding the contents of this tree; if it
@@ -141,10 +127,10 @@
         root_mtime = None
 
     is_stdout = False
+    basename = None
     if fileobj is not None:
         stream = fileobj
     elif dest == '-':
-        basename = None
         stream = sys.stdout
         is_stdout = True
     else:
@@ -161,7 +147,7 @@
         zipstream = gzip.GzipFile(basename, 'w', fileobj=stream)
     ball = tarfile.open(None, 'w|', fileobj=zipstream)
     for _ in export_tarball_generator(
-        tree, ball, root, subdir, filtered, force_mtime):
+        tree, ball, root, subdir, force_mtime):
         yield
     # Closing zipstream may trigger writes to stream
     zipstream.close()
@@ -170,7 +156,7 @@
         stream.close()
 
 
-def tbz_exporter_generator(tree, dest, root, subdir, filtered=False,
+def tbz_exporter_generator(tree, dest, root, subdir,
                            force_mtime=None, fileobj=None):
     """Export this tree to a new tar file.
 
@@ -189,12 +175,11 @@
         # Python 2.6.5 and 2.7b1)
         ball = tarfile.open(dest.encode(osutils._fs_enc), 'w:bz2')
     return export_tarball_generator(
-        tree, ball, root, subdir, filtered, force_mtime)
+        tree, ball, root, subdir, force_mtime)
 
 
 def plain_tar_exporter_generator(tree, dest, root, subdir, compression=None,
-                                 filtered=False, force_mtime=None,
-                                 fileobj=None):
+    force_mtime=None, fileobj=None):
     """Export this tree to a new tar file.
 
     `dest` will be created holding the contents of this tree; if it
@@ -208,16 +193,16 @@
         stream = open(dest, 'wb')
     ball = tarfile.open(None, 'w|', stream)
     return export_tarball_generator(
-        tree, ball, root, subdir, filtered, force_mtime)
-
-
-def tar_xz_exporter_generator(tree, dest, root, subdir, filtered=False,
+        tree, ball, root, subdir, force_mtime)
+
+
+def tar_xz_exporter_generator(tree, dest, root, subdir,
                               force_mtime=None, fileobj=None):
-    return tar_lzma_exporter_generator(tree, dest, root, subdir, filtered,
+    return tar_lzma_exporter_generator(tree, dest, root, subdir,
                                        force_mtime, fileobj, "xz")
 
 
-def tar_lzma_exporter_generator(tree, dest, root, subdir, filtered=False,
+def tar_lzma_exporter_generator(tree, dest, root, subdir,
                       force_mtime=None, fileobj=None,
                                 compression_format="alone"):
     """Export this tree to a new .tar.lzma file.
@@ -240,4 +225,4 @@
         options={"format": compression_format})
     ball = tarfile.open(None, 'w:', fileobj=stream)
     return export_tarball_generator(
-        tree, ball, root, subdir, filtered=filtered, force_mtime=force_mtime)
+        tree, ball, root, subdir, force_mtime=force_mtime)

=== modified file 'bzrlib/export/zip_exporter.py'
--- bzrlib/export/zip_exporter.py	2011-06-13 16:34:53 +0000
+++ bzrlib/export/zip_exporter.py	2011-07-04 21:55:35 +0000
@@ -27,10 +27,6 @@
     osutils,
     )
 from bzrlib.export import _export_iter_entries
-from bzrlib.filters import (
-    ContentFilterContext,
-    filtered_output_bytes,
-    )
 from bzrlib.trace import mutter
 
 
@@ -44,7 +40,7 @@
 _DIR_ATTR = stat.S_IFDIR | ZIP_DIRECTORY_BIT | DIR_PERMISSIONS
 
 
-def zip_exporter_generator(tree, dest, root, subdir=None, filtered=False,
+def zip_exporter_generator(tree, dest, root, subdir=None,
     force_mtime=None, fileobj=None):
     """ Export this tree to a new zip file.
 
@@ -77,14 +73,7 @@
                             date_time=date_time)
                 zinfo.compress_type = compression
                 zinfo.external_attr = _FILE_ATTR
-                if filtered:
-                    chunks = tree.get_file_lines(file_id)
-                    filters = tree._content_filter_stack(dp)
-                    context = ContentFilterContext(dp, tree, ie)
-                    contents = filtered_output_bytes(chunks, filters, context)
-                    content = ''.join(contents)
-                else:
-                    content = tree.get_file_text(file_id)
+                content = tree.get_file_text(file_id)
                 zipf.writestr(zinfo, content)
             elif ie.kind == "directory":
                 # Directories must contain a trailing slash, to indicate

=== added file 'bzrlib/filter_tree.py'
--- bzrlib/filter_tree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/filter_tree.py	2011-07-04 21:10:37 +0000
@@ -0,0 +1,82 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Content-filtered view of any tree.
+"""
+
+
+from bzrlib import (
+    tree,
+    )
+from bzrlib.filters import (
+    ContentFilter,
+    ContentFilterContext,
+    filtered_input_file,
+    filtered_output_bytes,
+    _get_filter_stack_for,
+    _get_registered_names,
+    internal_size_sha_file_byname,
+    register_filter_stack_map,
+    )
+
+
+class ContentFilterTree(tree.Tree):
+    """A virtual tree that applies content filters to an underlying tree.
+    
+    Not every operation is supported yet.
+    """
+
+    def __init__(self, backing_tree, filter_stack_callback):
+        """Construct a new filtered tree view.
+
+        :param filter_stack_callback: A callable taking a path that returns
+            the filter stack that should be used for that path.
+        :param backing_tree: An underlying tree to wrap.
+        """
+        self.backing_tree = backing_tree
+        self.filter_stack_callback = filter_stack_callback
+
+    def get_file_text(self, file_id, path=None):
+        chunks = self.backing_tree.get_file_lines(file_id, path)
+        filters = self.filter_stack_callback(path)
+        if path is None:
+            path = self.backing_tree.id2path(file_id)
+        context = ContentFilterContext(path, self, None)
+        contents = filtered_output_bytes(chunks, filters, context)
+        content = ''.join(contents)
+        return content
+
+    def has_filename(self, filename):
+        return self.backing_tree.has_filename
+
+    def is_executable(self, file_id, path=None):
+        return self.backing_tree.is_executable(file_id, path)
+
+    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=None):
+        # NB: This simply returns the parent tree's entries; the length may be
+        # wrong but it can't easily be calculated without filtering the whole
+        # text.  Currently all callers cope with this; perhaps they should be
+        # updated to a narrower interface that only provides things guaranteed
+        # cheaply available across all trees. -- mbp 20110705
+        return self.backing_tree.iter_entries_by_dir(
+            specific_file_ids=specific_file_ids,
+            yield_parents=yield_parents)
+
+    def lock_read(self):
+        return self.backing_tree.lock_read()
+
+    def unlock(self):
+        return self.backing_tree.unlock()

=== modified file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	2011-07-01 10:42:26 +0000
+++ bzrlib/gpg.py	2011-07-11 10:53:46 +0000
@@ -175,7 +175,9 @@
             return False
 
     def _command_line(self):
-        return [self._config.gpg_signing_command(), '--clearsign']
+        
+        return [self._config.gpg_signing_command(), '--clearsign', '-u',
+                                                self._config.gpg_signing_key()]
 
     def __init__(self, config):
         self._config = config

=== modified file 'bzrlib/help_topics/__init__.py'
--- bzrlib/help_topics/__init__.py	2011-05-28 13:46:51 +0000
+++ bzrlib/help_topics/__init__.py	2011-06-30 16:47:06 +0000
@@ -270,8 +270,8 @@
 
 would refer to ``/home/remote/myproject/trunk``.
 
-Many commands that accept URLs also accept location aliases too.  See
-::doc:`location-alias-help`.
+Many commands that accept URLs also accept location aliases too.
+See :doc:`location-alias-help` and :doc:`url-special-chars-help`.
 """
 
     return out
@@ -778,6 +778,8 @@
                         'Aliases for remembered locations')
 topic_registry.register('log-formats', _load_from_file,
                         'Details on the logging formats available')
+topic_registry.register('url-special-chars', _load_from_file,
+                        'Special character handling in URLs')
 
 
 # Register concept topics.
@@ -898,6 +900,6 @@
         elif line.endswith('::'):
             line = line[:-1]
         # Map :doc:`xxx-help` to ``bzr help xxx``
-        line = re.sub(":doc:`(.+)-help`", r'``bzr help \1``', line)
+        line = re.sub(":doc:`(.+?)-help`", r'``bzr help \1``', line)
         result.append(line)
     return "\n".join(result) + "\n"

=== modified file 'bzrlib/help_topics/en/configuration.txt'
--- bzrlib/help_topics/en/configuration.txt	2011-07-25 09:14:31 +0000
+++ bzrlib/help_topics/en/configuration.txt	2011-08-02 01:10:27 +0000
@@ -258,7 +258,7 @@
 variable name, an equals sign and a value.  For example::
 
     email            = John Doe <jdoe@isp.com>
-    check_signatures = require
+    gpg_signing_key  = Amy Pond <amy@example.com>
 
 A variable can reference other variables **in the same configuration file** by
 enclosing them in curly brackets::
@@ -311,7 +311,6 @@
     [DEFAULT]
     email             = John Doe <jdoe@isp.com>
     editor            = /usr/bin/vim
-    check_signatures  = check-available
     create_signatures = when-required
 
 
@@ -329,10 +328,6 @@
 
     [http://hypothetical.site.com/branches/devel-branch]
     create_signatures = always
-    check_signatures  = always
-
-    [http://example.com/bzr/*]
-    check_signatures  = require
 
 The authentication configuration file, authentication.conf
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -427,6 +422,13 @@
 OS buffers to physical disk.  This is somewhat slower, but means data
 should not be lost if the machine crashes.  See also repository.fdatasync.
 
+gpg_signing_key
+~~~~~~~~~~~
+
+The GnuPG user identity to use when signing commits.  Can be an e-mail
+address, key fingerprint or full key ID.  When unset or when set to
+"default" Bazaar will use the user e-mail set with ``whoami``.
+
 recurse
 ~~~~~~~
 
@@ -448,6 +450,8 @@
 
     gpg_signing_command = /usr/bin/gnpg
 
+The specified command must accept the options "--clearsign" and "-u <email>".
+
 bzr_remote_path
 ~~~~~~~~~~~~~~~
 

=== added file 'bzrlib/help_topics/en/url-special-chars.txt'
--- bzrlib/help_topics/en/url-special-chars.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/help_topics/en/url-special-chars.txt	2011-07-23 11:31:21 +0000
@@ -0,0 +1,48 @@
+Special character handling in URLs
+==================================
+
+Bazaar allows locations to be specified in multiple ways, either:
+
+ * Fully qualified URLs
+
+ * File system paths, relative or absolute
+
+Internally bzr treats all locations as URLs. For any file system paths
+that are specified it will automatically determine the appropriate URL
+representation, and escape special characters where necessary.
+
+There are a few characters which have special meaning in URLs and need careful
+handling to avoid ambiguities. Characters can be escaped with a % and a hex
+value in URLs. Any non-ASCII characters in a file path will automatically be
+urlencoded when the path is converted to a URL.
+
+URLs represent non-ASCII characters in an encoding defined by the server, but
+usually UTF-8.  The % escapes should be of the UTF-8 bytes.  Bazaar tries to be
+generous in what it accepts as a URL and to print them in a way that
+will be readable.
+
+For example, if you have a directory named '/tmp/%2False' these are all valid
+ways of accessing the content (0x2F, or 47, is the ASCII code for forward slash)::
+
+  cd /tmp
+  bzr log /tmp/%2False
+  bzr log %2False
+  bzr log file:///tmp/%252False
+  bzr log file://localhost/tmp/%252False
+  bzr log file:%252False
+
+These are valid but do not refer to the same file::
+
+  bzr log file:///tmp/%2False (refers to a file called /tmp/\/alse)
+  bzr log %252False (refers to a file called /tmp/%252False)
+
+Comma also has special meaning in URLs, because it denotes `segment parameters`_
+
+_`segment parameters`: http://www.ietf.org/rfc/rfc3986.txt (section 3.3)
+
+Comma is also special in any file system paths that are specified. To use a literal
+comma in a file system path, specify a URL and URL encode the comma::
+
+  bzr log foo,branch=bla # path "foo" with the segment parameter "branch" set to "bla"
+  bzr log file:foo%2Cbranch=bla # path "foo,branch=bla"
+  bzr log file:foo,branch=bla # path "foo" with segment parameter "branch" set to "bla"

=== modified file 'bzrlib/i18n.py'
--- bzrlib/i18n.py	2011-06-30 11:14:06 +0000
+++ bzrlib/i18n.py	2011-08-08 17:03:51 +0000
@@ -74,10 +74,14 @@
     global _translations
     if lang is None:
         lang = _get_current_locale()
+    if lang is not None:
+        languages = lang.split(':')
+    else:
+        languages = None
     _translations = _gettext.translation(
             'bzr',
             localedir=_get_locale_dir(),
-            languages=lang.split(':'),
+            languages=languages,
             fallback=True)
 
 
@@ -130,7 +134,7 @@
 def _get_current_locale():
     if not os.environ.get('LANGUAGE'):
         from bzrlib import config
-        lang = config.GlobalConfig().get_user_option('language')
+        lang = config.GlobalStack().get('language')
         if lang:
             os.environ['LANGUAGE'] = lang
             return lang

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2011-06-30 16:48:11 +0000
+++ bzrlib/log.py	2011-07-25 07:41:52 +0000
@@ -89,10 +89,6 @@
     get_terminal_encoding,
     terminal_width,
     )
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    deprecated_in,
-    )
 
 
 def find_touching_revisions(branch, file_id):
@@ -157,7 +153,8 @@
              end_revision=None,
              search=None,
              limit=None,
-             show_diff=False):
+             show_diff=False,
+             match=None):
     """Write out human-readable log of commits to this branch.
 
     This function is being retained for backwards compatibility but
@@ -186,6 +183,9 @@
         if None or 0.
 
     :param show_diff: If True, output a diff after each revision.
+
+    :param match: Dictionary of search lists to use when matching revision
+      properties.
     """
     # Convert old-style parameters to new-style parameters
     if specific_fileid is not None:
@@ -219,7 +219,7 @@
 # make_log_request_dict() below
 _DEFAULT_REQUEST_PARAMS = {
     'direction': 'reverse',
-    'levels': 1,
+    'levels': None,
     'generate_tags': True,
     'exclude_common_ancestry': False,
     '_match_using_deltas': True,
@@ -228,10 +228,10 @@
 
 def make_log_request_dict(direction='reverse', specific_fileids=None,
                           start_revision=None, end_revision=None, limit=None,
-                          message_search=None, levels=1, generate_tags=True,
+                          message_search=None, levels=None, generate_tags=True,
                           delta_type=None,
                           diff_type=None, _match_using_deltas=True,
-                          exclude_common_ancestry=False,
+                          exclude_common_ancestry=False, match=None,
                           signature=False,
                           ):
     """Convenience function for making a logging request dictionary.
@@ -259,7 +259,8 @@
       matching commit messages
 
     :param levels: the number of levels of revisions to
-      generate; 1 for just the mainline; 0 for all levels.
+      generate; 1 for just the mainline; 0 for all levels, or None for
+      a sensible default.
 
     :param generate_tags: If True, include tags for matched revisions.
 `
@@ -282,20 +283,34 @@
       range operator or as a graph difference.
 
     :param signature: show digital signature information
+
+    :param match: Dictionary of list of search strings to use when filtering
+      revisions. Keys can be 'message', 'author', 'committer', 'bugs' or
+      the empty string to match any of the preceding properties.
+
     """
+    # Take care of old style message_search parameter
+    if message_search:
+        if match:
+            if 'message' in match:
+                match['message'].append(message_search)
+            else:
+                match['message'] = [message_search]
+        else:
+            match={ 'message': [message_search] }
     return {
         'direction': direction,
         'specific_fileids': specific_fileids,
         'start_revision': start_revision,
         'end_revision': end_revision,
         'limit': limit,
-        'message_search': message_search,
         'levels': levels,
         'generate_tags': generate_tags,
         'delta_type': delta_type,
         'diff_type': diff_type,
         'exclude_common_ancestry': exclude_common_ancestry,
         'signature': signature,
+        'match': match,
         # Add 'private' attributes for features that may be deprecated
         '_match_using_deltas': _match_using_deltas,
     }
@@ -311,7 +326,7 @@
 
 def format_signature_validity(rev_id, repo):
     """get the signature validity
-    
+
     :param rev_id: revision id to validate
     :param repo: repository of revision
     :return: human readable string to print to log
@@ -380,7 +395,10 @@
         # Tweak the LogRequest based on what the LogFormatter can handle.
         # (There's no point generating stuff if the formatter can't display it.)
         rqst = self.rqst
-        rqst['levels'] = lf.get_levels()
+        if rqst['levels'] is None or lf.get_levels() > rqst['levels']:
+            # user didn't specify levels, use whatever the LF can handle:
+            rqst['levels'] = lf.get_levels()
+
         if not getattr(lf, 'supports_tags', False):
             rqst['generate_tags'] = False
         if not getattr(lf, 'supports_delta', False):
@@ -398,7 +416,7 @@
 
     def _generator_factory(self, branch, rqst):
         """Make the LogGenerator object to use.
-        
+
         Subclasses may wish to override this.
         """
         return _DefaultLogGenerator(branch, rqst)
@@ -507,7 +525,7 @@
 
         # Apply the other filters
         return make_log_rev_iterator(self.branch, view_revisions,
-            rqst.get('delta_type'), rqst.get('message_search'),
+            rqst.get('delta_type'), rqst.get('match'),
             file_ids=rqst.get('specific_fileids'),
             direction=rqst.get('direction'))
 
@@ -526,7 +544,7 @@
             rqst.get('specific_fileids')[0], view_revisions,
             include_merges=rqst.get('levels') != 1)
         return make_log_rev_iterator(self.branch, view_revisions,
-            rqst.get('delta_type'), rqst.get('message_search'))
+            rqst.get('delta_type'), rqst.get('match'))
 
 
 def _calc_view_revisions(branch, start_rev_id, end_rev_id, direction,
@@ -783,25 +801,6 @@
             yield rev_id, '.'.join(map(str, revno)), merge_depth
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def calculate_view_revisions(branch, start_revision, end_revision, direction,
-        specific_fileid, generate_merge_revisions):
-    """Calculate the revisions to view.
-
-    :return: An iterator of (revision_id, dotted_revno, merge_depth) tuples OR
-             a list of the same tuples.
-    """
-    start_rev_id, end_rev_id = _get_revision_limits(branch, start_revision,
-        end_revision)
-    view_revisions = list(_calc_view_revisions(branch, start_rev_id, end_rev_id,
-        direction, generate_merge_revisions or specific_fileid))
-    if specific_fileid:
-        view_revisions = _filter_revisions_touching_file_id(branch,
-            specific_fileid, view_revisions,
-            include_merges=generate_merge_revisions)
-    return _rebase_merge_depth(view_revisions)
-
-
 def _rebase_merge_depth(view_revisions):
     """Adjust depths upwards so the top level is 0."""
     # If either the first or last revision have a merge_depth of 0, we're done
@@ -851,31 +850,49 @@
     return log_rev_iterator
 
 
-def _make_search_filter(branch, generate_delta, search, log_rev_iterator):
+def _make_search_filter(branch, generate_delta, match, log_rev_iterator):
     """Create a filtered iterator of log_rev_iterator matching on a regex.
 
     :param branch: The branch being logged.
     :param generate_delta: Whether to generate a delta for each revision.
-    :param search: A user text search string.
+    :param match: A dictionary with properties as keys and lists of strings
+        as values. To match, a revision may match any of the supplied strings
+        within a single property but must match at least one string for each
+        property.
     :param log_rev_iterator: An input iterator containing all revisions that
         could be displayed, in lists.
     :return: An iterator over lists of ((rev_id, revno, merge_depth), rev,
         delta).
     """
-    if search is None:
+    if match is None:
         return log_rev_iterator
-    searchRE = lazy_regex.lazy_compile(search, re.IGNORECASE)
-    return _filter_message_re(searchRE, log_rev_iterator)
-
-
-def _filter_message_re(searchRE, log_rev_iterator):
+    searchRE = [(k, [re.compile(x, re.IGNORECASE) for x in v])
+                for (k,v) in match.iteritems()]
+    return _filter_re(searchRE, log_rev_iterator)
+
+
+def _filter_re(searchRE, log_rev_iterator):
     for revs in log_rev_iterator:
-        new_revs = []
-        for (rev_id, revno, merge_depth), rev, delta in revs:
-            if searchRE.search(rev.message):
-                new_revs.append(((rev_id, revno, merge_depth), rev, delta))
-        yield new_revs
-
+        new_revs = [rev for rev in revs if _match_filter(searchRE, rev[1])]
+        if new_revs:
+            yield new_revs
+
+def _match_filter(searchRE, rev):
+    strings = {
+               'message': (rev.message,),
+               'committer': (rev.committer,),
+               'author': (rev.get_apparent_authors()),
+               'bugs': list(rev.iter_bugs())
+               }
+    strings[''] = [item for inner_list in strings.itervalues()
+                   for item in inner_list]
+    for (k,v) in searchRE:
+        if k in strings and not _match_any_filter(strings[k], v):
+            return False
+    return True
+
+def _match_any_filter(strings, res):
+    return any([filter(None, map(re.search, strings)) for re in res])
 
 def _make_delta_filter(branch, generate_delta, search, log_rev_iterator,
     fileids=None, direction='reverse'):
@@ -954,7 +971,7 @@
 
 def _update_fileids(delta, fileids, stop_on):
     """Update the set of file-ids to search based on file lifecycle events.
-    
+
     :param fileids: a set of fileids to update
     :param stop_on: either 'add' or 'remove' - take file-ids out of the
       fileids set once their add or remove entry is detected respectively
@@ -1001,7 +1018,6 @@
     :return: An iterator over lists of ((rev_id, revno, merge_depth), rev,
         delta).
     """
-    repository = branch.repository
     num = 9
     for batch in log_rev_iterator:
         batch = iter(batch)
@@ -1146,50 +1162,6 @@
     return mainline_revs, rev_nos, start_rev_id, end_rev_id
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def _filter_revision_range(view_revisions, start_rev_id, end_rev_id):
-    """Filter view_revisions based on revision ranges.
-
-    :param view_revisions: A list of (revision_id, dotted_revno, merge_depth)
-            tuples to be filtered.
-
-    :param start_rev_id: If not NONE specifies the first revision to be logged.
-            If NONE then all revisions up to the end_rev_id are logged.
-
-    :param end_rev_id: If not NONE specifies the last revision to be logged.
-            If NONE then all revisions up to the end of the log are logged.
-
-    :return: The filtered view_revisions.
-    """
-    if start_rev_id or end_rev_id:
-        revision_ids = [r for r, n, d in view_revisions]
-        if start_rev_id:
-            start_index = revision_ids.index(start_rev_id)
-        else:
-            start_index = 0
-        if start_rev_id == end_rev_id:
-            end_index = start_index
-        else:
-            if end_rev_id:
-                end_index = revision_ids.index(end_rev_id)
-            else:
-                end_index = len(view_revisions) - 1
-        # To include the revisions merged into the last revision,
-        # extend end_rev_id down to, but not including, the next rev
-        # with the same or lesser merge_depth
-        end_merge_depth = view_revisions[end_index][2]
-        try:
-            for index in xrange(end_index+1, len(view_revisions)+1):
-                if view_revisions[index][2] <= end_merge_depth:
-                    end_index = index - 1
-                    break
-        except IndexError:
-            # if the search falls off the end then log to the end as well
-            end_index = len(view_revisions) - 1
-        view_revisions = view_revisions[start_index:end_index+1]
-    return view_revisions
-
-
 def _filter_revisions_touching_file_id(branch, file_id, view_revisions,
     include_merges=True):
     r"""Return the list of revision ids which touch a given file id.
@@ -1274,47 +1246,6 @@
     return result
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def get_view_revisions(mainline_revs, rev_nos, branch, direction,
-                       include_merges=True):
-    """Produce an iterator of revisions to show
-    :return: an iterator of (revision_id, revno, merge_depth)
-    (if there is no revno for a revision, None is supplied)
-    """
-    if not include_merges:
-        revision_ids = mainline_revs[1:]
-        if direction == 'reverse':
-            revision_ids.reverse()
-        for revision_id in revision_ids:
-            yield revision_id, str(rev_nos[revision_id]), 0
-        return
-    graph = branch.repository.get_graph()
-    # This asks for all mainline revisions, which means we only have to spider
-    # sideways, rather than depth history. That said, its still size-of-history
-    # and should be addressed.
-    # mainline_revisions always includes an extra revision at the beginning, so
-    # don't request it.
-    parent_map = dict(((key, value) for key, value in
-        graph.iter_ancestry(mainline_revs[1:]) if value is not None))
-    # filter out ghosts; merge_sort errors on ghosts.
-    rev_graph = _mod_repository._strip_NULL_ghosts(parent_map)
-    merge_sorted_revisions = tsort.merge_sort(
-        rev_graph,
-        mainline_revs[-1],
-        mainline_revs,
-        generate_revno=True)
-
-    if direction == 'forward':
-        # forward means oldest first.
-        merge_sorted_revisions = reverse_by_depth(merge_sorted_revisions)
-    elif direction != 'reverse':
-        raise ValueError('invalid direction %r' % direction)
-
-    for (sequence, rev_id, merge_depth, revno, end_of_merge
-         ) in merge_sorted_revisions:
-        yield rev_id, '.'.join(map(str, revno)), merge_depth
-
-
 def reverse_by_depth(merge_sorted_revisions, _depth=0):
     """Reverse revisions by depth.
 
@@ -1417,7 +1348,7 @@
         """Create a LogFormatter.
 
         :param to_file: the file to output to
-        :param to_exact_file: if set, gives an output stream to which 
+        :param to_exact_file: if set, gives an output stream to which
              non-Unicode diffs are written.
         :param show_ids: if True, revision-ids are to be displayed
         :param show_timezone: the timezone to use
@@ -1659,7 +1590,7 @@
         if revision.delta is not None:
             # Use the standard status output to display changes
             from bzrlib.delta import report_delta
-            report_delta(to_file, revision.delta, short_status=False, 
+            report_delta(to_file, revision.delta, short_status=False,
                          show_ids=self.show_ids, indent=indent)
         if revision.diff is not None:
             to_file.write(indent + 'diff:\n')
@@ -1731,8 +1662,8 @@
         if revision.delta is not None:
             # Use the standard status output to display changes
             from bzrlib.delta import report_delta
-            report_delta(to_file, revision.delta, 
-                         short_status=self.delta_format==1, 
+            report_delta(to_file, revision.delta,
+                         short_status=self.delta_format==1,
                          show_ids=self.show_ids, indent=indent + offset)
         if revision.diff is not None:
             self.show_diff(self.to_exact_file, revision.diff, '      ')
@@ -1918,12 +1849,6 @@
                               'The committer')
 
 
-def show_one_log(revno, rev, delta, verbose, to_file, show_timezone):
-    # deprecated; for compatibility
-    lf = LongLogFormatter(to_file=to_file, show_timezone=show_timezone)
-    lf.show(revno, rev, delta)
-
-
 def show_changed_revisions(branch, old_rh, new_rh, to_file=None,
                            log_format='long'):
     """Show the change in revision history comparing the old revision history to the new one.

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2011-06-09 11:52:35 +0000
+++ bzrlib/msgeditor.py	2011-07-23 16:33:38 +0000
@@ -144,7 +144,7 @@
         if not msgfilename:
             return None
         basename = osutils.basename(msgfilename)
-        msg_transport = transport.get_transport(osutils.dirname(msgfilename))
+        msg_transport = transport.get_transport_from_path(osutils.dirname(msgfilename))
         reference_content = msg_transport.get_bytes(basename)
         if not _run_editor(msgfilename):
             return None

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2011-06-19 02:24:39 +0000
+++ bzrlib/mutabletree.py	2011-07-23 16:33:38 +0000
@@ -718,7 +718,7 @@
 
             if kind == 'directory' and directory != '':
                 try:
-                    transport = _mod_transport.get_transport(abspath)
+                    transport = _mod_transport.get_transport_from_path(abspath)
                     controldir.ControlDirFormat.find_format(transport)
                     sub_tree = True
                 except errors.NotBranchError:

=== modified file 'bzrlib/plugins/launchpad/account.py'
--- bzrlib/plugins/launchpad/account.py	2010-06-20 11:18:38 +0000
+++ bzrlib/plugins/launchpad/account.py	2011-07-23 16:33:38 +0000
@@ -100,7 +100,7 @@
     uploaded SSH keys.
     """
     if _transport is None:
-        _transport = transport.get_transport(LAUNCHPAD_BASE)
+        _transport = transport.get_transport_from_url(LAUNCHPAD_BASE)
 
     try:
         data = _transport.get_bytes('~%s/+sshkeys' % username)

=== modified file 'bzrlib/plugins/launchpad/test_lp_api.py'
--- bzrlib/plugins/launchpad/test_lp_api.py	2011-01-17 06:26:35 +0000
+++ bzrlib/plugins/launchpad/test_lp_api.py	2011-07-11 07:02:20 +0000
@@ -17,10 +17,12 @@
 
 from bzrlib import config, errors, osutils
 from bzrlib.tests import (
-    ModuleAvailableFeature,
     TestCase,
     TestCaseWithTransport,
     )
+from bzrlib.tests.features import (
+    ModuleAvailableFeature,
+    )
 
 
 launchpadlib_feature = ModuleAvailableFeature('launchpadlib')

=== modified file 'bzrlib/plugins/launchpad/test_lp_api_lite.py'
--- bzrlib/plugins/launchpad/test_lp_api_lite.py	2011-08-05 16:45:27 +0000
+++ bzrlib/plugins/launchpad/test_lp_api_lite.py	2011-08-12 09:49:24 +0000
@@ -21,13 +21,13 @@
 import socket
 
 from bzrlib import tests
+from bzrlib.tests import features
 from bzrlib.plugins import launchpad
 from bzrlib.plugins.launchpad import lp_api_lite
-
 from testtools.matchers import DocTestMatches
 
 
-class _JSONParserFeature(tests.Feature):
+class _JSONParserFeature(features.Feature):
 
     def _probe(self):
         return lp_api_lite.json is not None
@@ -35,8 +35,10 @@
     def feature_name(self):
         return 'simplejson or json'
 
+
 JSONParserFeature = _JSONParserFeature()
 
+
 _example_response = r"""
 {
     "total_size": 2,
@@ -185,7 +187,7 @@
             return
         # The content should be a valid json result
         content = lp_api_lite.json.loads(json_txt)
-        entries = content['entries'] # It should have an 'entries' field.
+        entries = content['entries']  # It should have an 'entries' field.
         # ws.size should mean we get 0 or 1, and there should be something
         self.assertEqual(1, len(entries))
         entry = entries[0]
@@ -482,6 +484,7 @@
         reported = []
         def report_func(value):
             reported.append(value)
+
         lp_api_lite._report_freshness(latest_ver, branch_latest_ver, place,
                                       verbosity, report_func)
         new_content = '\n'.join(reported)

=== modified file 'bzrlib/plugins/launchpad/test_lp_directory.py'
--- bzrlib/plugins/launchpad/test_lp_directory.py	2011-03-24 11:41:42 +0000
+++ bzrlib/plugins/launchpad/test_lp_directory.py	2011-07-11 07:02:20 +0000
@@ -28,6 +28,7 @@
 from bzrlib.branch import Branch
 from bzrlib.directory_service import directories
 from bzrlib.tests import (
+    features,
     TestCaseInTempDir,
     TestCaseWithMemoryTransport
 )
@@ -50,7 +51,7 @@
     transport_scenarios = [
         ('http', dict(server_class=PreCannedHTTPServer,)),
         ]
-    if tests.HTTPSServerFeature.available():
+    if features.HTTPSServerFeature.available():
         transport_scenarios.append(
             ('https', dict(server_class=PreCannedHTTPSServer,)),
             )
@@ -425,7 +426,7 @@
     pass
 
 
-if tests.HTTPSServerFeature.available():
+if features.HTTPSServerFeature.available():
     from bzrlib.tests import https_server
     class PreCannedHTTPSServer(PreCannedServerMixin, https_server.HTTPSServer):
         pass

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2011-06-30 11:14:06 +0000
+++ bzrlib/repository.py	2011-07-23 11:16:56 +0000
@@ -340,18 +340,6 @@
         """
         self.control_files.break_lock()
 
-    @needs_read_lock
-    def _eliminate_revisions_not_present(self, revision_ids):
-        """Check every revision id in revision_ids to see if we have it.
-
-        Returns a set of the present revisions.
-        """
-        result = []
-        graph = self.get_graph()
-        parent_map = graph.get_parent_map(revision_ids)
-        # The old API returned a list, should this actually be a set?
-        return parent_map.keys()
-
     @staticmethod
     def create(a_bzrdir):
         """Construct the current default format repository in a_bzrdir."""

=== modified file 'bzrlib/smart/server.py'
--- bzrlib/smart/server.py	2011-05-28 09:10:36 +0000
+++ bzrlib/smart/server.py	2011-07-23 16:33:38 +0000
@@ -330,14 +330,14 @@
         chroot_server = chroot.ChrootServer(transport)
         chroot_server.start_server()
         self.cleanups.append(chroot_server.stop_server)
-        transport = _mod_transport.get_transport(chroot_server.get_url())
+        transport = _mod_transport.get_transport_from_url(chroot_server.get_url())
         if self.base_path is not None:
             # Decorate the server's backing transport with a filter that can
             # expand homedirs.
             expand_userdirs = self._make_expand_userdirs_filter(transport)
             expand_userdirs.start_server()
             self.cleanups.append(expand_userdirs.stop_server)
-            transport = _mod_transport.get_transport(expand_userdirs.get_url())
+            transport = _mod_transport.get_transport_from_url(expand_userdirs.get_url())
         self.transport = transport
 
     def _make_smart_server(self, host, port, inet):

=== modified file 'bzrlib/strace.py'
--- bzrlib/strace.py	2010-09-25 20:08:01 +0000
+++ bzrlib/strace.py	2011-06-14 01:26:41 +0000
@@ -28,7 +28,7 @@
 
 # this is currently test-focused, so importing bzrlib.tests is ok. We might
 # want to move feature to its own module though.
-from bzrlib.tests import Feature
+from bzrlib.tests.features import Feature
 
 
 def strace(function, *args, **kwargs):
@@ -98,23 +98,3 @@
         self.err_messages = err_messages
 
 
-class _StraceFeature(Feature):
-
-    def _probe(self):
-        try:
-            proc = subprocess.Popen(['strace'],
-                stderr=subprocess.PIPE,
-                stdout=subprocess.PIPE)
-            proc.communicate()
-            return True
-        except OSError, e:
-            if e.errno == errno.ENOENT:
-                # strace is not installed
-                return False
-            else:
-                raise
-
-    def feature_name(self):
-        return 'strace'
-
-StraceFeature = _StraceFeature()

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2011-08-09 09:11:17 +0000
+++ bzrlib/tests/__init__.py	2011-08-12 09:49:24 +0000
@@ -16,12 +16,6 @@
 
 """Testing framework extensions"""
 
-# TODO: Perhaps there should be an API to find out if bzr running under the
-# test suite -- some plugins might want to avoid making intrusive changes if
-# this is the case.  However, we want behaviour under to test to diverge as
-# little as possible, so this should be used rarely if it's added at all.
-# (Suggestion from j-a-meinel, 2005-11-24)
-
 # NOTE: Some classes in here use camelCaseNaming() rather than
 # underscore_naming().  That's for consistency with unittest; it's not the
 # general style of bzrlib.  Please continue that consistency when adding e.g.
@@ -94,6 +88,10 @@
     memory,
     pathfilter,
     )
+from bzrlib.symbol_versioning import (
+    deprecated_function,
+    deprecated_in,
+    )
 from bzrlib.tests import (
     test_server,
     TestUtil,
@@ -1172,7 +1170,7 @@
 
     def permit_dir(self, name):
         """Permit a directory to be used by this test. See permit_url."""
-        name_transport = _mod_transport.get_transport(name)
+        name_transport = _mod_transport.get_transport_from_path(name)
         self.permit_url(name)
         self.permit_url(name_transport.base)
 
@@ -1257,7 +1255,7 @@
         self.addCleanup(transport_server.stop_server)
         # Obtain a real transport because if the server supplies a password, it
         # will be hidden from the base on the client side.
-        t = _mod_transport.get_transport(transport_server.get_url())
+        t = _mod_transport.get_transport_from_url(transport_server.get_url())
         # Some transport servers effectively chroot the backing transport;
         # others like SFTPServer don't - users of the transport can walk up the
         # transport to read the entire backing transport. This wouldn't matter
@@ -1804,8 +1802,31 @@
         self._preserved_lazy_hooks.clear()
 
     def knownFailure(self, reason):
-        """This test has failed for some known reason."""
-        raise KnownFailure(reason)
+        """Declare that this test fails for a known reason
+
+        Tests that are known to fail should generally be using expectedFailure
+        with an appropriate reverse assertion if a change could cause the test
+        to start passing. Conversely if the test has no immediate prospect of
+        succeeding then using skip is more suitable.
+
+        When this method is called while an exception is being handled, that
+        traceback will be used, otherwise a new exception will be thrown to
+        provide one but won't be reported.
+        """
+        self._add_reason(reason)
+        try:
+            exc_info = sys.exc_info()
+            if exc_info != (None, None, None):
+                self._report_traceback(exc_info)
+            else:
+                try:
+                    raise self.failureException(reason)
+                except self.failureException:
+                    exc_info = sys.exc_info()
+            # GZ 02-08-2011: Maybe cleanup this err.exc_info attribute too?
+            raise testtools.testcase._ExpectedFailure(exc_info)
+        finally:
+            del exc_info
 
     def _suppress_log(self):
         """Remove the log info from details."""
@@ -2398,7 +2419,7 @@
 
         :param relpath: a path relative to the base url.
         """
-        t = _mod_transport.get_transport(self.get_url(relpath))
+        t = _mod_transport.get_transport_from_url(self.get_url(relpath))
         self.assertFalse(t.is_readonly())
         return t
 
@@ -2629,7 +2650,7 @@
             backing_server = self.get_server()
         smart_server = test_server.SmartTCPServer_for_testing()
         self.start_server(smart_server, backing_server)
-        remote_transport = _mod_transport.get_transport(smart_server.get_url()
+        remote_transport = _mod_transport.get_transport_from_url(smart_server.get_url()
                                                    ).clone(path)
         return remote_transport
 
@@ -2652,14 +2673,15 @@
     def setUp(self):
         super(TestCaseWithMemoryTransport, self).setUp()
         # Ensure that ConnectedTransport doesn't leak sockets
-        def get_transport_with_cleanup(*args, **kwargs):
-            t = orig_get_transport(*args, **kwargs)
+        def get_transport_from_url_with_cleanup(*args, **kwargs):
+            t = orig_get_transport_from_url(*args, **kwargs)
             if isinstance(t, _mod_transport.ConnectedTransport):
                 self.addCleanup(t.disconnect)
             return t
 
-        orig_get_transport = self.overrideAttr(_mod_transport, 'get_transport',
-                                               get_transport_with_cleanup)
+        orig_get_transport_from_url = self.overrideAttr(
+            _mod_transport, 'get_transport_from_url',
+            get_transport_from_url_with_cleanup)
         self._make_test_root()
         self.addCleanup(os.chdir, os.getcwdu())
         self.makeAndChdirToTestDir()
@@ -3924,10 +3946,12 @@
         'bzrlib.tests.test_export',
         'bzrlib.tests.test_export_pot',
         'bzrlib.tests.test_extract',
+        'bzrlib.tests.test_features',
         'bzrlib.tests.test_fetch',
         'bzrlib.tests.test_fixtures',
         'bzrlib.tests.test_fifo_cache',
         'bzrlib.tests.test_filters',
+        'bzrlib.tests.test_filter_tree',
         'bzrlib.tests.test_ftp_transport',
         'bzrlib.tests.test_foreign',
         'bzrlib.tests.test_generate_docs',
@@ -4324,6 +4348,7 @@
         the module is available.
     """
 
+    from bzrlib.tests.features import ModuleAvailableFeature
     py_module = pyutils.get_named_object(py_module_name)
     scenarios = [
         ('python', {'module': py_module}),
@@ -4370,157 +4395,6 @@
                          % (os.path.basename(dirname), printable_e))
 
 
-class Feature(object):
-    """An operating system Feature."""
-
-    def __init__(self):
-        self._available = None
-
-    def available(self):
-        """Is the feature available?
-
-        :return: True if the feature is available.
-        """
-        if self._available is None:
-            self._available = self._probe()
-        return self._available
-
-    def _probe(self):
-        """Implement this method in concrete features.
-
-        :return: True if the feature is available.
-        """
-        raise NotImplementedError
-
-    def __str__(self):
-        if getattr(self, 'feature_name', None):
-            return self.feature_name()
-        return self.__class__.__name__
-
-
-class _SymlinkFeature(Feature):
-
-    def _probe(self):
-        return osutils.has_symlinks()
-
-    def feature_name(self):
-        return 'symlinks'
-
-SymlinkFeature = _SymlinkFeature()
-
-
-class _HardlinkFeature(Feature):
-
-    def _probe(self):
-        return osutils.has_hardlinks()
-
-    def feature_name(self):
-        return 'hardlinks'
-
-HardlinkFeature = _HardlinkFeature()
-
-
-class _OsFifoFeature(Feature):
-
-    def _probe(self):
-        return getattr(os, 'mkfifo', None)
-
-    def feature_name(self):
-        return 'filesystem fifos'
-
-OsFifoFeature = _OsFifoFeature()
-
-
-class _UnicodeFilenameFeature(Feature):
-    """Does the filesystem support Unicode filenames?"""
-
-    def _probe(self):
-        try:
-            # Check for character combinations unlikely to be covered by any
-            # single non-unicode encoding. We use the characters
-            # - greek small letter alpha (U+03B1) and
-            # - braille pattern dots-123456 (U+283F).
-            os.stat(u'\u03b1\u283f')
-        except UnicodeEncodeError:
-            return False
-        except (IOError, OSError):
-            # The filesystem allows the Unicode filename but the file doesn't
-            # exist.
-            return True
-        else:
-            # The filesystem allows the Unicode filename and the file exists,
-            # for some reason.
-            return True
-
-UnicodeFilenameFeature = _UnicodeFilenameFeature()
-
-
-class _CompatabilityThunkFeature(Feature):
-    """This feature is just a thunk to another feature.
-
-    It issues a deprecation warning if it is accessed, to let you know that you
-    should really use a different feature.
-    """
-
-    def __init__(self, dep_version, module, name,
-                 replacement_name, replacement_module=None):
-        super(_CompatabilityThunkFeature, self).__init__()
-        self._module = module
-        if replacement_module is None:
-            replacement_module = module
-        self._replacement_module = replacement_module
-        self._name = name
-        self._replacement_name = replacement_name
-        self._dep_version = dep_version
-        self._feature = None
-
-    def _ensure(self):
-        if self._feature is None:
-            depr_msg = self._dep_version % ('%s.%s'
-                                            % (self._module, self._name))
-            use_msg = ' Use %s.%s instead.' % (self._replacement_module,
-                                               self._replacement_name)
-            symbol_versioning.warn(depr_msg + use_msg, DeprecationWarning)
-            # Import the new feature and use it as a replacement for the
-            # deprecated one.
-            self._feature = pyutils.get_named_object(
-                self._replacement_module, self._replacement_name)
-
-    def _probe(self):
-        self._ensure()
-        return self._feature._probe()
-
-
-class ModuleAvailableFeature(Feature):
-    """This is a feature than describes a module we want to be available.
-
-    Declare the name of the module in __init__(), and then after probing, the
-    module will be available as 'self.module'.
-
-    :ivar module: The module if it is available, else None.
-    """
-
-    def __init__(self, module_name):
-        super(ModuleAvailableFeature, self).__init__()
-        self.module_name = module_name
-
-    def _probe(self):
-        try:
-            self._module = __import__(self.module_name, {}, {}, [''])
-            return True
-        except ImportError:
-            return False
-
-    @property
-    def module(self):
-        if self.available(): # Make sure the probe has been done
-            return self._module
-        return None
-
-    def feature_name(self):
-        return self.module_name
-
-
 def probe_unicode_in_user_encoding():
     """Try to encode several unicode strings to use in unicode-aware tests.
     Return first successfull match.
@@ -4554,165 +4428,6 @@
     return None
 
 
-class _HTTPSServerFeature(Feature):
-    """Some tests want an https Server, check if one is available.
-
-    Right now, the only way this is available is under python2.6 which provides
-    an ssl module.
-    """
-
-    def _probe(self):
-        try:
-            import ssl
-            return True
-        except ImportError:
-            return False
-
-    def feature_name(self):
-        return 'HTTPSServer'
-
-
-HTTPSServerFeature = _HTTPSServerFeature()
-
-
-class _UnicodeFilename(Feature):
-    """Does the filesystem support Unicode filenames?"""
-
-    def _probe(self):
-        try:
-            os.stat(u'\u03b1')
-        except UnicodeEncodeError:
-            return False
-        except (IOError, OSError):
-            # The filesystem allows the Unicode filename but the file doesn't
-            # exist.
-            return True
-        else:
-            # The filesystem allows the Unicode filename and the file exists,
-            # for some reason.
-            return True
-
-UnicodeFilename = _UnicodeFilename()
-
-
-class _ByteStringNamedFilesystem(Feature):
-    """Is the filesystem based on bytes?"""
-
-    def _probe(self):
-        if os.name == "posix":
-            return True
-        return False
-
-ByteStringNamedFilesystem = _ByteStringNamedFilesystem()
-
-
-class _UTF8Filesystem(Feature):
-    """Is the filesystem UTF-8?"""
-
-    def _probe(self):
-        if osutils._fs_enc.upper() in ('UTF-8', 'UTF8'):
-            return True
-        return False
-
-UTF8Filesystem = _UTF8Filesystem()
-
-
-class _BreakinFeature(Feature):
-    """Does this platform support the breakin feature?"""
-
-    def _probe(self):
-        from bzrlib import breakin
-        if breakin.determine_signal() is None:
-            return False
-        if sys.platform == 'win32':
-            # Windows doesn't have os.kill, and we catch the SIGBREAK signal.
-            # We trigger SIGBREAK via a Console api so we need ctypes to
-            # access the function
-            try:
-                import ctypes
-            except OSError:
-                return False
-        return True
-
-    def feature_name(self):
-        return "SIGQUIT or SIGBREAK w/ctypes on win32"
-
-
-BreakinFeature = _BreakinFeature()
-
-
-class _CaseInsCasePresFilenameFeature(Feature):
-    """Is the file-system case insensitive, but case-preserving?"""
-
-    def _probe(self):
-        fileno, name = tempfile.mkstemp(prefix='MixedCase')
-        try:
-            # first check truly case-preserving for created files, then check
-            # case insensitive when opening existing files.
-            name = osutils.normpath(name)
-            base, rel = osutils.split(name)
-            found_rel = osutils.canonical_relpath(base, name)
-            return (found_rel == rel
-                    and os.path.isfile(name.upper())
-                    and os.path.isfile(name.lower()))
-        finally:
-            os.close(fileno)
-            os.remove(name)
-
-    def feature_name(self):
-        return "case-insensitive case-preserving filesystem"
-
-CaseInsCasePresFilenameFeature = _CaseInsCasePresFilenameFeature()
-
-
-class _CaseInsensitiveFilesystemFeature(Feature):
-    """Check if underlying filesystem is case-insensitive but *not* case
-    preserving.
-    """
-    # Note that on Windows, Cygwin, MacOS etc, the file-systems are far
-    # more likely to be case preserving, so this case is rare.
-
-    def _probe(self):
-        if CaseInsCasePresFilenameFeature.available():
-            return False
-
-        if TestCaseWithMemoryTransport.TEST_ROOT is None:
-            root = osutils.mkdtemp(prefix='testbzr-', suffix='.tmp')
-            TestCaseWithMemoryTransport.TEST_ROOT = root
-        else:
-            root = TestCaseWithMemoryTransport.TEST_ROOT
-        tdir = osutils.mkdtemp(prefix='case-sensitive-probe-', suffix='',
-            dir=root)
-        name_a = osutils.pathjoin(tdir, 'a')
-        name_A = osutils.pathjoin(tdir, 'A')
-        os.mkdir(name_a)
-        result = osutils.isdir(name_A)
-        _rmtree_temp_dir(tdir)
-        return result
-
-    def feature_name(self):
-        return 'case-insensitive filesystem'
-
-CaseInsensitiveFilesystemFeature = _CaseInsensitiveFilesystemFeature()
-
-
-class _CaseSensitiveFilesystemFeature(Feature):
-
-    def _probe(self):
-        if CaseInsCasePresFilenameFeature.available():
-            return False
-        elif CaseInsensitiveFilesystemFeature.available():
-            return False
-        else:
-            return True
-
-    def feature_name(self):
-        return 'case-sensitive filesystem'
-
-# new coding style is for feature instances to be lowercase
-case_sensitive_filesystem_feature = _CaseSensitiveFilesystemFeature()
-
-
 # Only define SubUnitBzrRunner if subunit is available.
 try:
     from subunit import TestProtocolClient
@@ -4736,26 +4451,9 @@
 except ImportError:
     pass
 
-class _PosixPermissionsFeature(Feature):
-
-    def _probe(self):
-        def has_perms():
-            # create temporary file and check if specified perms are maintained.
-            import tempfile
-
-            write_perms = stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
-            f = tempfile.mkstemp(prefix='bzr_perms_chk_')
-            fd, name = f
-            os.close(fd)
-            os.chmod(name, write_perms)
-
-            read_perms = os.stat(name).st_mode & 0777
-            os.unlink(name)
-            return (write_perms == read_perms)
-
-        return (os.name == 'posix') and has_perms()
-
-    def feature_name(self):
-        return 'POSIX permissions support'
-
-posix_permissions_feature = _PosixPermissionsFeature()
+
+@deprecated_function(deprecated_in((2, 5, 0)))
+def ModuleAvailableFeature(name):
+    from bzrlib.tests import features
+    return features.ModuleAvailableFeature(name)
+    

=== modified file 'bzrlib/tests/blackbox/test_add.py'
--- bzrlib/tests/blackbox/test_add.py	2011-01-10 22:20:12 +0000
+++ bzrlib/tests/blackbox/test_add.py	2011-07-11 06:47:32 +0000
@@ -24,6 +24,7 @@
     tests,
     )
 from bzrlib.tests import (
+    features,
     script,
     )
 from bzrlib.tests.scenarios import load_tests_apply_scenarios
@@ -205,7 +206,7 @@
         self.assertContainsRe(err, r'ERROR:.*\.bzr.*control file')
 
     def test_add_via_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.make_branch_and_tree('source')
         self.build_tree(['source/top.txt'])
         os.symlink('source', 'link')
@@ -213,7 +214,7 @@
         self.assertEquals(out, 'adding top.txt\n')
 
     def test_add_symlink_to_abspath(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.make_branch_and_tree('tree')
         os.symlink(osutils.abspath('target'), 'tree/link')
         out = self.run_bzr(['add', 'tree/link'])[0]
@@ -232,7 +233,7 @@
 
     def test_add_multiple_files_in_unicode_cwd(self):
         """Adding multiple files in a non-ascii cwd, see lp:686611"""
-        self.requireFeature(tests.UnicodeFilename)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self.make_branch_and_tree(u"\xA7")
         self.build_tree([u"\xA7/a", u"\xA7/b"])
         out, err = self.run_bzr(["add", "a", "b"], working_dir=u"\xA7")

=== modified file 'bzrlib/tests/blackbox/test_alias.py'
--- bzrlib/tests/blackbox/test_alias.py	2010-09-23 07:40:07 +0000
+++ bzrlib/tests/blackbox/test_alias.py	2011-06-14 01:26:41 +0000
@@ -21,6 +21,9 @@
     config,
     tests,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestAlias(tests.TestCaseWithTransport):
@@ -48,7 +51,7 @@
         """Unicode aliases should work (Bug #529930)"""
         # XXX: strictly speaking, lack of unicode filenames doesn't imply that
         # unicode command lines aren't available.
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         file_name = u'foo\xb6'
 
         tree = self.make_branch_and_tree('.')

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2011-06-02 08:11:47 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2011-08-12 09:49:24 +0000
@@ -29,10 +29,12 @@
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests import (
     fixtures,
-    HardlinkFeature,
     script,
     test_server,
     )
+from bzrlib.tests.features import (
+    HardlinkFeature,
+    )
 from bzrlib.tests.blackbox import test_switch
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.tests.script import run_script
@@ -287,6 +289,7 @@
         builder = self.make_branch_builder('source')
         source = fixtures.build_branch_with_non_ancestral_rev(builder)
         source.tags.set_tag('tag-a', 'rev-2')
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
         # Now source has a tag not in its ancestry.  Make a branch from it.
         self.run_bzr('branch source new-branch')
         new_branch = branch.Branch.open('new-branch')
@@ -438,7 +441,7 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(36, self.hpss_calls)
+        self.assertLength(37, self.hpss_calls)
 
     def test_branch_from_trivial_branch_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -453,7 +456,7 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(9, self.hpss_calls)
+        self.assertLength(10, self.hpss_calls)
 
     def test_branch_from_trivial_stacked_branch_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -473,12 +476,13 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(14, self.hpss_calls)
+        self.assertLength(15, self.hpss_calls)
 
     def test_branch_from_branch_with_tags(self):
         self.setup_smart_server_with_call_log()
         builder = self.make_branch_builder('source')
         source = fixtures.build_branch_with_non_ancestral_rev(builder)
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
         source.tags.set_tag('tag-a', 'rev-2')
         source.tags.set_tag('tag-missing', 'missing-rev')
         # Now source has a tag not in its ancestry.  Make a branch from it.
@@ -489,7 +493,7 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(9, self.hpss_calls)
+        self.assertLength(10, self.hpss_calls)
 
     def test_branch_to_stacked_from_trivial_branch_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()

=== modified file 'bzrlib/tests/blackbox/test_checkout.py'
--- bzrlib/tests/blackbox/test_checkout.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_checkout.py	2011-06-14 01:26:41 +0000
@@ -32,8 +32,10 @@
     TestCaseWithTransport,
     )
 from bzrlib.tests import (
+    KnownFailure,
+    )
+from bzrlib.tests.features import (
     HardlinkFeature,
-    KnownFailure,
     )
 
 

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2011-06-09 11:48:03 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2011-06-14 01:26:41 +0000
@@ -36,7 +36,7 @@
     probe_bad_non_ascii,
     test_foreign,
     TestSkipped,
-    UnicodeFilenameFeature,
+    features,
     )
 from bzrlib.tests import TestCaseWithTransport
 
@@ -138,7 +138,7 @@
     def test_unicode_commit_message_is_filename(self):
         """Unicode commit message same as a filename (Bug #563646).
         """
-        self.requireFeature(UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         file_name = u'\N{euro sign}'
         self.run_bzr(['init'])
         open(file_name, 'w').write('hello world')

=== modified file 'bzrlib/tests/blackbox/test_conflicts.py'
--- bzrlib/tests/blackbox/test_conflicts.py	2011-05-22 11:51:47 +0000
+++ bzrlib/tests/blackbox/test_conflicts.py	2011-06-14 01:26:41 +0000
@@ -19,7 +19,7 @@
     tests,
     workingtree,
     )
-from bzrlib.tests import script, UnicodeFilenameFeature
+from bzrlib.tests import script, features
 
 
 def make_tree_with_conflicts(test, this_path='this', other_path='other',
@@ -86,7 +86,7 @@
 class TestUnicodePaths(tests.TestCaseWithTransport):
     """Unicode characters in conflicts should be displayed properly"""
 
-    _test_needs_features = [UnicodeFilenameFeature]
+    _test_needs_features = [features.UnicodeFilenameFeature]
     encoding = "UTF-8"
 
     def _as_output(self, text):

=== modified file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	2011-06-10 14:47:25 +0000
+++ bzrlib/tests/blackbox/test_export.py	2011-07-08 23:01:39 +0000
@@ -32,7 +32,10 @@
     osutils,
     tests,
     )
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import (
+    features,
+    TestCaseWithTransport,
+    )
 
 
 class TestExport(TestCaseWithTransport):
@@ -98,7 +101,7 @@
                          sorted(ball.getnames()))
 
     def test_tar_export_unicode_filename(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('tar')
         # FIXME: using fname = u'\xe5.txt' below triggers a bug revealed since
         # bzr.dev revno 4216 but more related to OSX/working trees/unicode than
@@ -116,7 +119,7 @@
 
     def test_tar_export_unicode_basedir(self):
         """Test for bug #413406"""
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         basedir = u'\N{euro sign}'
         os.mkdir(basedir)
         self.run_bzr(['init', basedir])
@@ -194,7 +197,7 @@
     # TODO: test_xz_export, I don't have pylzma working here to test it.
 
     def test_zip_export_unicode(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('zip')
         fname = u'\N{Euro Sign}.txt'
         self.build_tree(['zip/' + fname])

=== modified file 'bzrlib/tests/blackbox/test_filesystem_cicp.py'
--- bzrlib/tests/blackbox/test_filesystem_cicp.py	2010-10-12 10:21:15 +0000
+++ bzrlib/tests/blackbox/test_filesystem_cicp.py	2011-06-14 01:26:41 +0000
@@ -23,9 +23,13 @@
     osutils,
     tests,
     )
-from bzrlib.tests import CaseInsCasePresFilenameFeature, KnownFailure
+from bzrlib.tests import KnownFailure
 from bzrlib.osutils import canonical_relpath, pathjoin
 from bzrlib.tests.script import run_script
+from bzrlib.tests.features import (
+    CaseInsCasePresFilenameFeature,
+    )
+
 
 
 class TestCICPBase(tests.TestCaseWithTransport):

=== modified file 'bzrlib/tests/blackbox/test_log.py'
--- bzrlib/tests/blackbox/test_log.py	2011-06-20 15:25:50 +0000
+++ bzrlib/tests/blackbox/test_log.py	2011-08-04 00:17:53 +0000
@@ -966,7 +966,7 @@
         self.assertLogRevnos([], ["2", "1"])
 
     def test_log_range_open_begin(self):
-        raise tests.KnownFailure("log with ghosts fails. bug #726466")
+        self.knownFailure("log with ghosts fails. bug #726466")
         (stdout, stderr) = self.run_bzr(['log', '-r..2'], retcode=3)
         self.assertEqual(["2", "1"],
                          [r.revno for r in self.get_captured_revisions()])
@@ -974,3 +974,58 @@
 
     def test_log_range_open_end(self):
         self.assertLogRevnos(["-r1.."], ["2", "1"])
+
+class TestLogMatch(TestLogWithLogCatcher):
+    def prepare_tree(self):
+        tree = self.make_branch_and_tree('')
+        self.build_tree(
+            ['/hello.txt', '/goodbye.txt'])
+        tree.add('hello.txt')
+        tree.commit(message='message1', committer='committer1', authors=['author1'])
+        tree.add('goodbye.txt')
+        tree.commit(message='message2', committer='committer2', authors=['author2'])
+    
+    def test_message(self):
+        self.prepare_tree()
+        self.assertLogRevnos(["-m", "message1"], ["1"])
+        self.assertLogRevnos(["-m", "message2"], ["2"])
+        self.assertLogRevnos(["-m", "message"], ["2", "1"])
+        self.assertLogRevnos(["-m", "message1", "-m", "message2"], ["2", "1"])
+        self.assertLogRevnos(["--match-message", "message1"], ["1"])
+        self.assertLogRevnos(["--match-message", "message2"], ["2"])
+        self.assertLogRevnos(["--match-message", "message"], ["2", "1"])
+        self.assertLogRevnos(["--match-message", "message1", 
+                              "--match-message", "message2"], ["2", "1"])
+        self.assertLogRevnos(["--message", "message1"], ["1"])
+        self.assertLogRevnos(["--message", "message2"], ["2"])
+        self.assertLogRevnos(["--message", "message"], ["2", "1"])
+        self.assertLogRevnos(["--match-message", "message1", 
+                              "--message", "message2"], ["2", "1"])
+        self.assertLogRevnos(["--message", "message1", 
+                              "--match-message", "message2"], ["2", "1"])
+
+    def test_committer(self):
+        self.prepare_tree()
+        self.assertLogRevnos(["-m", "committer1"], ["1"])
+        self.assertLogRevnos(["-m", "committer2"], ["2"])
+        self.assertLogRevnos(["-m", "committer"], ["2", "1"])
+        self.assertLogRevnos(["-m", "committer1", "-m", "committer2"], 
+                             ["2", "1"])
+        self.assertLogRevnos(["--match-committer", "committer1"], ["1"])
+        self.assertLogRevnos(["--match-committer", "committer2"], ["2"])
+        self.assertLogRevnos(["--match-committer", "committer"], ["2", "1"])
+        self.assertLogRevnos(["--match-committer", "committer1", 
+                              "--match-committer", "committer2"], ["2", "1"])
+
+    def test_author(self):
+        self.prepare_tree()
+        self.assertLogRevnos(["-m", "author1"], ["1"])
+        self.assertLogRevnos(["-m", "author2"], ["2"])
+        self.assertLogRevnos(["-m", "author"], ["2", "1"])
+        self.assertLogRevnos(["-m", "author1", "-m", "author2"], 
+                             ["2", "1"])
+        self.assertLogRevnos(["--match-author", "author1"], ["1"])
+        self.assertLogRevnos(["--match-author", "author2"], ["2"])
+        self.assertLogRevnos(["--match-author", "author"], ["2", "1"])
+        self.assertLogRevnos(["--match-author", "author1", 
+                              "--match-author", "author2"], ["2", "1"])

=== modified file 'bzrlib/tests/blackbox/test_lsprof.py'
--- bzrlib/tests/blackbox/test_lsprof.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_lsprof.py	2011-06-14 01:26:41 +0000
@@ -15,12 +15,12 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 from bzrlib import tests
-from bzrlib.tests import test_lsprof
+from bzrlib.tests import features
 
 
 class TestLSProf(tests.TestCaseInTempDir):
 
-    _test_needs_features = [test_lsprof.LSProfFeature]
+    _test_needs_features = [features.lsprof_feature]
 
     def test_file(self):
         out, err = self.run_bzr('--lsprof-file output.callgrind rocks')

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2011-06-13 14:32:13 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2011-08-09 13:30:21 +0000
@@ -681,6 +681,7 @@
         builder.build_commit(message="Rev 2a", rev_id='rev-2a')
         source.tags.set_tag('tag-a', 'rev-2a')
         source.set_last_revision_info(1, 'rev-1')
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
         builder.build_commit(message="Rev 2b", rev_id='rev-2b')
         # Merge from source
         self.run_bzr('merge -d target source')

=== modified file 'bzrlib/tests/blackbox/test_mv.py'
--- bzrlib/tests/blackbox/test_mv.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_mv.py	2011-07-11 06:47:32 +0000
@@ -25,10 +25,12 @@
     )
 
 from bzrlib.tests import (
+    TestCaseWithTransport,
+    )
+from bzrlib.tests.features import (
     CaseInsensitiveFilesystemFeature,
     SymlinkFeature,
-    TestCaseWithTransport,
-    UnicodeFilename,
+    UnicodeFilenameFeature,
     )
 
 
@@ -508,7 +510,7 @@
 
     def test_mv_unversioned_non_ascii(self):
         """Clear error on mv of an unversioned non-ascii file, see lp:707954"""
-        self.requireFeature(UnicodeFilename)
+        self.requireFeature(UnicodeFilenameFeature)
         tree = self.make_branch_and_tree(".")
         self.build_tree([u"\xA7"])
         out, err = self.run_bzr_error(["Could not rename", "not versioned"],

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2011-08-04 00:17:53 +0000
@@ -89,7 +89,7 @@
             normal_thing = unicodedata.normalize('NFD', path)
             mac_encoded = normal_thing.encode(fs_enc)
             if mac_encoded != encoded:
-                raise tests.KnownFailure(
+                self.knownFailure(
                     'Unable to roundtrip path %r on OSX filesystem'
                     ' using encoding "%s"'
                     % (path, fs_enc))

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2011-02-11 05:57:31 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2011-08-10 14:02:04 +0000
@@ -152,6 +152,7 @@
         # Make a source, sprout a target off it
         builder = self.make_branch_builder('source')
         source = fixtures.build_branch_with_non_ancestral_rev(builder)
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
         target_bzrdir = source.bzrdir.sprout('target')
         source.tags.set_tag('tag-a', 'rev-2')
         # Pull from source
@@ -410,7 +411,7 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(18, self.hpss_calls)
+        self.assertLength(19, self.hpss_calls)
         remote = Branch.open('stacked')
         self.assertEndsWith(remote.get_stacked_on_url(), '/parent')
     

=== modified file 'bzrlib/tests/blackbox/test_remove.py'
--- bzrlib/tests/blackbox/test_remove.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_remove.py	2011-06-14 01:26:41 +0000
@@ -20,7 +20,7 @@
 
 from bzrlib.tests import (
     script,
-    SymlinkFeature,
+    features,
     TestCaseWithTransport,
     TestSkipped,
     )
@@ -125,7 +125,7 @@
         self.assertNotInWorkingTree('bar', tree=tree)
 
     def test_remove_no_files_specified_missing_link(self):
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self._make_tree_and_add(['foo'])
         os.symlink('foo', 'linkname')
         tree.add(['linkname'])

=== modified file 'bzrlib/tests/features.py'
--- bzrlib/tests/features.py	2011-06-17 14:38:08 +0000
+++ bzrlib/tests/features.py	2011-07-25 02:38:30 +0000
@@ -14,19 +14,313 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""A collection of commonly used 'Features' which bzrlib uses to skip tests."""
+"""A collection of commonly used 'Features' to optionally run tests.
+"""
 
 import os
 import stat
 import sys
+import tempfile
 
 from bzrlib import (
     osutils,
+    symbol_versioning,
     tests,
     )
 
 
-class _NotRunningAsRoot(tests.Feature):
+class Feature(object):
+    """An operating system Feature."""
+
+    def __init__(self):
+        self._available = None
+
+    def available(self):
+        """Is the feature available?
+
+        :return: True if the feature is available.
+        """
+        if self._available is None:
+            self._available = self._probe()
+        return self._available
+
+    def _probe(self):
+        """Implement this method in concrete features.
+
+        :return: True if the feature is available.
+        """
+        raise NotImplementedError
+
+    def __str__(self):
+        if getattr(self, 'feature_name', None):
+            return self.feature_name()
+        return self.__class__.__name__
+
+
+class _SymlinkFeature(Feature):
+
+    def _probe(self):
+        return osutils.has_symlinks()
+
+    def feature_name(self):
+        return 'symlinks'
+
+SymlinkFeature = _SymlinkFeature()
+
+
+class _HardlinkFeature(Feature):
+
+    def _probe(self):
+        return osutils.has_hardlinks()
+
+    def feature_name(self):
+        return 'hardlinks'
+
+HardlinkFeature = _HardlinkFeature()
+
+
+class _OsFifoFeature(Feature):
+
+    def _probe(self):
+        return getattr(os, 'mkfifo', None)
+
+    def feature_name(self):
+        return 'filesystem fifos'
+
+OsFifoFeature = _OsFifoFeature()
+
+
+class _UnicodeFilenameFeature(Feature):
+    """Does the filesystem support Unicode filenames?"""
+
+    def _probe(self):
+        try:
+            # Check for character combinations unlikely to be covered by any
+            # single non-unicode encoding. We use the characters
+            # - greek small letter alpha (U+03B1) and
+            # - braille pattern dots-123456 (U+283F).
+            os.stat(u'\u03b1\u283f')
+        except UnicodeEncodeError:
+            return False
+        except (IOError, OSError):
+            # The filesystem allows the Unicode filename but the file doesn't
+            # exist.
+            return True
+        else:
+            # The filesystem allows the Unicode filename and the file exists,
+            # for some reason.
+            return True
+
+UnicodeFilenameFeature = _UnicodeFilenameFeature()
+
+
+class _CompatabilityThunkFeature(Feature):
+    """This feature is just a thunk to another feature.
+
+    It issues a deprecation warning if it is accessed, to let you know that you
+    should really use a different feature.
+    """
+
+    def __init__(self, dep_version, module, name,
+                 replacement_name, replacement_module=None):
+        super(_CompatabilityThunkFeature, self).__init__()
+        self._module = module
+        if replacement_module is None:
+            replacement_module = module
+        self._replacement_module = replacement_module
+        self._name = name
+        self._replacement_name = replacement_name
+        self._dep_version = dep_version
+        self._feature = None
+
+    def _ensure(self):
+        if self._feature is None:
+            from bzrlib import pyutils
+            depr_msg = self._dep_version % ('%s.%s'
+                                            % (self._module, self._name))
+            use_msg = ' Use %s.%s instead.' % (self._replacement_module,
+                                               self._replacement_name)
+            symbol_versioning.warn(depr_msg + use_msg, DeprecationWarning)
+            # Import the new feature and use it as a replacement for the
+            # deprecated one.
+            self._feature = pyutils.get_named_object(
+                self._replacement_module, self._replacement_name)
+
+    def _probe(self):
+        self._ensure()
+        return self._feature._probe()
+
+
+class ModuleAvailableFeature(Feature):
+    """This is a feature than describes a module we want to be available.
+
+    Declare the name of the module in __init__(), and then after probing, the
+    module will be available as 'self.module'.
+
+    :ivar module: The module if it is available, else None.
+    """
+
+    def __init__(self, module_name):
+        super(ModuleAvailableFeature, self).__init__()
+        self.module_name = module_name
+
+    def _probe(self):
+        try:
+            self._module = __import__(self.module_name, {}, {}, [''])
+            return True
+        except ImportError:
+            return False
+
+    @property
+    def module(self):
+        if self.available():
+            return self._module
+        return None
+
+    def feature_name(self):
+        return self.module_name
+
+
+class _HTTPSServerFeature(Feature):
+    """Some tests want an https Server, check if one is available.
+
+    Right now, the only way this is available is under python2.6 which provides
+    an ssl module.
+    """
+
+    def _probe(self):
+        try:
+            import ssl
+            return True
+        except ImportError:
+            return False
+
+    def feature_name(self):
+        return 'HTTPSServer'
+
+
+HTTPSServerFeature = _HTTPSServerFeature()
+
+
+class _ByteStringNamedFilesystem(Feature):
+    """Is the filesystem based on bytes?"""
+
+    def _probe(self):
+        if os.name == "posix":
+            return True
+        return False
+
+ByteStringNamedFilesystem = _ByteStringNamedFilesystem()
+
+
+class _UTF8Filesystem(Feature):
+    """Is the filesystem UTF-8?"""
+
+    def _probe(self):
+        if osutils._fs_enc.upper() in ('UTF-8', 'UTF8'):
+            return True
+        return False
+
+UTF8Filesystem = _UTF8Filesystem()
+
+
+class _BreakinFeature(Feature):
+    """Does this platform support the breakin feature?"""
+
+    def _probe(self):
+        from bzrlib import breakin
+        if breakin.determine_signal() is None:
+            return False
+        if sys.platform == 'win32':
+            # Windows doesn't have os.kill, and we catch the SIGBREAK signal.
+            # We trigger SIGBREAK via a Console api so we need ctypes to
+            # access the function
+            try:
+                import ctypes
+            except OSError:
+                return False
+        return True
+
+    def feature_name(self):
+        return "SIGQUIT or SIGBREAK w/ctypes on win32"
+
+
+BreakinFeature = _BreakinFeature()
+
+
+class _CaseInsCasePresFilenameFeature(Feature):
+    """Is the file-system case insensitive, but case-preserving?"""
+
+    def _probe(self):
+        fileno, name = tempfile.mkstemp(prefix='MixedCase')
+        try:
+            # first check truly case-preserving for created files, then check
+            # case insensitive when opening existing files.
+            name = osutils.normpath(name)
+            base, rel = osutils.split(name)
+            found_rel = osutils.canonical_relpath(base, name)
+            return (found_rel == rel
+                    and os.path.isfile(name.upper())
+                    and os.path.isfile(name.lower()))
+        finally:
+            os.close(fileno)
+            os.remove(name)
+
+    def feature_name(self):
+        return "case-insensitive case-preserving filesystem"
+
+CaseInsCasePresFilenameFeature = _CaseInsCasePresFilenameFeature()
+
+
+class _CaseInsensitiveFilesystemFeature(Feature):
+    """Check if underlying filesystem is case-insensitive but *not* case
+    preserving.
+    """
+    # Note that on Windows, Cygwin, MacOS etc, the file-systems are far
+    # more likely to be case preserving, so this case is rare.
+
+    def _probe(self):
+        if CaseInsCasePresFilenameFeature.available():
+            return False
+
+        if tests.TestCaseWithMemoryTransport.TEST_ROOT is None:
+            root = osutils.mkdtemp(prefix='testbzr-', suffix='.tmp')
+            tests.TestCaseWithMemoryTransport.TEST_ROOT = root
+        else:
+            root = tests.TestCaseWithMemoryTransport.TEST_ROOT
+        tdir = osutils.mkdtemp(prefix='case-sensitive-probe-', suffix='',
+            dir=root)
+        name_a = osutils.pathjoin(tdir, 'a')
+        name_A = osutils.pathjoin(tdir, 'A')
+        os.mkdir(name_a)
+        result = osutils.isdir(name_A)
+        tests._rmtree_temp_dir(tdir)
+        return result
+
+    def feature_name(self):
+        return 'case-insensitive filesystem'
+
+CaseInsensitiveFilesystemFeature = _CaseInsensitiveFilesystemFeature()
+
+
+class _CaseSensitiveFilesystemFeature(Feature):
+
+    def _probe(self):
+        if CaseInsCasePresFilenameFeature.available():
+            return False
+        elif CaseInsensitiveFilesystemFeature.available():
+            return False
+        else:
+            return True
+
+    def feature_name(self):
+        return 'case-sensitive filesystem'
+
+# new coding style is for feature instances to be lowercase
+case_sensitive_filesystem_feature = _CaseSensitiveFilesystemFeature()
+
+
+class _NotRunningAsRoot(Feature):
 
     def _probe(self):
         try:
@@ -42,19 +336,24 @@
 
 not_running_as_root = _NotRunningAsRoot()
 
-apport = tests.ModuleAvailableFeature('apport')
-gpgme = tests.ModuleAvailableFeature('gpgme')
-lzma = tests.ModuleAvailableFeature('lzma')
-meliae = tests.ModuleAvailableFeature('meliae')
-paramiko = tests.ModuleAvailableFeature('paramiko')
-pycurl = tests.ModuleAvailableFeature('pycurl')
-pywintypes = tests.ModuleAvailableFeature('pywintypes')
-sphinx = tests.ModuleAvailableFeature('sphinx')
-subunit = tests.ModuleAvailableFeature('subunit')
-testtools = tests.ModuleAvailableFeature('testtools')
-
-
-class _BackslashDirSeparatorFeature(tests.Feature):
+apport = ModuleAvailableFeature('apport')
+gpgme = ModuleAvailableFeature('gpgme')
+lzma = ModuleAvailableFeature('lzma')
+meliae = ModuleAvailableFeature('meliae')
+paramiko = ModuleAvailableFeature('paramiko')
+pycurl = ModuleAvailableFeature('pycurl')
+pywintypes = ModuleAvailableFeature('pywintypes')
+sphinx = ModuleAvailableFeature('sphinx')
+subunit = ModuleAvailableFeature('subunit')
+testtools = ModuleAvailableFeature('testtools')
+
+compiled_patiencediff_feature = ModuleAvailableFeature(
+    'bzrlib._patiencediff_c')
+meliae_feature = ModuleAvailableFeature('meliae.scanner')
+lsprof_feature = ModuleAvailableFeature('bzrlib.lsprof')
+
+
+class _BackslashDirSeparatorFeature(Feature):
 
     def _probe(self):
         try:
@@ -70,33 +369,7 @@
 backslashdir_feature = _BackslashDirSeparatorFeature()
 
 
-class _PosixPermissionsFeature(tests.Feature):
-
-    def _probe(self):
-        def has_perms():
-            # create temporary file and check if specified perms are maintained.
-            import tempfile
-
-            write_perms = stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
-            f = tempfile.mkstemp(prefix='bzr_perms_chk_')
-            fd, name = f
-            os.close(fd)
-            os.chmod(name, write_perms)
-
-            read_perms = os.stat(name).st_mode & 0777
-            os.unlink(name)
-            return (write_perms == read_perms)
-
-        return (os.name == 'posix') and has_perms()
-
-    def feature_name(self):
-        return 'POSIX permissions support'
-
-
-posix_permissions_feature = _PosixPermissionsFeature()
-
-
-class _ChownFeature(tests.Feature):
+class _ChownFeature(Feature):
     """os.chown is supported"""
 
     def _probe(self):
@@ -105,7 +378,7 @@
 chown_feature = _ChownFeature()
 
 
-class ExecutableFeature(tests.Feature):
+class ExecutableFeature(Feature):
     """Feature testing whether an executable of a given name is on the PATH."""
 
     def __init__(self, name):
@@ -132,7 +405,73 @@
 diff_feature = ExecutableFeature('diff')
 
 
-class Win32Feature(tests.Feature):
+class _PosixPermissionsFeature(Feature):
+
+    def _probe(self):
+        def has_perms():
+            # Create temporary file and check if specified perms are
+            # maintained.
+            write_perms = stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
+            f = tempfile.mkstemp(prefix='bzr_perms_chk_')
+            fd, name = f
+            os.close(fd)
+            os.chmod(name, write_perms)
+
+            read_perms = os.stat(name).st_mode & 0777
+            os.unlink(name)
+            return (write_perms == read_perms)
+
+        return (os.name == 'posix') and has_perms()
+
+    def feature_name(self):
+        return 'POSIX permissions support'
+
+
+posix_permissions_feature = _PosixPermissionsFeature()
+
+
+class _StraceFeature(Feature):
+
+    def _probe(self):
+        try:
+            proc = subprocess.Popen(['strace'],
+                stderr=subprocess.PIPE,
+                stdout=subprocess.PIPE)
+            proc.communicate()
+            return True
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                # strace is not installed
+                return False
+            else:
+                raise
+
+    def feature_name(self):
+        return 'strace'
+
+
+strace_feature = _StraceFeature()
+
+
+class _AttribFeature(Feature):
+
+    def _probe(self):
+        if (sys.platform not in ('cygwin', 'win32')):
+            return False
+        try:
+            proc = subprocess.Popen(['attrib', '.'], stdout=subprocess.PIPE)
+        except OSError, e:
+            return False
+        return (0 == proc.wait())
+
+    def feature_name(self):
+        return 'attrib Windows command-line tool'
+
+
+AttribFeature = _AttribFeature()
+
+
+class Win32Feature(Feature):
     """Feature testing whether we're running selftest on Windows
     or Windows-like platform.
     """
@@ -143,4 +482,28 @@
     def feature_name(self):
         return "win32 platform"
 
+
 win32_feature = Win32Feature()
+
+
+for name in ['HTTPServerFeature', 
+    'HTTPSServerFeature', 'SymlinkFeature', 'HardlinkFeature',
+    'OsFifoFeature', 'UnicodeFilenameFeature',
+    'ByteStringNamedFilesystem', 'UTF8Filesystem',
+    'BreakinFeature', 'CaseInsCasePresFilenameFeature',
+    'CaseInsensitiveFilesystemFeature', 'case_sensitive_filesystem_feature',
+    'posix_permissions_feature',
+    ]:
+    setattr(tests, name, _CompatabilityThunkFeature(
+        symbol_versioning.deprecated_in((2, 5, 0)),
+        'bzrlib.tests', name,
+        name, 'bzrlib.tests.features'))
+
+
+for (old_name, new_name) in [
+    ('UnicodeFilename', 'UnicodeFilenameFeature'),
+    ]:
+    setattr(tests, name, _CompatabilityThunkFeature(
+        symbol_versioning.deprecated_in((2, 5, 0)),
+        'bzrlib.tests', old_name,
+        new_name, 'bzrlib.tests.features'))

=== modified file 'bzrlib/tests/fixtures.py'
--- bzrlib/tests/fixtures.py	2011-02-09 06:36:35 +0000
+++ bzrlib/tests/fixtures.py	2011-07-04 20:17:44 +0000
@@ -125,3 +125,16 @@
     source.set_last_revision_info(1, 'rev-1')
     return source
 
+
+def make_branch_and_populated_tree(testcase):
+    """Make a simple branch and tree.
+
+    The tree holds some added but uncommitted files.
+    """
+    # TODO: Either accept or return the names of the files, so the caller
+    # doesn't need to be bound to the particular files created? -- mbp
+    # 20110705
+    tree = testcase.make_branch_and_tree('t')
+    testcase.build_tree_contents([('t/hello', 'hello world')])
+    tree.add(['hello'], ['hello-id'])
+    return tree

=== modified file 'bzrlib/tests/ftp_server/__init__.py'
--- bzrlib/tests/ftp_server/__init__.py	2011-06-07 07:05:46 +0000
+++ bzrlib/tests/ftp_server/__init__.py	2011-06-14 01:26:41 +0000
@@ -13,6 +13,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
 """
 Facilities to use ftp test servers.
 """
@@ -20,6 +21,9 @@
 import sys
 
 from bzrlib import tests
+from bzrlib.tests import (
+    features,
+    )
 
 
 try:
@@ -42,7 +46,7 @@
     pyftpdlib_available = False
 
 
-class _FTPServerFeature(tests.Feature):
+class _FTPServerFeature(features.Feature):
     """Some tests want an FTP Server, check if one is available.
 
     Right now, the only way this is available is if one of the following is

=== modified file 'bzrlib/tests/http_utils.py'
--- bzrlib/tests/http_utils.py	2011-06-07 11:11:08 +0000
+++ bzrlib/tests/http_utils.py	2011-07-23 16:33:38 +0000
@@ -51,12 +51,12 @@
 
     def do_POST(self):
         """Hand the request off to a smart server instance."""
-        backing = transport.get_transport(
+        backing = transport.get_transport_from_path(
             self.server.test_case_server._home_dir)
         chroot_server = chroot.ChrootServer(backing)
         chroot_server.start_server()
         try:
-            t = transport.get_transport(chroot_server.get_url())
+            t = transport.get_transport_from_url(chroot_server.get_url())
             self.do_POST_inner(t)
         finally:
             chroot_server.stop_server()
@@ -153,7 +153,7 @@
         return self._adjust_url(base, relpath)
 
     def get_secondary_transport(self, relpath=None):
-        t = transport.get_transport(self.get_secondary_url(relpath))
+        t = transport.get_transport_from_url(self.get_secondary_url(relpath))
         self.assertTrue(t.is_readonly())
         return t
 
@@ -257,7 +257,7 @@
         return self._adjust_url(base, relpath)
 
    def get_old_transport(self, relpath=None):
-        t = transport.get_transport(self.get_old_url(relpath))
+        t = transport.get_transport_from_url(self.get_old_url(relpath))
         self.assertTrue(t.is_readonly())
         return t
 
@@ -266,7 +266,7 @@
         return self._adjust_url(base, relpath)
 
    def get_new_transport(self, relpath=None):
-        t = transport.get_transport(self.get_new_url(relpath))
+        t = transport.get_transport_from_url(self.get_new_url(relpath))
         self.assertTrue(t.is_readonly())
         return t
 

=== modified file 'bzrlib/tests/per_branch/test_branch.py'
--- bzrlib/tests/per_branch/test_branch.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/per_branch/test_branch.py	2011-07-23 16:33:38 +0000
@@ -673,7 +673,7 @@
             return
         # supported formats must be able to init and open
         t = self.get_transport()
-        readonly_t = transport.get_transport(self.get_readonly_url())
+        readonly_t = transport.get_transport_from_url(self.get_readonly_url())
         made_branch = self.make_branch('.')
         self.assertIsInstance(made_branch, _mod_branch.Branch)
 

=== modified file 'bzrlib/tests/per_branch/test_pull.py'
--- bzrlib/tests/per_branch/test_pull.py	2011-02-09 06:36:35 +0000
+++ bzrlib/tests/per_branch/test_pull.py	2011-08-09 14:18:05 +0000
@@ -156,6 +156,7 @@
         except errors.TagsNotSupported:
             raise TestNotApplicable('format does not support tags.')
         source.tags.set_tag('tag-a', 'rev-2')
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
         target.pull(source)
         # The tag is present, and so is its revision.
         self.assertEqual('rev-2', target.tags.lookup_tag('tag-a'))
@@ -177,6 +178,7 @@
             source.tags.set_tag('tag-a', 'rev-2')
         except errors.TagsNotSupported:
             raise TestNotApplicable('format does not support tags.')
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
         target.pull(source, 'rev-2-again')
         # The tag is present, and so is its revision.
         self.assertEqual('rev-2', target.tags.lookup_tag('tag-a'))

=== modified file 'bzrlib/tests/per_branch/test_push.py'
--- bzrlib/tests/per_branch/test_push.py	2011-05-14 20:01:04 +0000
+++ bzrlib/tests/per_branch/test_push.py	2011-07-23 16:33:38 +0000
@@ -424,7 +424,7 @@
     def test_empty_branch_api(self):
         """The branch_obj.push API should make a limited number of HPSS calls.
         """
-        t = transport.get_transport(self.smart_server.get_url()).clone('target')
+        t = transport.get_transport_from_url(self.smart_server.get_url()).clone('target')
         target = branch.Branch.open_from_transport(t)
         self.empty_branch.push(target)
         self.assertEqual(

=== modified file 'bzrlib/tests/per_branch/test_sprout.py'
--- bzrlib/tests/per_branch/test_sprout.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/per_branch/test_sprout.py	2011-06-14 01:26:41 +0000
@@ -25,6 +25,9 @@
     revision as _mod_revision,
     tests,
     )
+from bzrlib.tests import (
+    features,
+    )
 from bzrlib.tests.per_branch import TestCaseWithBranch
 
 
@@ -151,8 +154,8 @@
         # Since the trigger function seems to be set_parent_trees, there exists
         # also a similar test, with name test_unicode_symlink, in class
         # TestSetParents at file per_workingtree/test_parents.py
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
 
         tree = self.make_branch_and_tree('tree1')
 

=== modified file 'bzrlib/tests/per_controldir/test_controldir.py'
--- bzrlib/tests/per_controldir/test_controldir.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/per_controldir/test_controldir.py	2011-08-10 14:02:04 +0000
@@ -690,6 +690,7 @@
             source.tags.set_tag('tag-a', 'rev-2')
         except errors.TagsNotSupported:
             raise TestNotApplicable('Branch format does not support tags.')
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
         # Now source has a tag not in its ancestry.  Sprout its controldir.
         dir = source.bzrdir
         target = dir.sprout(self.get_url('target'))
@@ -761,6 +762,7 @@
             source.tags.set_tag('tag-absent', 'absent-rev')
         except errors.TagsNotSupported:
             raise TestNotApplicable('Branch format does not support tags.')
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
         # And ask sprout for C2
         dir = source.bzrdir
         target = dir.sprout(self.get_url('target'), revision_id='rev-c2')

=== modified file 'bzrlib/tests/per_intertree/__init__.py'
--- bzrlib/tests/per_intertree/__init__.py	2011-05-03 23:54:46 +0000
+++ bzrlib/tests/per_intertree/__init__.py	2011-08-04 14:01:47 +0000
@@ -122,6 +122,7 @@
     default_tree_format = WorkingTreeFormat3()
     submod_tests = loader.loadTestsFromModuleNames([
         'bzrlib.tests.per_intertree.test_compare',
+        'bzrlib.tests.per_intertree.test_file_content_matches',
         ])
     test_intertree_permutations = [
         # test InterTree with two default-format working trees.

=== modified file 'bzrlib/tests/per_intertree/test_compare.py'
--- bzrlib/tests/per_intertree/test_compare.py	2009-10-02 05:43:41 +0000
+++ bzrlib/tests/per_intertree/test_compare.py	2011-08-04 14:01:47 +0000
@@ -23,11 +23,12 @@
     errors,
     mutabletree,
     tests,
-    workingtree_4,
     )
-from bzrlib.osutils import file_kind, has_symlinks
-from bzrlib.tests import TestNotApplicable
+from bzrlib.osutils import has_symlinks
 from bzrlib.tests.per_intertree import TestCaseWithTwoTrees
+from bzrlib.tests import (
+    features,
+    )
 
 # TODO: test the include_root option.
 # TODO: test that renaming a directory x->y does not emit a rename for the
@@ -963,7 +964,7 @@
         # To add confusion, the totally generic code path works - but it
         # doesn't update persistent metadata. So this test permits InterTrees
         # to either work, or fail with BadFileKindError.
-        self.requireFeature(tests.OsFifoFeature)
+        self.requireFeature(features.OsFifoFeature)
         tree1 = self.make_branch_and_tree('1')
         self.build_tree(['1/a'])
         tree1.set_root_id('root-id')
@@ -1368,7 +1369,7 @@
         return self.mutable_trees_to_locked_test_trees(tree1, tree2)
 
     def test_versioned_symlinks(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1, tree2 = self.make_trees_with_symlinks()
         self.not_applicable_if_cannot_represent_unversioned(tree2)
         root_id = tree1.path2id('')
@@ -1391,7 +1392,7 @@
         self.check_has_changes(True, tree1, tree2)
 
     def test_versioned_symlinks_specific_files(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1, tree2 = self.make_trees_with_symlinks()
         root_id = tree1.path2id('')
         expected = [

=== added file 'bzrlib/tests/per_intertree/test_file_content_matches.py'
--- bzrlib/tests/per_intertree/test_file_content_matches.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_intertree/test_file_content_matches.py	2011-08-04 14:01:47 +0000
@@ -0,0 +1,48 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for the InterTree.file_content_matches() function."""
+
+from bzrlib.tests.per_intertree import TestCaseWithTwoTrees
+
+
+class TestFileContentMatches(TestCaseWithTwoTrees):
+
+    def test_same_contents_and_verifier(self):
+        tree1 = self.make_branch_and_tree('1')
+        tree2 = self.make_to_branch_and_tree('2')
+        self.build_tree_contents([
+            ('1/file', 'apples'),
+            ('2/file', 'apples'),
+            ])
+        tree1.add('file', 'file-id-1')
+        tree2.add('file', 'file-id-2')
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        inter = self.intertree_class(tree1, tree2)
+        self.assertTrue(inter.file_content_matches('file-id-1', 'file-id-2'))
+
+    def test_different_contents_and_same_verifier(self):
+        tree1 = self.make_branch_and_tree('1')
+        tree2 = self.make_to_branch_and_tree('2')
+        self.build_tree_contents([
+            ('1/file', 'apples'),
+            ('2/file', 'oranges'),
+            ])
+        tree1.add('file', 'file-id-1')
+        tree2.add('file', 'file-id-2')
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        inter = self.intertree_class(tree1, tree2)
+        self.assertFalse(inter.file_content_matches('file-id-1', 'file-id-2'))

=== modified file 'bzrlib/tests/per_lock/test_lock.py'
--- bzrlib/tests/per_lock/test_lock.py	2010-09-29 22:03:03 +0000
+++ bzrlib/tests/per_lock/test_lock.py	2011-06-14 01:26:41 +0000
@@ -21,7 +21,9 @@
     osutils,
     )
 
-from bzrlib.tests import (features, UnicodeFilenameFeature)
+from bzrlib.tests import (
+    features,
+    )
 from bzrlib.tests.per_lock import TestCaseWithLock
 
 
@@ -166,7 +168,7 @@
 
 class TestLockUnicodePath(TestCaseWithLock):
 
-    _test_needs_features = [UnicodeFilenameFeature]
+    _test_needs_features = [features.UnicodeFilenameFeature]
 
     def test_read_lock(self):
         self.build_tree([u'\u1234'])

=== modified file 'bzrlib/tests/per_repository/test_commit_builder.py'
--- bzrlib/tests/per_repository/test_commit_builder.py	2011-06-13 15:16:17 +0000
+++ bzrlib/tests/per_repository/test_commit_builder.py	2011-07-08 23:01:39 +0000
@@ -28,6 +28,9 @@
     tests,
     )
 from bzrlib.tests import per_repository
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestCommitBuilder(per_repository.TestCaseWithRepository):
@@ -627,14 +630,14 @@
 
     def test_last_modified_revision_after_commit_link_unchanged(self):
         # committing without changing a link does not change the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_check_unchanged(tree, 'link')
 
     def test_last_modified_revision_after_commit_link_unchanged_ric(self):
         # committing without changing a link does not change the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_check_unchanged(tree, 'link',
@@ -709,14 +712,14 @@
 
     def test_last_modified_revision_after_rename_link_changes(self):
         # renaming a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_renamed_check_changed(tree, 'link')
 
     def test_last_modified_revision_after_rename_link_changes_ric(self):
         # renaming a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_renamed_check_changed(tree, 'link',
@@ -784,14 +787,14 @@
 
     def test_last_modified_revision_after_reparent_link_changes(self):
         # reparenting a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_reparent_check_changed(tree, 'link')
 
     def test_last_modified_revision_after_reparent_link_changes_ric(self):
         # reparenting a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_reparent_check_changed(tree, 'link',
@@ -1019,7 +1022,7 @@
 
     def test_last_modified_revision_after_content_link_changes(self):
         # changing a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         def change_link():
@@ -1032,7 +1035,7 @@
         if file_id is None:
             file_id = link
         # changing a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink(target, link)
         def change_link():
@@ -1048,7 +1051,7 @@
             'link', 'target', 'newtarget')
 
     def test_last_modified_rev_after_content_unicode_link_changes_ric(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_last_mod_rev_after_content_link_changes_ric(
             u'li\u1234nk', u'targ\N{Euro Sign}t', u'n\N{Euro Sign}wtarget',
 
@@ -1113,14 +1116,14 @@
 
     def test_last_modified_revision_after_merge_link_changes(self):
         # merge a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge(tree1, 'link')
 
     def test_last_modified_revision_after_merge_link_changes_ric(self):
         # merge a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge(tree1, 'link',
@@ -1207,14 +1210,14 @@
 
     def test_last_modified_revision_after_converged_merge_link_unchanged(self):
         # merge a link that changed preserves the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge_converged(tree1, 'link')
 
     def test_last_modified_revision_after_converged_merge_link_unchanged_ric(self):
         # merge a link that changed preserves the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge_converged(tree1, 'link',
@@ -1260,7 +1263,7 @@
         self.build_tree([name])
 
     def make_link(self, name):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         os.symlink('target', name)
 
     def make_reference(self, name):

=== modified file 'bzrlib/tests/per_repository/test_fetch.py'
--- bzrlib/tests/per_repository/test_fetch.py	2011-07-12 23:32:24 +0000
+++ bzrlib/tests/per_repository/test_fetch.py	2011-08-04 00:17:53 +0000
@@ -344,7 +344,7 @@
             # The code inside fetch() that tries to lock and then fails, also
             # causes weird problems with 'lock_not_held' later on...
             target.lock_read()
-            raise tests.KnownFailure('some repositories fail to fetch'
+            self.knownFailure('some repositories fail to fetch'
                 ' via the smart server because of locking issues.')
 
     def test_fetch_from_smart_with_ghost(self):

=== modified file 'bzrlib/tests/per_repository/test_repository.py'
--- bzrlib/tests/per_repository/test_repository.py	2011-06-28 11:42:02 +0000
+++ bzrlib/tests/per_repository/test_repository.py	2011-08-04 00:17:53 +0000
@@ -691,7 +691,7 @@
         except errors.IncompatibleFormat:
             raise tests.TestNotApplicable('Cannot make a shared repository')
         if repo.bzrdir._format.fixed_components:
-            raise tests.KnownFailure(
+            self.knownFailure(
                 "pre metadir branches do not upgrade on push "
                 "with stacking policy")
         if isinstance(repo._format,

=== modified file 'bzrlib/tests/per_transport.py'
--- bzrlib/tests/per_transport.py	2011-05-18 14:45:15 +0000
+++ bzrlib/tests/per_transport.py	2011-08-04 00:17:53 +0000
@@ -1136,7 +1136,7 @@
             raise TestSkipped("Transport %s does not support symlinks." %
                               self._server.__class__)
         except IOError:
-            raise tests.KnownFailure("Paramiko fails to create symlinks during tests")
+            self.knownFailure("Paramiko fails to create symlinks during tests")
 
     def test_list_dir(self):
         # TODO: Test list_dir, just try once, and if it throws, stop testing
@@ -1555,7 +1555,7 @@
 
         no_unicode_support = getattr(self._server, 'no_unicode_support', False)
         if no_unicode_support:
-            raise tests.KnownFailure("test server cannot handle unicode paths")
+            self.knownFailure("test server cannot handle unicode paths")
 
         try:
             self.build_tree(files, transport=t, line_endings='binary')

=== modified file 'bzrlib/tests/per_tree/__init__.py'
--- bzrlib/tests/per_tree/__init__.py	2011-04-28 05:30:49 +0000
+++ bzrlib/tests/per_tree/__init__.py	2011-06-14 01:26:41 +0000
@@ -37,6 +37,9 @@
     )
 from bzrlib.revisiontree import RevisionTree
 from bzrlib.transform import TransformPreview
+from bzrlib.tests import (
+    features,
+    )
 from bzrlib.workingtree import (
     format_registry,
     )
@@ -242,7 +245,7 @@
         note that the order of the paths and fileids is deliberately
         mismatched to ensure that the result order is path based.
         """
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('.')
         paths = ['0file',
             '1top-dir/',
@@ -274,7 +277,7 @@
 
     def _create_tree_with_utf8(self, tree):
         """Generate a tree with a utf8 revision and unicode paths."""
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         # We avoid combining characters in file names here, normalization
         # checks (as performed by some file systems (OSX) are outside the scope
         # of these tests).  We use the euro sign \N{Euro Sign} or \u20ac in

=== modified file 'bzrlib/tests/per_tree/test_export.py'
--- bzrlib/tests/per_tree/test_export.py	2011-05-05 15:24:56 +0000
+++ bzrlib/tests/per_tree/test_export.py	2011-06-14 01:26:41 +0000
@@ -22,6 +22,9 @@
 from bzrlib import osutils
 from bzrlib import tests
 from bzrlib.tests.per_tree import TestCaseWithTree
+from bzrlib.tests import (
+    features,
+    )
 
 
 class ExportTest(object):
@@ -37,7 +40,7 @@
         export(tree_a, 'output', self.exporter)
 
     def prepare_symlink_export(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         work_a = self.make_branch_and_tree('wta')
         os.symlink('target', 'wta/link')
         work_a.add('link', 'link-id')

=== modified file 'bzrlib/tests/per_tree/test_get_symlink_target.py'
--- bzrlib/tests/per_tree/test_get_symlink_target.py	2011-05-13 14:02:14 +0000
+++ bzrlib/tests/per_tree/test_get_symlink_target.py	2011-06-14 01:26:41 +0000
@@ -23,12 +23,15 @@
     tests,
     )
 from bzrlib.tests import per_tree
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestGetSymlinkTarget(per_tree.TestCaseWithTree):
 
     def get_tree_with_symlinks(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('tree')
         os.symlink('foo', 'tree/link')
         os.symlink('../bar', 'tree/rel_link')
@@ -48,8 +51,8 @@
         self.assertEqual('foo', tree.get_symlink_target('link-id', 'link'))
 
     def test_get_unicode_symlink_target(self):
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('tree')
         target = u'targ\N{Euro Sign}t'
         os.symlink(target,  u'tree/\u03b2_link'.encode(osutils._fs_enc))

=== modified file 'bzrlib/tests/per_tree/test_inv.py'
--- bzrlib/tests/per_tree/test_inv.py	2011-05-06 14:53:40 +0000
+++ bzrlib/tests/per_tree/test_inv.py	2011-06-14 01:26:41 +0000
@@ -28,6 +28,9 @@
 from bzrlib.tree import InventoryTree
 from bzrlib.transform import _PreviewTree
 from bzrlib.uncommit import uncommit
+from bzrlib.tests import (
+    features,
+    )
 
 
 def get_entry(tree, file_id):
@@ -36,7 +39,7 @@
 
 class TestInventoryWithSymlinks(per_tree.TestCaseWithTree):
 
-    _test_needs_features = [tests.SymlinkFeature]
+    _test_needs_features = [features.SymlinkFeature]
 
     def setUp(self):
         per_tree.TestCaseWithTree.setUp(self)
@@ -153,7 +156,7 @@
         # some of the trees we want to use can only exist on a disk, not in
         # memory - therefore we can only test this if the filesystem is
         # case-sensitive.
-        self.requireFeature(tests.case_sensitive_filesystem_feature)
+        self.requireFeature(features.case_sensitive_filesystem_feature)
         work_tree = self.make_branch_and_tree('.')
         self.build_tree(['test/', 'test/file', 'Test'])
         work_tree.add(['test/', 'test/file', 'Test'])

=== modified file 'bzrlib/tests/per_tree/test_is_executable.py'
--- bzrlib/tests/per_tree/test_is_executable.py	2010-11-29 23:35:13 +0000
+++ bzrlib/tests/per_tree/test_is_executable.py	2011-06-14 01:26:41 +0000
@@ -16,6 +16,8 @@
 
 from bzrlib.tests import (
     per_tree,
+    )
+from bzrlib.tests.features import (
     SymlinkFeature,
     )
 

=== modified file 'bzrlib/tests/per_tree/test_path_content_summary.py'
--- bzrlib/tests/per_tree/test_path_content_summary.py	2011-04-18 18:52:19 +0000
+++ bzrlib/tests/per_tree/test_path_content_summary.py	2011-06-14 01:26:41 +0000
@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Test that all Tree's implement path_content_summary."""
+"""Test that all Trees implement path_content_summary."""
 
 import os
 
@@ -24,7 +24,13 @@
     transform,
     )
 
-from bzrlib.tests import per_tree
+from bzrlib.tests import (
+    features,
+    per_tree,
+    )
+from bzrlib.tests.features import (
+    SymlinkFeature,
+    )
 
 
 class TestPathContentSummary(per_tree.TestCaseWithTree):
@@ -48,7 +54,7 @@
             self.fail("invalid size in summary: %r" % (returned_size,))
 
     def test_symlink_content_summary(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(SymlinkFeature)
         tree = self.make_branch_and_tree('tree')
         os.symlink('target', 'tree/path')
         tree.add(['path'])
@@ -56,8 +62,8 @@
         self.assertEqual(('symlink', None, None, 'target'), summary)
 
     def test_unicode_symlink_content_summary(self):
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('tree')
         os.symlink('target', u'tree/\u03b2-path'.encode(osutils._fs_enc))
         tree.add([u'\u03b2-path'])
@@ -65,8 +71,8 @@
         self.assertEqual(('symlink', None, None, 'target'), summary)
 
     def test_unicode_symlink_target_summary(self):
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('tree')
         os.symlink(u'tree/\u03b2-path'.encode(osutils._fs_enc), 'tree/link')
         tree.add(['link'])

=== modified file 'bzrlib/tests/per_tree/test_test_trees.py'
--- bzrlib/tests/per_tree/test_test_trees.py	2011-05-08 13:07:11 +0000
+++ bzrlib/tests/per_tree/test_test_trees.py	2011-06-14 01:26:41 +0000
@@ -18,6 +18,9 @@
 
 from bzrlib import tests
 from bzrlib.tests import per_tree
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestTreeShapes(per_tree.TestCaseWithTree):
@@ -147,7 +150,7 @@
         # currently this test tree requires unicode. It might be good
         # to have it simply stop having the single unicode file in it
         # when dealing with a non-unicode filesystem.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.get_tree_with_subdirs_and_all_content_types()
         tree.lock_read()
         self.addCleanup(tree.unlock)

=== modified file 'bzrlib/tests/per_tree/test_tree.py'
--- bzrlib/tests/per_tree/test_tree.py	2011-06-14 02:21:41 +0000
+++ bzrlib/tests/per_tree/test_tree.py	2011-08-05 13:34:40 +0000
@@ -303,3 +303,23 @@
         self.addCleanup(tree.unlock)
         expected = osutils.sha_strings('file content')
         self.assertEqual(expected, tree.get_file_sha1('file-id'))
+
+
+class TestGetFileVerifier(TestCaseWithTree):
+
+    def test_get_file_verifier(self):
+        work_tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([
+            ('tree/file1', 'file content'),
+            ('tree/file2', 'file content')])
+        work_tree.add(['file1', 'file2'], ['file-id-1', 'file-id-2'])
+        tree = self._convert_tree(work_tree)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        (kind, data) = tree.get_file_verifier('file-id-1')
+        self.assertEquals(
+            tree.get_file_verifier('file-id-1'),
+            tree.get_file_verifier('file-id-2'))
+        if kind == "SHA1":
+            expected = osutils.sha_strings('file content')
+            self.assertEqual(expected, data)

=== modified file 'bzrlib/tests/per_workingtree/test_move.py'
--- bzrlib/tests/per_workingtree/test_move.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/per_workingtree/test_move.py	2011-07-25 02:51:30 +0000
@@ -25,6 +25,9 @@
     )
 
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestMove(TestCaseWithWorkingTree):
@@ -348,6 +351,9 @@
         tree._validate()
 
     def test_move_directory_into_parent(self):
+        if not self.workingtree_format.supports_versioned_directories:
+            raise tests.TestNotApplicable(
+                "test requires versioned directories")
         tree = self.make_branch_and_tree('.')
         self.build_tree(['c/', 'c/b/', 'c/b/d/'])
         tree.add(['c', 'c/b', 'c/b/d'],
@@ -372,14 +378,20 @@
         tree.commit('initial', rev_id='rev-1')
         root_id = tree.get_root_id()
 
-
         tree.rename_one('a/b', 'a/c/b')
-        self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
-                               ('d', 'd-id'),
-                               ('a/c', 'c-id'),
-                               ('a/c/b', 'b-id'),
-                              ], tree)
+        if self.workingtree_format.supports_versioned_directories:
+            self.assertTreeLayout([('', root_id),
+                                   ('a', 'a-id'),
+                                   ('d', 'd-id'),
+                                   ('a/c', 'c-id'),
+                                   ('a/c/b', 'b-id'),
+                                  ], tree)
+        else:
+            self.assertTreeLayout([('', root_id),
+                                   ('a', 'a-id'),
+                                   ('a/c', 'c-id'),
+                                   ('a/c/b', 'b-id'),
+                                  ], tree)
         self.assertEqual([('a', 'd/a')],
                          tree.move(['a'], 'd'))
         self.assertTreeLayout([('', root_id),
@@ -546,7 +558,7 @@
 
     def test_move_to_unversioned_non_ascii_dir(self):
         """Check error when moving to unversioned non-ascii directory"""
-        self.requireFeature(tests.UnicodeFilename)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree(".")
         self.build_tree(["a", u"\xA7/"])
         tree.add(["a"])
@@ -557,7 +569,7 @@
 
     def test_move_unversioned_non_ascii(self):
         """Check error when moving an unversioned non-ascii file"""
-        self.requireFeature(tests.UnicodeFilename)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree(".")
         self.build_tree([u"\xA7", "dir/"])
         tree.add("dir")

=== modified file 'bzrlib/tests/per_workingtree/test_parents.py'
--- bzrlib/tests/per_workingtree/test_parents.py	2011-05-08 16:02:52 +0000
+++ bzrlib/tests/per_workingtree/test_parents.py	2011-06-14 01:26:41 +0000
@@ -33,6 +33,9 @@
     )
 from bzrlib.revisiontree import InventoryRevisionTree
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
+from bzrlib.tests import (
+    features,
+    )
 from bzrlib.uncommit import uncommit
 
 
@@ -228,8 +231,8 @@
 
     def test_unicode_symlink(self):
         # this tests bug #272444
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
 
         tree = self.make_branch_and_tree('tree1')
 
@@ -239,7 +242,7 @@
         target = u'\u03a9'
         link_name = u'\N{Euro Sign}link'
         os.symlink(target, 'tree1/' + link_name)
-        tree.add([link_name],['link-id'])
+        tree.add([link_name], ['link-id'])
 
         revision1 = tree.commit('added a link to a Unicode target')
         revision2 = tree.commit('this revision will be discarded')
@@ -476,8 +479,8 @@
 
     def test_no_parents_just_root(self):
         """Test doing an empty commit - no parent, set a root only."""
-        basis_shape = Inventory(root_id=None) # empty tree
-        new_shape = Inventory() # tree with a root
+        basis_shape = Inventory(root_id=None)  # empty tree
+        new_shape = Inventory()  # tree with a root
         self.assertTransitionFromBasisToShape(basis_shape, None, new_shape,
             'new_parent')
 
@@ -528,10 +531,13 @@
         def do_file(inv, revid):
             self.add_file(inv, revid, 'path-id', 'root-id', 'path', '1' * 32,
                 12)
+
         def do_link(inv, revid):
             self.add_link(inv, revid, 'path-id', 'root-id', 'path', 'target')
+
         def do_dir(inv, revid):
             self.add_dir(inv, revid, 'path-id', 'root-id', 'path')
+
         for old_factory in (do_file, do_link, do_dir):
             for new_factory in (do_file, do_link, do_dir):
                 if old_factory == new_factory:

=== modified file 'bzrlib/tests/per_workingtree/test_rename_one.py'
--- bzrlib/tests/per_workingtree/test_rename_one.py	2011-05-24 21:09:26 +0000
+++ bzrlib/tests/per_workingtree/test_rename_one.py	2011-07-11 06:47:32 +0000
@@ -23,6 +23,9 @@
     osutils,
     tests,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
@@ -384,7 +387,7 @@
 
     def test_rename_unversioned_non_ascii(self):
         """Check error when renaming an unversioned non-ascii file"""
-        self.requireFeature(tests.UnicodeFilename)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree(".")
         self.build_tree([u"\xA7"])
         e = self.assertRaises(errors.BzrRenameFailedError,
@@ -394,7 +397,7 @@
 
     def test_rename_into_unversioned_non_ascii_dir(self):
         """Check error when renaming into unversioned non-ascii directory"""
-        self.requireFeature(tests.UnicodeFilename)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree(".")
         self.build_tree(["a", u"\xA7/"])
         tree.add(["a"])
@@ -405,7 +408,7 @@
 
     def test_rename_over_already_versioned_non_ascii(self):
         """Check error renaming over an already versioned non-ascii file"""
-        self.requireFeature(tests.UnicodeFilename)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree(".")
         self.build_tree(["a", u"\xA7"])
         tree.add(["a", u"\xA7"])
@@ -416,7 +419,7 @@
 
     def test_rename_after_non_existant_non_ascii(self):
         """Check error renaming after move with missing non-ascii file"""
-        self.requireFeature(tests.UnicodeFilename)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree(".")
         self.build_tree(["a"])
         tree.add(["a"])

=== modified file 'bzrlib/tests/per_workingtree/test_smart_add.py'
--- bzrlib/tests/per_workingtree/test_smart_add.py	2011-05-28 14:56:33 +0000
+++ bzrlib/tests/per_workingtree/test_smart_add.py	2011-06-14 01:26:41 +0000
@@ -27,8 +27,9 @@
     tests,
     )
 from bzrlib.tests import (
+    features,
+    per_workingtree,
     test_smart_add,
-    per_workingtree,
     )
 
 
@@ -288,7 +289,7 @@
 
 class TestSmartAddTreeUnicode(per_workingtree.TestCaseWithWorkingTree):
 
-    _test_needs_features = [tests.UnicodeFilenameFeature]
+    _test_needs_features = [features.UnicodeFilenameFeature]
 
     def setUp(self):
         super(TestSmartAddTreeUnicode, self).setUp()

=== modified file 'bzrlib/tests/per_workingtree/test_symlinks.py'
--- bzrlib/tests/per_workingtree/test_symlinks.py	2011-05-04 11:15:37 +0000
+++ bzrlib/tests/per_workingtree/test_symlinks.py	2011-07-25 02:51:30 +0000
@@ -25,13 +25,16 @@
     workingtree,
     )
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestSmartAddTree(TestCaseWithWorkingTree):
 
     # See eg <https://bugs.launchpad.net/bzr/+bug/192859>
 
-    _test_needs_features = [tests.SymlinkFeature]
+    _test_needs_features = [features.SymlinkFeature]
 
     def test_smart_add_symlink(self):
         tree = self.make_branch_and_tree('tree')
@@ -77,7 +80,7 @@
 
 class TestKindChanges(TestCaseWithWorkingTree):
 
-    _test_needs_features = [tests.SymlinkFeature]
+    _test_needs_features = [features.SymlinkFeature]
 
     def test_symlink_changes_to_dir(self):
         # <https://bugs.launchpad.net/bzr/+bug/192859>:
@@ -101,8 +104,11 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         self.assertEquals([], list(tree.iter_changes(tree.basis_tree())))
-        self.assertEquals(
-            ['a', 'a/f'], sorted(info[0] for info in tree.list_files()))
+        if tree._format.supports_versioned_directories:
+            self.assertEquals(
+                ['a', 'a/f'], sorted(info[0] for info in tree.list_files()))
+        else:
+            self.assertEquals([], list(tree.list_files()))
 
     def test_dir_changes_to_symlink(self):
         # <https://bugs.launchpad.net/bzr/+bug/192859>:
@@ -127,7 +133,7 @@
 
 class TestOpenTree(TestCaseWithWorkingTree):
 
-    _test_needs_features = [tests.SymlinkFeature]
+    _test_needs_features = [features.SymlinkFeature]
 
     def test_open_containing_through_symlink(self):
         self.make_test_tree()

=== modified file 'bzrlib/tests/per_workingtree/test_walkdirs.py'
--- bzrlib/tests/per_workingtree/test_walkdirs.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_walkdirs.py	2011-06-14 01:26:41 +0000
@@ -19,7 +19,7 @@
 import os
 
 from bzrlib import transform
-from bzrlib.tests import SymlinkFeature
+from bzrlib.tests.features import SymlinkFeature
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 # tests to write:

=== modified file 'bzrlib/tests/per_workingtree/test_workingtree.py'
--- bzrlib/tests/per_workingtree/test_workingtree.py	2011-06-28 11:29:03 +0000
+++ bzrlib/tests/per_workingtree/test_workingtree.py	2011-07-08 23:01:39 +0000
@@ -36,7 +36,11 @@
     )
 from bzrlib.inventory import Inventory
 from bzrlib.osutils import pathjoin, getcwd, has_symlinks
-from bzrlib.tests import TestSkipped, TestNotApplicable
+from bzrlib.tests import (
+    features,
+    TestSkipped,
+    TestNotApplicable,
+    )
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 from bzrlib.workingtree import (
     TreeDirectory,
@@ -1085,7 +1089,7 @@
         if osutils.normalizes_filenames():
             # You *can't* create an illegal filename on OSX.
             raise tests.TestNotApplicable('OSX normalizes filenames')
-        self.requireFeature(tests.UTF8Filesystem)
+        self.requireFeature(features.UTF8Filesystem)
         # We require a UTF8 filesystem, because otherwise we would need to get
         # tricky to figure out how to create an illegal filename.
         # \xb5 is an illegal path because it should be \xc2\xb5 for UTF-8

=== modified file 'bzrlib/tests/test__chunks_to_lines.py'
--- bzrlib/tests/test__chunks_to_lines.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__chunks_to_lines.py	2011-06-14 01:26:41 +0000
@@ -18,6 +18,9 @@
 """Tests for chunks_to_lines."""
 
 from bzrlib import tests
+from bzrlib.tests import (
+    features,
+    )
 
 
 def load_tests(standard_tests, module, loader):
@@ -29,8 +32,8 @@
 # test_osutils depends on this feature being around. We can't just use the one
 # generated by load_tests, because if we only load osutils but not this module,
 # then that code never gets run
-compiled_chunkstolines_feature = tests.ModuleAvailableFeature(
-                                    'bzrlib._chunks_to_lines_pyx')
+compiled_chunkstolines_feature = features.ModuleAvailableFeature(
+    'bzrlib._chunks_to_lines_pyx')
 
 
 class TestChunksToLines(tests.TestCase):

=== modified file 'bzrlib/tests/test__dirstate_helpers.py'
--- bzrlib/tests/test__dirstate_helpers.py	2011-05-06 15:15:44 +0000
+++ bzrlib/tests/test__dirstate_helpers.py	2011-06-14 01:26:41 +0000
@@ -34,12 +34,15 @@
     load_tests_apply_scenarios,
     multiply_scenarios,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 
 load_tests = load_tests_apply_scenarios
 
 
-compiled_dirstate_helpers_feature = tests.ModuleAvailableFeature(
+compiled_dirstate_helpers_feature = features.ModuleAvailableFeature(
     'bzrlib._dirstate_helpers_pyx')
 
 
@@ -951,7 +954,7 @@
 
     def test_update_entry_symlink(self):
         """Update entry should read symlinks."""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         state, entry = self.get_state_with_a()
         state.save()
         self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
@@ -1148,7 +1151,7 @@
 
     def test_update_file_to_symlink(self):
         """File becomes a symlink"""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The file sha1 won't be cached unless the file is old
         state.adjust_time(+10)
@@ -1167,7 +1170,7 @@
 
     def test_update_dir_to_symlink(self):
         """Directory becomes a symlink"""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink target won't be cached if it isn't old
         state.adjust_time(+10)
@@ -1177,7 +1180,7 @@
 
     def test_update_symlink_to_file(self):
         """Symlink becomes a file"""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink and file info won't be cached unless old
         state.adjust_time(+10)
@@ -1187,7 +1190,7 @@
 
     def test_update_symlink_to_dir(self):
         """Symlink becomes a directory"""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink target won't be cached if it isn't old
         state.adjust_time(+10)

=== modified file 'bzrlib/tests/test__groupcompress.py'
--- bzrlib/tests/test__groupcompress.py	2011-05-14 10:28:59 +0000
+++ bzrlib/tests/test__groupcompress.py	2011-06-14 01:26:41 +0000
@@ -23,6 +23,9 @@
 from bzrlib.tests.scenarios import (
     load_tests_apply_scenarios,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 
 def module_scenarios():
@@ -57,8 +60,8 @@
 load_tests = load_tests_apply_scenarios
 
 
-compiled_groupcompress_feature = tests.ModuleAvailableFeature(
-                                    'bzrlib._groupcompress_pyx')
+compiled_groupcompress_feature = features.ModuleAvailableFeature(
+    'bzrlib._groupcompress_pyx')
 
 _text1 = """\
 This is a bit

=== modified file 'bzrlib/tests/test__known_graph.py'
--- bzrlib/tests/test__known_graph.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test__known_graph.py	2011-06-14 01:26:41 +0000
@@ -26,6 +26,9 @@
 from bzrlib.tests import test_graph
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests.scenarios import load_tests_apply_scenarios
+from bzrlib.tests import (
+    features,
+    )
 
 
 def caching_scenarios():
@@ -52,7 +55,7 @@
 load_tests = load_tests_apply_scenarios
 
 
-compiled_known_graph_feature = tests.ModuleAvailableFeature(
+compiled_known_graph_feature = features.ModuleAvailableFeature(
     'bzrlib._known_graph_pyx')
 
 

=== modified file 'bzrlib/tests/test__simple_set.py'
--- bzrlib/tests/test__simple_set.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test__simple_set.py	2011-06-14 01:26:41 +0000
@@ -21,6 +21,9 @@
 from bzrlib import (
     tests,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 try:
     from bzrlib import _simple_set_pyx
@@ -75,7 +78,7 @@
 
 # Even though this is an extension, we don't permute the tests for a python
 # version. As the plain python version is just a dict or set
-compiled_simpleset_feature = tests.ModuleAvailableFeature(
+compiled_simpleset_feature = features.ModuleAvailableFeature(
                                 'bzrlib._simple_set_pyx')
 
 

=== modified file 'bzrlib/tests/test__static_tuple.py'
--- bzrlib/tests/test__static_tuple.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test__static_tuple.py	2011-06-14 01:26:41 +0000
@@ -26,6 +26,9 @@
     static_tuple,
     tests,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 
 def load_tests(standard_tests, module, loader):
@@ -37,21 +40,6 @@
     return suite
 
 
-class _Meliae(tests.Feature):
-
-    def _probe(self):
-        try:
-            from meliae import scanner
-        except ImportError:
-            return False
-        return True
-
-    def feature_name(self):
-        return "Meliae - python memory debugger"
-
-Meliae = _Meliae()
-
-
 class TestStaticTuple(tests.TestCase):
 
     def assertRefcount(self, count, obj):
@@ -444,7 +432,7 @@
         # amount of referenced memory. Unfortunately gc.get_referents() first
         # checks the IS_GC flag before it traverses anything. We could write a
         # helper func, but that won't work for the generic implementation...
-        self.requireFeature(Meliae)
+        self.requireFeature(features.meliae_feature)
         from meliae import scanner
         strs = ['foo', 'bar', 'baz', 'bing']
         k = self.module.StaticTuple(*strs)
@@ -455,7 +443,7 @@
         self.assertEqual(sorted(refs), sorted(scanner.get_referents(k)))
 
     def test_nested_referents(self):
-        self.requireFeature(Meliae)
+        self.requireFeature(features.meliae_feature)
         from meliae import scanner
         strs = ['foo', 'bar', 'baz', 'bing']
         k1 = self.module.StaticTuple(*strs[:2])

=== modified file 'bzrlib/tests/test__walkdirs_win32.py'
--- bzrlib/tests/test__walkdirs_win32.py	2010-05-16 12:21:31 +0000
+++ bzrlib/tests/test__walkdirs_win32.py	2011-06-14 01:26:41 +0000
@@ -22,9 +22,12 @@
     osutils,
     tests,
     )
-
-
-win32_readdir_feature = tests.ModuleAvailableFeature('bzrlib._walkdirs_win32')
+from bzrlib.tests import (
+    features,
+    )
+
+
+win32_readdir_feature = features.ModuleAvailableFeature('bzrlib._walkdirs_win32')
 
 
 class TestWin32Finder(tests.TestCaseInTempDir):

=== modified file 'bzrlib/tests/test_atomicfile.py'
--- bzrlib/tests/test_atomicfile.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_atomicfile.py	2011-07-15 09:04:54 +0000
@@ -124,30 +124,3 @@
         f.commit()
         st = os.lstat('test')
         self.assertEqualMode(0666 & ~umask, stat.S_IMODE(st.st_mode))
-
-    def test_closed(self):
-        local_warnings = []
-        def capture_warnings(msg, cls, stacklevel=None):
-            self.assertEqual(cls, DeprecationWarning)
-            local_warnings.append(msg)
-
-        method = symbol_versioning.warn
-        try:
-            symbol_versioning.set_warning_method(capture_warnings)
-            f = atomicfile.AtomicFile('test', mode='wb')
-            self.assertEqual(False, f.closed)
-            f.abort()
-            self.assertEqual(True, f.closed)
-
-            f = atomicfile.AtomicFile('test', mode='wb')
-            f.close()
-            self.assertEqual(True, f.closed)
-
-            f = atomicfile.AtomicFile('test', mode='wb')
-            f.commit()
-            self.assertEqual(True, f.closed)
-        finally:
-            symbol_versioning.set_warning_method(method)
-
-        txt = 'AtomicFile.closed deprecated in bzr 0.10'
-        self.assertEqual([txt]*4, local_warnings)

=== modified file 'bzrlib/tests/test_bad_files.py'
--- bzrlib/tests/test_bad_files.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_bad_files.py	2011-06-14 01:26:41 +0000
@@ -24,7 +24,10 @@
     errors,
     )
 from bzrlib.status import show_tree_status
-from bzrlib.tests import OsFifoFeature, TestCaseWithTransport
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.features import (
+    OsFifoFeature,
+    )
 
 
 def verify_status(tester, tree, value):

=== modified file 'bzrlib/tests/test_btree_index.py'
--- bzrlib/tests/test_btree_index.py	2011-01-27 14:27:18 +0000
+++ bzrlib/tests/test_btree_index.py	2011-06-14 01:26:41 +0000
@@ -33,6 +33,9 @@
     TestCaseWithTransport,
     scenarios,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 
 load_tests = scenarios.load_tests_apply_scenarios
@@ -47,7 +50,7 @@
     return scenarios
 
 
-compiled_btreeparser_feature = tests.ModuleAvailableFeature(
+compiled_btreeparser_feature = features.ModuleAvailableFeature(
     'bzrlib._btree_serializer_pyx')
 
 

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_bundle.py	2011-06-14 01:26:41 +0000
@@ -46,6 +46,9 @@
     test_commit,
     )
 from bzrlib.transform import TreeTransform
+from bzrlib.tests import (
+    features,
+    )
 
 
 def get_text(vf, key):
@@ -682,7 +685,7 @@
     def _test_symlink_bundle(self, link_name, link_target, new_link_target):
         link_id = 'link-1'
 
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
 
@@ -729,7 +732,7 @@
         self._test_symlink_bundle('link', 'bar/foo', 'mars')
 
     def test_unicode_symlink_bundle(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_symlink_bundle(u'\N{Euro Sign}link',
                                   u'bar/\N{Euro Sign}foo',
                                   u'mars\N{Euro Sign}')
@@ -836,7 +839,7 @@
         return bundle_file.getvalue()
 
     def test_unicode_bundle(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         # Handle international characters
         os.mkdir('b1')
         f = open(u'b1/with Dod\N{Euro Sign}', 'wb')

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_bzrdir.py	2011-08-10 14:02:04 +0000
@@ -26,6 +26,7 @@
 from bzrlib import (
     branch,
     bzrdir,
+    config,
     controldir,
     errors,
     help_topics,
@@ -1208,7 +1209,7 @@
 
     def __init__(self, *args, **kwargs):
         super(_TestBzrDir, self).__init__(*args, **kwargs)
-        self.test_branch = _TestBranch()
+        self.test_branch = _TestBranch(self.transport)
         self.test_branch.repository = self.create_repository()
 
     def open_branch(self, unsupported=False):
@@ -1225,15 +1226,17 @@
 class _TestBranch(bzrlib.branch.Branch):
     """Test Branch implementation for TestBzrDirSprout."""
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, transport, *args, **kwargs):
         self._format = _TestBranchFormat()
+        self._transport = transport
+        self.base = transport.base
         super(_TestBranch, self).__init__(*args, **kwargs)
         self.calls = []
         self._parent = None
 
     def sprout(self, *args, **kwargs):
         self.calls.append('sprout')
-        return _TestBranch()
+        return _TestBranch(self._transport)
 
     def copy_content_into(self, destination, revision_id=None):
         self.calls.append('copy_content_into')
@@ -1244,6 +1247,9 @@
     def get_parent(self):
         return self._parent
 
+    def _get_config(self):
+        return config.TransportConfig(self._transport, 'branch.conf')
+
     def set_parent(self, parent):
         self._parent = parent
 

=== modified file 'bzrlib/tests/test_commands.py'
--- bzrlib/tests/test_commands.py	2011-04-19 03:23:05 +0000
+++ bzrlib/tests/test_commands.py	2011-07-15 14:27:20 +0000
@@ -354,11 +354,3 @@
         cmds = list(commands.all_command_names())
         self.assertEqual(['called'], hook_calls)
         self.assertSubset(['foo', 'bar'], cmds)
-
-
-class TestDeprecations(tests.TestCase):
-
-    def test_shlex_split_unicode_deprecation(self):
-        res = self.applyDeprecated(
-                symbol_versioning.deprecated_in((2, 2, 0)),
-                commands.shlex_split_unicode, 'whatever')

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2011-06-16 15:29:10 +0000
+++ bzrlib/tests/test_commit.py	2011-07-08 23:01:39 +0000
@@ -33,10 +33,12 @@
     LockContention,
     )
 from bzrlib.tests import (
-    SymlinkFeature,
     TestCaseWithTransport,
     test_foreign,
     )
+from bzrlib.tests.features import (
+    SymlinkFeature,
+    )
 from bzrlib.tests.matchers import MatchesAncestry
 
 

=== modified file 'bzrlib/tests/test_commit_merge.py'
--- bzrlib/tests/test_commit_merge.py	2011-04-17 23:06:22 +0000
+++ bzrlib/tests/test_commit_merge.py	2011-06-14 01:26:41 +0000
@@ -20,8 +20,10 @@
 from bzrlib import check, osutils
 from bzrlib.errors import PointlessCommit
 from bzrlib.tests import (
+    TestCaseWithTransport,
+    )
+from bzrlib.tests.features import (
     SymlinkFeature,
-    TestCaseWithTransport,
     )
 
 

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2011-06-22 13:53:20 +0000
+++ bzrlib/tests/test_config.py	2011-08-09 10:02:34 +0000
@@ -160,6 +160,7 @@
 editor=vim
 change_editor=vimdiff -of @new_path @old_path
 gpg_signing_command=gnome-gpg
+gpg_signing_key=DD4D5088
 log_format=short
 validate_signatures_in_log=true
 acceptable_keys=amy
@@ -214,6 +215,7 @@
 [/a/]
 check_signatures=check-available
 gpg_signing_command=false
+gpg_signing_key=default
 user_local_option=local
 # test trailing / matching
 [/a/*]
@@ -1225,6 +1227,10 @@
         self.assertEqual("gnome-gpg", my_config.gpg_signing_command())
         self.assertEqual(False, my_config.signature_needed())
 
+    def test_gpg_signing_key(self):
+        my_config = self._get_sample_config()
+        self.assertEqual("DD4D5088", my_config.gpg_signing_key())
+
     def _get_empty_config(self):
         my_config = config.GlobalConfig()
         return my_config
@@ -1520,6 +1526,14 @@
         self.get_branch_config('/a')
         self.assertEqual("false", self.my_config.gpg_signing_command())
 
+    def test_gpg_signing_key(self):
+        self.get_branch_config('/b')
+        self.assertEqual("DD4D5088", self.my_config.gpg_signing_key())
+
+    def test_gpg_signing_key_default(self):
+        self.get_branch_config('/a')
+        self.assertEqual("erik@bagfors.nu", self.my_config.gpg_signing_key())
+
     def test_get_user_option_global(self):
         self.get_branch_config('/a')
         self.assertEqual('something',
@@ -2205,26 +2219,31 @@
     def setUp(self):
         super(TestOptionRegistry, self).setUp()
         # Always start with an empty registry
-        self.overrideAttr(config, 'option_registry', registry.Registry())
+        self.overrideAttr(config, 'option_registry', config.OptionRegistry())
         self.registry = config.option_registry
 
     def test_register(self):
         opt = config.Option('foo')
-        self.registry.register('foo', opt)
+        self.registry.register(opt)
         self.assertIs(opt, self.registry.get('foo'))
 
-    lazy_option = config.Option('lazy_foo')
-
-    def test_register_lazy(self):
-        self.registry.register_lazy('foo', self.__module__,
-                                    'TestOptionRegistry.lazy_option')
-        self.assertIs(self.lazy_option, self.registry.get('foo'))
-
     def test_registered_help(self):
-        opt = config.Option('foo')
-        self.registry.register('foo', opt, help='A simple option')
+        opt = config.Option('foo', help='A simple option')
+        self.registry.register(opt)
         self.assertEquals('A simple option', self.registry.get_help('foo'))
 
+    lazy_option = config.Option('lazy_foo', help='Lazy help')
+
+    def test_register_lazy(self):
+        self.registry.register_lazy('lazy_foo', self.__module__,
+                                    'TestOptionRegistry.lazy_option')
+        self.assertIs(self.lazy_option, self.registry.get('lazy_foo'))
+
+    def test_registered_lazy_help(self):
+        self.registry.register_lazy('lazy_foo', self.__module__,
+                                    'TestOptionRegistry.lazy_option')
+        self.assertEquals('Lazy help', self.registry.get_help('lazy_foo'))
+
 
 class TestRegisteredOptions(tests.TestCase):
     """All registered options should verify some constraints."""
@@ -2244,8 +2263,9 @@
     def test_help_is_set(self):
         option_help = self.registry.get_help(self.option_name)
         self.assertNotEquals(None, option_help)
-        # Come on, think about the user, he really wants to know whst the
+        # Come on, think about the user, he really wants to know what the
         # option is about
+        self.assertIsNot(None, option_help)
         self.assertNotEquals('', option_help)
 
 
@@ -2852,6 +2872,9 @@
     # FIXME: This should be parametrized for all known Stack or dedicated
     # paramerized tests created to avoid bloating -- vila 2011-03-31
 
+    def overrideOptionRegistry(self):
+        self.overrideAttr(config, 'option_registry', config.OptionRegistry())
+
     def test_single_config_get(self):
         conf = dict(foo='bar')
         conf_stack = config.Stack([conf])
@@ -2860,21 +2883,21 @@
     def test_get_with_registered_default_value(self):
         conf_stack = config.Stack([dict()])
         opt = config.Option('foo', default='bar')
-        self.overrideAttr(config, 'option_registry', registry.Registry())
+        self.overrideOptionRegistry()
         config.option_registry.register('foo', opt)
         self.assertEquals('bar', conf_stack.get('foo'))
 
     def test_get_without_registered_default_value(self):
         conf_stack = config.Stack([dict()])
         opt = config.Option('foo')
-        self.overrideAttr(config, 'option_registry', registry.Registry())
+        self.overrideOptionRegistry()
         config.option_registry.register('foo', opt)
         self.assertEquals(None, conf_stack.get('foo'))
 
     def test_get_without_default_value_for_not_registered(self):
         conf_stack = config.Stack([dict()])
         opt = config.Option('foo')
-        self.overrideAttr(config, 'option_registry', registry.Registry())
+        self.overrideOptionRegistry()
         self.assertEquals(None, conf_stack.get('foo'))
 
     def test_get_first_definition(self):

=== modified file 'bzrlib/tests/test_conflicts.py'
--- bzrlib/tests/test_conflicts.py	2011-07-07 10:20:59 +0000
+++ bzrlib/tests/test_conflicts.py	2011-08-04 00:17:53 +0000
@@ -971,7 +971,7 @@
         if self._other['xfail']:
             # It's a bit hackish to raise from here relying on being called for
             # both tests but this avoid overriding test_resolve_taking_other
-            raise tests.KnownFailure(
+            self.knownFailure(
                 "ParentLoop doesn't carry enough info to resolve --take-other")
     _assert_conflict = assertParentLoop
 

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2011-05-18 16:11:05 +0000
+++ bzrlib/tests/test_diff.py	2011-07-21 06:46:34 +0000
@@ -35,27 +35,9 @@
 from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests import features, EncodingAdapter
 from bzrlib.tests.blackbox.test_diff import subst_dates
-
-
-class _AttribFeature(tests.Feature):
-
-    def _probe(self):
-        if (sys.platform not in ('cygwin', 'win32')):
-            return False
-        try:
-            proc = subprocess.Popen(['attrib', '.'], stdout=subprocess.PIPE)
-        except OSError, e:
-            return False
-        return (0 == proc.wait())
-
-    def feature_name(self):
-        return 'attrib Windows command-line tool'
-
-AttribFeature = _AttribFeature()
-
-
-compiled_patiencediff_feature = tests.ModuleAvailableFeature(
-                                    'bzrlib._patiencediff_c')
+from bzrlib.tests import (
+    features,
+    )
 
 
 def udiff_lines(old, new, allow_binary=False):
@@ -514,7 +496,7 @@
         is a binary file in the diff.
         """
         # See https://bugs.launchpad.net/bugs/110092.
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
 
         # This bug isn't triggered with cStringIO.
         from StringIO import StringIO
@@ -539,7 +521,7 @@
 
     def test_unicode_filename(self):
         """Test when the filename are unicode."""
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
 
         alpha, omega = u'\u03b1', u'\u03c9'
         autf8, outf8 = alpha.encode('utf8'), omega.encode('utf8')
@@ -571,7 +553,7 @@
         """Test for bug #382699: unicode filenames on Windows should be shown
         in user encoding.
         """
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         # The word 'test' in Russian
         _russian_test = u'\u0422\u0435\u0441\u0442'
         directory = _russian_test + u'/'
@@ -708,7 +690,7 @@
              ' \@\@\n-old\n\+new\n\n')
 
     def test_diff_kind_change(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.build_tree_contents([('old-tree/olddir/',),
                                   ('old-tree/olddir/oldfile', 'old\n')])
         self.old_tree.add('olddir')
@@ -1172,7 +1154,7 @@
 
 class TestPatienceDiffLib_c(TestPatienceDiffLib):
 
-    _test_needs_features = [compiled_patiencediff_feature]
+    _test_needs_features = [features.compiled_patiencediff_feature]
 
     def setUp(self):
         super(TestPatienceDiffLib_c, self).setUp()
@@ -1268,7 +1250,7 @@
 
 class TestPatienceDiffLibFiles_c(TestPatienceDiffLibFiles):
 
-    _test_needs_features = [compiled_patiencediff_feature]
+    _test_needs_features = [features.compiled_patiencediff_feature]
 
     def setUp(self):
         super(TestPatienceDiffLibFiles_c, self).setUp()
@@ -1280,7 +1262,7 @@
 class TestUsingCompiledIfAvailable(tests.TestCase):
 
     def test_PatienceSequenceMatcher(self):
-        if compiled_patiencediff_feature.available():
+        if features.compiled_patiencediff_feature.available():
             from bzrlib._patiencediff_c import PatienceSequenceMatcher_c
             self.assertIs(PatienceSequenceMatcher_c,
                           patiencediff.PatienceSequenceMatcher)
@@ -1290,7 +1272,7 @@
                           patiencediff.PatienceSequenceMatcher)
 
     def test_unique_lcs(self):
-        if compiled_patiencediff_feature.available():
+        if features.compiled_patiencediff_feature.available():
             from bzrlib._patiencediff_c import unique_lcs_c
             self.assertIs(unique_lcs_c,
                           patiencediff.unique_lcs)
@@ -1300,7 +1282,7 @@
                           patiencediff.unique_lcs)
 
     def test_recurse_matches(self):
-        if compiled_patiencediff_feature.available():
+        if features.compiled_patiencediff_feature.available():
             from bzrlib._patiencediff_c import recurse_matches_c
             self.assertIs(recurse_matches_c,
                           patiencediff.recurse_matches)
@@ -1356,7 +1338,7 @@
                          ' on this machine', str(e))
 
     def test_prepare_files_creates_paths_readable_by_windows_tool(self):
-        self.requireFeature(AttribFeature)
+        self.requireFeature(features.AttribFeature)
         output = StringIO()
         tree = self.make_branch_and_tree('tree')
         self.build_tree_contents([('tree/file', 'content')])
@@ -1470,9 +1452,7 @@
 class TestGetTreesAndBranchesToDiffLocked(tests.TestCaseWithTransport):
 
     def call_gtabtd(self, path_list, revision_specs, old_url, new_url):
-        """Call get_trees_and_branches_to_diff_locked.  Overridden by
-        TestGetTreesAndBranchesToDiff.
-        """
+        """Call get_trees_and_branches_to_diff_locked."""
         return diff.get_trees_and_branches_to_diff_locked(
             path_list, revision_specs, old_url, new_url, self.addCleanup)
 
@@ -1515,14 +1495,3 @@
         self.assertEqual(tree.branch.base, new_branch.base)
         self.assertIs(None, specific_files)
         self.assertEqual(tree.basedir, extra_trees[0].basedir)
-
-
-class TestGetTreesAndBranchesToDiff(TestGetTreesAndBranchesToDiffLocked):
-    """Apply the tests for get_trees_and_branches_to_diff_locked to the
-    deprecated get_trees_and_branches_to_diff function.
-    """
-
-    def call_gtabtd(self, path_list, revision_specs, old_url, new_url):
-        return self.applyDeprecated(
-            deprecated_in((2, 2, 0)), diff.get_trees_and_branches_to_diff,
-            path_list, revision_specs, old_url, new_url)

=== modified file 'bzrlib/tests/test_dirstate.py'
--- bzrlib/tests/test_dirstate.py	2011-05-25 13:44:28 +0000
+++ bzrlib/tests/test_dirstate.py	2011-06-14 01:26:41 +0000
@@ -32,7 +32,10 @@
     workingtree_4,
     )
 from bzrlib.transport import memory
-from bzrlib.tests import test_osutils
+from bzrlib.tests import (
+    features,
+    test_osutils,
+    )
 from bzrlib.tests.scenarios import load_tests_apply_scenarios
 
 
@@ -1209,7 +1212,7 @@
         # The most trivial addition of a symlink when there are no parents and
         # its in the root and all data about the file is supplied
         # bzr doesn't support fake symlinks on windows, yet.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         os.symlink(target, link_name)
         stat = os.lstat(link_name)
         expected_entries = [
@@ -1240,7 +1243,7 @@
         self._test_add_symlink_to_root_no_parents_all_data('a link', 'target')
 
     def test_add_symlink_unicode_to_root_no_parents_all_data(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_add_symlink_to_root_no_parents_all_data(
             u'\N{Euro Sign}link', u'targ\N{Euro Sign}et')
 

=== modified file 'bzrlib/tests/test_export.py'
--- bzrlib/tests/test_export.py	2011-06-28 13:55:39 +0000
+++ bzrlib/tests/test_export.py	2011-07-08 23:01:39 +0000
@@ -49,7 +49,7 @@
         self.assertEquals([], os.listdir("target"))
 
     def test_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         wt = self.make_branch_and_tree('.')
         os.symlink('source', 'link')
         wt.add(['link'])

=== added file 'bzrlib/tests/test_features.py'
--- bzrlib/tests/test_features.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_features.py	2011-07-11 06:47:32 +0000
@@ -0,0 +1,115 @@
+# Copyright (C) 2005-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for test feature dependencies.
+"""
+
+from bzrlib import (
+    symbol_versioning,
+    tests,
+    )
+from bzrlib.tests import (
+    features,
+    )
+
+
+class TestFeature(tests.TestCase):
+
+    def test_caching(self):
+        """Feature._probe is called by the feature at most once."""
+        class InstrumentedFeature(features.Feature):
+            def __init__(self):
+                super(InstrumentedFeature, self).__init__()
+                self.calls = []
+
+            def _probe(self):
+                self.calls.append('_probe')
+                return False
+        feature = InstrumentedFeature()
+        feature.available()
+        self.assertEqual(['_probe'], feature.calls)
+        feature.available()
+        self.assertEqual(['_probe'], feature.calls)
+
+    def test_named_str(self):
+        """Feature.__str__ should thunk to feature_name()."""
+        class NamedFeature(features.Feature):
+            def feature_name(self):
+                return 'symlinks'
+        feature = NamedFeature()
+        self.assertEqual('symlinks', str(feature))
+
+    def test_default_str(self):
+        """Feature.__str__ should default to __class__.__name__."""
+        class NamedFeature(features.Feature):
+            pass
+        feature = NamedFeature()
+        self.assertEqual('NamedFeature', str(feature))
+
+
+class TestUnavailableFeature(tests.TestCase):
+
+    def test_access_feature(self):
+        feature = features.Feature()
+        exception = tests.UnavailableFeature(feature)
+        self.assertIs(feature, exception.args[0])
+
+
+# Although this was deprecated a long time ago, please keep it here because
+# it's really just a test fixture for test-feature deprecation.
+simple_thunk_feature = features._CompatabilityThunkFeature(
+    symbol_versioning.deprecated_in((2, 1, 0)),
+    'bzrlib.tests.test_features',
+    'simple_thunk_feature',
+    'UnicodeFilenameFeature',
+    replacement_module='bzrlib.tests.features')
+
+
+class Test_CompatibilityFeature(tests.TestCase):
+
+    def test_does_thunk(self):
+        res = self.callDeprecated(
+            ['bzrlib.tests.test_features.simple_thunk_feature '
+             'was deprecated in version 2.1.0. '
+             'Use bzrlib.tests.features.UnicodeFilenameFeature instead.'],
+            simple_thunk_feature.available)
+        self.assertEqual(features.UnicodeFilenameFeature.available(), res)
+
+
+class TestModuleAvailableFeature(tests.TestCase):
+
+    def test_available_module(self):
+        feature = features.ModuleAvailableFeature('bzrlib.tests')
+        self.assertEqual('bzrlib.tests', feature.module_name)
+        self.assertEqual('bzrlib.tests', str(feature))
+        self.assertTrue(feature.available())
+        self.assertIs(tests, feature.module)
+
+    def test_unavailable_module(self):
+        feature = features.ModuleAvailableFeature(
+            'bzrlib.no_such_module_exists')
+        self.assertEqual('bzrlib.no_such_module_exists', str(feature))
+        self.assertFalse(feature.available())
+        self.assertIs(None, feature.module)
+
+
+class TestUnicodeFilenameFeature(tests.TestCase):
+
+    def test_probe_passes(self):
+        """UnicodeFilenameFeature._probe passes."""
+        # We can't test much more than that because the behaviour depends
+        # on the platform.
+        features.UnicodeFilenameFeature._probe()

=== added file 'bzrlib/tests/test_filter_tree.py'
--- bzrlib/tests/test_filter_tree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_filter_tree.py	2011-07-04 21:15:24 +0000
@@ -0,0 +1,68 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for ContentFilterTree"""
+
+import tarfile
+import zipfile
+
+from bzrlib import (
+    export,
+    filter_tree,
+    tests,
+    )
+from bzrlib.tests import (
+    fixtures,
+    )
+from bzrlib.tests.test_filters import _stack_1
+
+
+class TestFilterTree(tests.TestCaseWithTransport):
+
+    def make_tree(self):
+        self.underlying_tree = fixtures.make_branch_and_populated_tree(
+            self)
+        def stack_callback(path):
+            return _stack_1
+        self.filter_tree = filter_tree.ContentFilterTree(
+            self.underlying_tree, stack_callback)
+        return self.filter_tree
+
+    def test_get_file_text(self):
+        self.make_tree()
+        self.assertEquals(
+            self.underlying_tree.get_file_text('hello-id'),
+            'hello world')
+        self.assertEquals(
+            self.filter_tree.get_file_text('hello-id'),
+            'HELLO WORLD')
+
+    def test_tar_export_content_filter_tree(self):
+        # TODO: this could usefully be run generically across all exporters.
+        self.make_tree()
+        export.export(self.filter_tree, "out.tgz")
+        ball = tarfile.open("out.tgz", "r:gz")
+        self.assertEquals(
+            'HELLO WORLD',
+            ball.extractfile('out/hello').read())
+
+    def test_zip_export_content_filter_tree(self):
+        self.make_tree()
+        export.export(self.filter_tree, 'out.zip')
+        zipf = zipfile.ZipFile('out.zip', 'r')
+        self.assertEquals(
+            'HELLO WORLD',
+            zipf.read('out/hello'))

=== modified file 'bzrlib/tests/test_gpg.py'
--- bzrlib/tests/test_gpg.py	2011-06-30 15:10:10 +0000
+++ bzrlib/tests/test_gpg.py	2011-07-12 13:20:12 +0000
@@ -27,6 +27,9 @@
 
 class FakeConfig(object):
 
+    def gpg_signing_key(self):
+        return "amy@example.com"
+
     def gpg_signing_command(self):
         return "false"
 
@@ -38,7 +41,7 @@
 
     def test_signing_command_line(self):
         my_gpg = gpg.GPGStrategy(FakeConfig())
-        self.assertEqual(['false',  '--clearsign'],
+        self.assertEqual(['false',  '--clearsign', '-u', 'amy@example.com'],
                          my_gpg._command_line())
 
     def test_checks_return_code(self):

=== modified file 'bzrlib/tests/test_hashcache.py'
--- bzrlib/tests/test_hashcache.py	2011-05-11 14:26:41 +0000
+++ bzrlib/tests/test_hashcache.py	2011-06-14 01:26:41 +0000
@@ -22,8 +22,10 @@
 from bzrlib.errors import BzrError
 from bzrlib.hashcache import HashCache
 from bzrlib.tests import (
+    TestCaseInTempDir,
+    )
+from bzrlib.tests.features import (
     OsFifoFeature,
-    TestCaseInTempDir,
     )
 
 

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2011-08-01 06:24:48 +0000
+++ bzrlib/tests/test_http.py	2011-08-04 00:17:53 +0000
@@ -128,7 +128,7 @@
         ('urllib,http', dict(_activity_server=ActivityHTTPServer,
                             _transport=_urllib.HttpTransport_urllib,)),
         ]
-    if tests.HTTPSServerFeature.available():
+    if features.HTTPSServerFeature.available():
         activity_scenarios.append(
             ('urllib,https', dict(_activity_server=ActivityHTTPSServer,
                                 _transport=_urllib.HttpTransport_urllib,)),)
@@ -136,7 +136,7 @@
         activity_scenarios.append(
             ('pycurl,http', dict(_activity_server=ActivityHTTPServer,
                                 _transport=PyCurlTransport,)),)
-        if tests.HTTPSServerFeature.available():
+        if features.HTTPSServerFeature.available():
             from bzrlib.tests import (
                 ssl_certs,
                 )
@@ -1767,7 +1767,7 @@
                                      http_utils.ProxyDigestAuthServer):
             raise tests.TestNotApplicable('HTTP/proxy auth digest only test')
         if self._testing_pycurl():
-            raise tests.KnownFailure(
+            self.knownFailure(
                 'pycurl does not handle a nonce change')
         self.server.add_user('joe', 'foo')
         t = self.get_user_transport('joe', 'foo')
@@ -1855,7 +1855,7 @@
         if self._testing_pycurl():
             import pycurl
             if pycurl.version_info()[1] < '7.16.0':
-                raise tests.KnownFailure(
+                self.knownFailure(
                     'pycurl < 7.16.0 does not handle empty proxy passwords')
         super(TestProxyAuth, self).test_empty_pass()
 
@@ -2086,7 +2086,7 @@
     pass
 
 
-if tests.HTTPSServerFeature.available():
+if features.HTTPSServerFeature.available():
     from bzrlib.tests import https_server
     class ActivityHTTPSServer(ActivityServerMixin, https_server.HTTPSServer):
         pass

=== modified file 'bzrlib/tests/test_i18n.py'
--- bzrlib/tests/test_i18n.py	2011-06-27 15:52:06 +0000
+++ bzrlib/tests/test_i18n.py	2011-07-14 22:14:52 +0000
@@ -19,8 +19,6 @@
 from bzrlib import i18n, tests
 
 
-
-
 class ZzzTranslations(object):
     """Special Zzz translation for debugging i18n stuff.
 
@@ -91,3 +89,18 @@
     def test_multiline(self):
         self.assertEqual(u"zz{{spam\nham}}\n\nzz{{eggs\n}}",
                          i18n.gettext_per_paragraph("spam\nham\n\neggs\n"))
+
+
+class TestInstall(tests.TestCase):
+
+    def test_custom_languages(self):
+        self.addCleanup(i18n.install)
+        i18n.install('nl:fy')
+
+    def test_no_env_variables(self):
+        self.addCleanup(i18n.install)
+        self.overrideEnv('LANGUAGE', None)
+        self.overrideEnv('LC_ALL', None)
+        self.overrideEnv('LC_MESSAGES', None)
+        self.overrideEnv('LANG', None)
+        i18n.install()

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2011-05-26 08:05:45 +0000
+++ bzrlib/tests/test_knit.py	2011-06-14 01:26:41 +0000
@@ -62,10 +62,13 @@
     network_bytes_to_kind_and_offset,
     RecordingVersionedFilesDecorator,
     )
-
-
-compiled_knit_feature = tests.ModuleAvailableFeature(
-                            'bzrlib._knit_load_data_pyx')
+from bzrlib.tests import (
+    features,
+    )
+
+
+compiled_knit_feature = features.ModuleAvailableFeature(
+    'bzrlib._knit_load_data_pyx')
 
 
 class KnitContentTestsMixin(object):

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2011-06-10 07:31:54 +0000
+++ bzrlib/tests/test_lockdir.py	2011-08-04 00:05:53 +0000
@@ -686,9 +686,6 @@
         """Detect that the holder (this process) is still running."""
         info = LockHeldInfo.for_this_process(None)
         info.info_dict['pid'] = '123123123'
-        if sys.platform == 'win32':
-            self.knownFailure(
-                'live lock holder detection not implemented yet on win32')
         self.assertTrue(info.is_lock_holder_known_dead())
 
     def test_lock_holder_other_machine(self):

=== modified file 'bzrlib/tests/test_log.py'
--- bzrlib/tests/test_log.py	2011-04-08 03:47:34 +0000
+++ bzrlib/tests/test_log.py	2011-07-25 07:41:52 +0000
@@ -983,349 +983,6 @@
             wt.branch, log.GnuChangelogLogFormatter,
             show_log_kwargs=dict(verbose=True))
 
-class TestGetViewRevisions(tests.TestCaseWithTransport, TestLogMixin):
-
-    def _get_view_revisions(self, *args, **kwargs):
-        return self.applyDeprecated(symbol_versioning.deprecated_in((2, 2, 0)),
-                                    log.get_view_revisions, *args, **kwargs)
-
-    def make_tree_with_commits(self):
-        """Create a tree with well-known revision ids"""
-        wt = self.make_branch_and_tree('tree1')
-        self.wt_commit(wt, 'commit one', rev_id='1')
-        self.wt_commit(wt, 'commit two', rev_id='2')
-        self.wt_commit(wt, 'commit three', rev_id='3')
-        mainline_revs = [None, '1', '2', '3']
-        rev_nos = {'1': 1, '2': 2, '3': 3}
-        return mainline_revs, rev_nos, wt
-
-    def make_tree_with_merges(self):
-        """Create a tree with well-known revision ids and a merge"""
-        mainline_revs, rev_nos, wt = self.make_tree_with_commits()
-        tree2 = wt.bzrdir.sprout('tree2').open_workingtree()
-        self.wt_commit(tree2, 'four-a', rev_id='4a')
-        wt.merge_from_branch(tree2.branch)
-        self.wt_commit(wt, 'four-b', rev_id='4b')
-        mainline_revs.append('4b')
-        rev_nos['4b'] = 4
-        # 4a: 3.1.1
-        return mainline_revs, rev_nos, wt
-
-    def make_branch_with_many_merges(self):
-        """Create a tree with well-known revision ids"""
-        builder = self.make_branch_builder('tree1')
-        builder.start_series()
-        builder.build_snapshot('1', None, [
-            ('add', ('', 'TREE_ROOT', 'directory', '')),
-            ('add', ('f', 'f-id', 'file', '1\n'))])
-        builder.build_snapshot('2', ['1'], [])
-        builder.build_snapshot('3a', ['2'], [
-            ('modify', ('f-id', '1\n2\n3a\n'))])
-        builder.build_snapshot('3b', ['2', '3a'], [
-            ('modify', ('f-id', '1\n2\n3a\n'))])
-        builder.build_snapshot('3c', ['2', '3b'], [
-            ('modify', ('f-id', '1\n2\n3a\n'))])
-        builder.build_snapshot('4a', ['3b'], [])
-        builder.build_snapshot('4b', ['3c', '4a'], [])
-        builder.finish_series()
-
-        # 1
-        # |
-        # 2-.
-        # |\ \
-        # | | 3a
-        # | |/
-        # | 3b
-        # |/|
-        # 3c4a
-        # |/
-        # 4b
-
-        mainline_revs = [None, '1', '2', '3c', '4b']
-        rev_nos = {'1':1, '2':2, '3c': 3, '4b':4}
-        full_rev_nos_for_reference = {
-            '1': '1',
-            '2': '2',
-            '3a': '2.1.1', #first commit tree 3
-            '3b': '2.2.1', # first commit tree 2
-            '3c': '3', #merges 3b to main
-            '4a': '2.2.2', # second commit tree 2
-            '4b': '4', # merges 4a to main
-            }
-        return mainline_revs, rev_nos, builder.get_branch()
-
-    def test_get_view_revisions_forward(self):
-        """Test the get_view_revisions method"""
-        mainline_revs, rev_nos, wt = self.make_tree_with_commits()
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'forward'))
-        self.assertEqual([('1', '1', 0), ('2', '2', 0), ('3', '3', 0)],
-                         revisions)
-        revisions2 = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'forward',
-                include_merges=False))
-        self.assertEqual(revisions, revisions2)
-
-    def test_get_view_revisions_reverse(self):
-        """Test the get_view_revisions with reverse"""
-        mainline_revs, rev_nos, wt = self.make_tree_with_commits()
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'reverse'))
-        self.assertEqual([('3', '3', 0), ('2', '2', 0), ('1', '1', 0), ],
-                         revisions)
-        revisions2 = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'reverse',
-                include_merges=False))
-        self.assertEqual(revisions, revisions2)
-
-    def test_get_view_revisions_merge(self):
-        """Test get_view_revisions when there are merges"""
-        mainline_revs, rev_nos, wt = self.make_tree_with_merges()
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'forward'))
-        self.assertEqual([('1', '1', 0), ('2', '2', 0), ('3', '3', 0),
-                          ('4b', '4', 0), ('4a', '3.1.1', 1)],
-                         revisions)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'forward',
-                include_merges=False))
-        self.assertEqual([('1', '1', 0), ('2', '2', 0), ('3', '3', 0),
-                          ('4b', '4', 0)],
-                         revisions)
-
-    def test_get_view_revisions_merge_reverse(self):
-        """Test get_view_revisions in reverse when there are merges"""
-        mainline_revs, rev_nos, wt = self.make_tree_with_merges()
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'reverse'))
-        self.assertEqual([('4b', '4', 0), ('4a', '3.1.1', 1),
-                          ('3', '3', 0), ('2', '2', 0), ('1', '1', 0)],
-                         revisions)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'reverse',
-                include_merges=False))
-        self.assertEqual([('4b', '4', 0), ('3', '3', 0), ('2', '2', 0),
-                          ('1', '1', 0)],
-                         revisions)
-
-    def test_get_view_revisions_merge2(self):
-        """Test get_view_revisions when there are merges"""
-        mainline_revs, rev_nos, b = self.make_branch_with_many_merges()
-        b.lock_read()
-        self.addCleanup(b.unlock)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, b, 'forward'))
-        expected = [('1', '1', 0), ('2', '2', 0), ('3c', '3', 0),
-                    ('3b', '2.2.1', 1), ('3a', '2.1.1', 2), ('4b', '4', 0),
-                    ('4a', '2.2.2', 1)]
-        self.assertEqual(expected, revisions)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, b, 'forward',
-                include_merges=False))
-        self.assertEqual([('1', '1', 0), ('2', '2', 0), ('3c', '3', 0),
-                          ('4b', '4', 0)],
-                         revisions)
-
-    def test_file_id_for_range(self):
-        mainline_revs, rev_nos, b = self.make_branch_with_many_merges()
-        b.lock_read()
-        self.addCleanup(b.unlock)
-
-        def rev_from_rev_id(revid, branch):
-            revspec = revisionspec.RevisionSpec.from_string('revid:%s' % revid)
-            return revspec.in_history(branch)
-
-        def view_revs(start_rev, end_rev, file_id, direction):
-            revs = self.applyDeprecated(
-                symbol_versioning.deprecated_in((2, 2, 0)),
-                log.calculate_view_revisions,
-                b,
-                start_rev, # start_revision
-                end_rev, # end_revision
-                direction, # direction
-                file_id, # specific_fileid
-                True, # generate_merge_revisions
-                )
-            return revs
-
-        rev_3a = rev_from_rev_id('3a', b)
-        rev_4b = rev_from_rev_id('4b', b)
-        self.assertEqual([('3c', '3', 0), ('3b', '2.2.1', 1),
-                          ('3a', '2.1.1', 2)],
-                          view_revs(rev_3a, rev_4b, 'f-id', 'reverse'))
-        # Note: 3c still appears before 3a here because of depth-based sorting
-        self.assertEqual([('3c', '3', 0), ('3b', '2.2.1', 1),
-                          ('3a', '2.1.1', 2)],
-                          view_revs(rev_3a, rev_4b, 'f-id', 'forward'))
-
-
-class TestGetRevisionsTouchingFileID(tests.TestCaseWithTransport):
-
-    def get_view_revisions(self, *args):
-        return self.applyDeprecated(symbol_versioning.deprecated_in((2, 2, 0)),
-                                    log.get_view_revisions, *args)
-
-    def create_tree_with_single_merge(self):
-        """Create a branch with a moderate layout.
-
-        The revision graph looks like:
-
-           A
-           |\
-           B C
-           |/
-           D
-
-        In this graph, A introduced files f1 and f2 and f3.
-        B modifies f1 and f3, and C modifies f2 and f3.
-        D merges the changes from B and C and resolves the conflict for f3.
-        """
-        # TODO: jam 20070218 This seems like it could really be done
-        #       with make_branch_and_memory_tree() if we could just
-        #       create the content of those files.
-        # TODO: jam 20070218 Another alternative is that we would really
-        #       like to only create this tree 1 time for all tests that
-        #       use it. Since 'log' only uses the tree in a readonly
-        #       fashion, it seems a shame to regenerate an identical
-        #       tree for each test.
-        # TODO: vila 20100122 One way to address the shame above will be to
-        #       create a memory tree during test parametrization and give a
-        #       *copy* of this tree to each test. Copying a memory tree ought
-        #       to be cheap, at least cheaper than creating them with such
-        #       complex setups.
-        tree = self.make_branch_and_tree('tree')
-        tree.lock_write()
-        self.addCleanup(tree.unlock)
-
-        self.build_tree_contents([('tree/f1', 'A\n'),
-                                  ('tree/f2', 'A\n'),
-                                  ('tree/f3', 'A\n'),
-                                 ])
-        tree.add(['f1', 'f2', 'f3'], ['f1-id', 'f2-id', 'f3-id'])
-        tree.commit('A', rev_id='A')
-
-        self.build_tree_contents([('tree/f2', 'A\nC\n'),
-                                  ('tree/f3', 'A\nC\n'),
-                                 ])
-        tree.commit('C', rev_id='C')
-        # Revert back to A to build the other history.
-        tree.set_last_revision('A')
-        tree.branch.set_last_revision_info(1, 'A')
-        self.build_tree_contents([('tree/f1', 'A\nB\n'),
-                                  ('tree/f2', 'A\n'),
-                                  ('tree/f3', 'A\nB\n'),
-                                 ])
-        tree.commit('B', rev_id='B')
-        tree.set_parent_ids(['B', 'C'])
-        self.build_tree_contents([('tree/f1', 'A\nB\n'),
-                                  ('tree/f2', 'A\nC\n'),
-                                  ('tree/f3', 'A\nB\nC\n'),
-                                 ])
-        tree.commit('D', rev_id='D')
-
-        # Switch to a read lock for this tree.
-        # We still have an addCleanup(tree.unlock) pending
-        tree.unlock()
-        tree.lock_read()
-        return tree
-
-    def check_delta(self, delta, **kw):
-        """Check the filenames touched by a delta are as expected.
-
-        Caller only have to pass in the list of files for each part, all
-        unspecified parts are considered empty (and checked as such).
-        """
-        for n in 'added', 'removed', 'renamed', 'modified', 'unchanged':
-            # By default we expect an empty list
-            expected = kw.get(n, [])
-            # strip out only the path components
-            got = [x[0] for x in getattr(delta, n)]
-            self.assertEqual(expected, got)
-
-    def test_tree_with_single_merge(self):
-        """Make sure the tree layout is correct."""
-        tree = self.create_tree_with_single_merge()
-        rev_A_tree = tree.branch.repository.revision_tree('A')
-        rev_B_tree = tree.branch.repository.revision_tree('B')
-        rev_C_tree = tree.branch.repository.revision_tree('C')
-        rev_D_tree = tree.branch.repository.revision_tree('D')
-
-        self.check_delta(rev_B_tree.changes_from(rev_A_tree),
-                         modified=['f1', 'f3'])
-
-        self.check_delta(rev_C_tree.changes_from(rev_A_tree),
-                         modified=['f2', 'f3'])
-
-        self.check_delta(rev_D_tree.changes_from(rev_B_tree),
-                         modified=['f2', 'f3'])
-
-        self.check_delta(rev_D_tree.changes_from(rev_C_tree),
-                         modified=['f1', 'f3'])
-
-    def assertAllRevisionsForFileID(self, tree, file_id, revisions):
-        """Ensure _filter_revisions_touching_file_id returns the right values.
-
-        Get the return value from _filter_revisions_touching_file_id and make
-        sure they are correct.
-        """
-        # The api for _filter_revisions_touching_file_id is a little crazy.
-        # So we do the setup here.
-        mainline = tree.branch.revision_history()
-        mainline.insert(0, None)
-        revnos = dict((rev, idx+1) for idx, rev in enumerate(mainline))
-        view_revs_iter = self.get_view_revisions(
-            mainline, revnos, tree.branch, 'reverse', True)
-        actual_revs = log._filter_revisions_touching_file_id(
-            tree.branch, file_id, list(view_revs_iter))
-        self.assertEqual(revisions, [r for r, revno, depth in actual_revs])
-
-    def test_file_id_f1(self):
-        tree = self.create_tree_with_single_merge()
-        # f1 should be marked as modified by revisions A and B
-        self.assertAllRevisionsForFileID(tree, 'f1-id', ['B', 'A'])
-
-    def test_file_id_f2(self):
-        tree = self.create_tree_with_single_merge()
-        # f2 should be marked as modified by revisions A, C, and D
-        # because D merged the changes from C.
-        self.assertAllRevisionsForFileID(tree, 'f2-id', ['D', 'C', 'A'])
-
-    def test_file_id_f3(self):
-        tree = self.create_tree_with_single_merge()
-        # f3 should be marked as modified by revisions A, B, C, and D
-        self.assertAllRevisionsForFileID(tree, 'f3-id', ['D', 'C', 'B', 'A'])
-
-    def test_file_id_with_ghosts(self):
-        # This is testing bug #209948, where having a ghost would cause
-        # _filter_revisions_touching_file_id() to fail.
-        tree = self.create_tree_with_single_merge()
-        # We need to add a revision, so switch back to a write-locked tree
-        # (still a single addCleanup(tree.unlock) pending).
-        tree.unlock()
-        tree.lock_write()
-        first_parent = tree.last_revision()
-        tree.set_parent_ids([first_parent, 'ghost-revision-id'])
-        self.build_tree_contents([('tree/f1', 'A\nB\nXX\n')])
-        tree.commit('commit with a ghost', rev_id='XX')
-        self.assertAllRevisionsForFileID(tree, 'f1-id', ['XX', 'B', 'A'])
-        self.assertAllRevisionsForFileID(tree, 'f2-id', ['D', 'C', 'A'])
-
-    def test_unknown_file_id(self):
-        tree = self.create_tree_with_single_merge()
-        self.assertAllRevisionsForFileID(tree, 'unknown', [])
-
-    def test_empty_branch_unknown_file_id(self):
-        tree = self.make_branch_and_tree('tree')
-        self.assertAllRevisionsForFileID(tree, 'unknown', [])
-
 
 class TestShowChangedRevisions(tests.TestCaseWithTransport):
 
@@ -1880,7 +1537,7 @@
     def make_branch_with_alternate_ancestries(self, relpath='.'):
         # See test_merge_sorted_exclude_ancestry below for the difference with
         # bt.per_branch.test_iter_merge_sorted_revision.
-        # TestIterMergeSortedRevisionsBushyGraph. 
+        # TestIterMergeSortedRevisionsBushyGraph.
         # make_branch_with_alternate_ancestries
         # and test_merge_sorted_exclude_ancestry
         # See the FIXME in assertLogRevnos too.
@@ -1941,3 +1598,41 @@
                              b, '1', '3', exclude_common_ancestry=True,
                              generate_merge_revisions=True)
 
+
+class TestLogDefaults(TestCaseForLogFormatter):
+    def test_default_log_level(self):
+        """
+        Test to ensure that specifying 'levels=1' to make_log_request_dict
+        doesn't get overwritten when using a LogFormatter that supports more
+        detail.
+        Fixes bug #747958.
+        """
+        wt = self._prepare_tree_with_merges()
+        b = wt.branch
+
+        class CustomLogFormatter(log.LogFormatter):
+            def __init__(self, *args, **kwargs):
+                super(CustomLogFormatter, self).__init__(*args, **kwargs)
+                self.revisions = []
+            def get_levels(self):
+                # log formatter supports all levels:
+                return 0
+            def log_revision(self, revision):
+                self.revisions.append(revision)
+
+        log_formatter = LogCatcher()
+        # First request we don't specify number of levels, we should get a
+        # sensible default (whatever the LogFormatter handles - which in this
+        # case is 0/everything):
+        request = log.make_log_request_dict(limit=10)
+        log.Logger(b, request).show(log_formatter)
+        # should have all three revisions:
+        self.assertEquals(len(log_formatter.revisions), 3)
+
+        del log_formatter
+        log_formatter = LogCatcher()
+        # now explicitly request mainline revisions only:
+        request = log.make_log_request_dict(limit=10, levels=1)
+        log.Logger(b, request).show(log_formatter)
+        # should now only have 2 revisions:
+        self.assertEquals(len(log_formatter.revisions), 2)

=== modified file 'bzrlib/tests/test_lsprof.py'
--- bzrlib/tests/test_lsprof.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_lsprof.py	2011-06-14 01:26:41 +0000
@@ -22,20 +22,9 @@
 
 import bzrlib
 from bzrlib import errors, tests
-
-
-class _LSProfFeature(tests.Feature):
-
-    def available(self):
-        try:
-            from bzrlib import lsprof
-        except ImportError:
-            return False
-        else:
-            return True
-
-
-LSProfFeature = _LSProfFeature()
+from bzrlib.tests import (
+    features,
+    )
 
 
 _TXT_HEADER = "   CallCount    Recursive    Total(ms)   " + \
@@ -56,7 +45,7 @@
 
 class TestStatsSave(tests.TestCaseInTempDir):
 
-    _test_needs_features = [LSProfFeature]
+    _test_needs_features = [features.lsprof_feature]
 
     def setUp(self):
         super(tests.TestCaseInTempDir, self).setUp()
@@ -96,7 +85,7 @@
 
 class TestBzrProfiler(tests.TestCase):
 
-    _test_needs_features = [LSProfFeature]
+    _test_needs_features = [features.lsprof_feature]
 
     def test_start_call_stuff_stop(self):
         profiler = bzrlib.lsprof.BzrProfiler()

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2011-07-12 23:32:24 +0000
+++ bzrlib/tests/test_merge.py	2011-07-21 06:46:34 +0000
@@ -37,6 +37,7 @@
 from bzrlib.merge import transform_tree, merge_inner, _PlanMerge
 from bzrlib.osutils import basename, pathjoin, file_kind
 from bzrlib.tests import (
+    features,
     TestCaseWithMemoryTransport,
     TestCaseWithTransport,
     test_merge_core,
@@ -137,26 +138,15 @@
 
     def test_merge_unrelated_retains_root(self):
         wt = self.make_branch_and_tree('tree')
-        root_id_before_merge = wt.get_root_id()
-        other_tree = self.make_branch_and_tree('other')
-        # Do a commit so there is something to merge
-        other_tree.commit('commit other')
-        self.assertNotEquals(root_id_before_merge, other_tree.get_root_id())
-        wt.merge_from_branch(other_tree.branch,
-                             from_revision=_mod_revision.NULL_REVISION)
-        self.assertEqual(root_id_before_merge, wt.get_root_id())
-
-    def test_merge_preview_unrelated_retains_root(self):
-        wt = self.make_branch_and_tree('tree')
-        other_tree = self.make_branch_and_tree('other')
-        # Do a commit so there is something to merge
-        other_tree.commit('commit other')
+        other_tree = self.make_branch_and_tree('other')
+        self.addCleanup(other_tree.lock_read().unlock)
         merger = _mod_merge.Merge3Merger(wt, wt, wt.basis_tree(), other_tree,
                                          this_branch=wt.branch,
                                          do_merge=False)
-        with merger.make_preview_transform() as tt:
-            preview = tt.get_preview_tree()
-            self.assertEqual(wt.get_root_id(), preview.get_root_id())
+        with transform.TransformPreview(wt) as merger.tt:
+            merger._compute_transform()
+            new_root_id = merger.tt.final_file_id(merger.tt.root)
+            self.assertEqual(wt.get_root_id(), new_root_id)
 
     def test_create_rename(self):
         """Rename an inventory entry while creating the file"""
@@ -2195,7 +2185,7 @@
         self.assertTrue(wt.is_executable('foo-id'))
 
     def test_create_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         #   A
         #  / \
         # B   C
@@ -2260,7 +2250,7 @@
                              wt.get_file_text('foo-id'))
 
     def test_modified_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         #   A       Create symlink foo => bar
         #  / \
         # B   C     B relinks foo => baz
@@ -2305,7 +2295,7 @@
         self.assertEqual('bing', wt.get_symlink_target('foo-id'))
 
     def test_renamed_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         #   A       Create symlink foo => bar
         #  / \
         # B   C     B renames foo => barry
@@ -2361,7 +2351,7 @@
         self.assertEqual('blah', wt.id2path('foo-id'))
 
     def test_symlink_no_content_change(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         #   A       Create symlink foo => bar
         #  / \
         # B   C     B relinks foo => baz
@@ -2412,7 +2402,7 @@
         self.assertEqual('bing', wt.get_symlink_target('foo-id'))
 
     def test_symlink_this_changed_kind(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         #   A       Nothing
         #  / \
         # B   C     B creates symlink foo => bar
@@ -2465,7 +2455,7 @@
 
     def test_symlink_all_wt(self):
         """Check behavior if all trees are Working Trees."""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         # The big issue is that entry.symlink_target is None for WorkingTrees.
         # So we need to make sure we handle that case correctly.
         #   A   foo => bar

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2011-06-09 11:48:03 +0000
+++ bzrlib/tests/test_msgeditor.py	2011-06-14 01:26:41 +0000
@@ -34,6 +34,7 @@
     edit_commit_message_encoded
 )
 from bzrlib.tests import (
+    features,
     TestCaseInTempDir,
     TestCaseWithTransport,
     TestNotApplicable,
@@ -309,7 +310,7 @@
         self.assertFileEqual(expected, msgfilename)
 
     def test__create_temp_file_with_commit_template_in_unicode_dir(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         if hasattr(self, 'info'):
             tmpdir = self.info['directory']
             os.mkdir(tmpdir)

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2011-06-16 18:34:26 +0000
+++ bzrlib/tests/test_osutils.py	2011-07-08 23:01:39 +0000
@@ -41,7 +41,7 @@
 from bzrlib.tests.scenarios import load_tests_apply_scenarios
 
 
-class _UTF8DirReaderFeature(tests.Feature):
+class _UTF8DirReaderFeature(features.Feature):
 
     def _probe(self):
         try:
@@ -54,9 +54,9 @@
     def feature_name(self):
         return 'bzrlib._readdir_pyx'
 
-UTF8DirReaderFeature = _UTF8DirReaderFeature()
+UTF8DirReaderFeature = features.ModuleAvailableFeature('bzrlib._readdir_pyx')
 
-term_ios_feature = tests.ModuleAvailableFeature('termios')
+term_ios_feature = features.ModuleAvailableFeature('termios')
 
 
 def _already_unicode(s):
@@ -429,7 +429,7 @@
 class TestLinks(tests.TestCaseInTempDir):
 
     def test_dereference_path(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         cwd = osutils.realpath('.')
         os.mkdir('bar')
         bar_path = osutils.pathjoin(cwd, 'bar')
@@ -482,7 +482,7 @@
 
 class TestCanonicalRelPath(tests.TestCaseInTempDir):
 
-    _test_needs_features = [tests.CaseInsCasePresFilenameFeature]
+    _test_needs_features = [features.CaseInsCasePresFilenameFeature]
 
     def test_canonical_relpath_simple(self):
         f = file('MixedCaseName', 'w')
@@ -888,7 +888,7 @@
     """Test win32 functions that create files."""
 
     def test_getcwd(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         os.mkdir(u'mu-\xb5')
         os.chdir(u'mu-\xb5')
         # TODO: jam 20060427 This will probably fail on Mac OSX because
@@ -984,13 +984,13 @@
     """Test mac special functions that require directories."""
 
     def test_getcwd(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         os.mkdir(u'B\xe5gfors')
         os.chdir(u'B\xe5gfors')
         self.assertEndsWith(osutils._mac_getcwd(), u'B\xe5gfors')
 
     def test_getcwd_nonnorm(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         # Test that _mac_getcwd() will normalize this path
         os.mkdir(u'Ba\u030agfors')
         os.chdir(u'Ba\u030agfors')
@@ -1106,7 +1106,7 @@
         # are not using the filesystem's encoding
 
         # require a bytestring based filesystem
-        self.requireFeature(tests.ByteStringNamedFilesystem)
+        self.requireFeature(features.ByteStringNamedFilesystem)
 
         tree = [
             '.bzr',
@@ -1207,21 +1207,24 @@
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
         osutils._fs_enc = 'UTF-8'
-        self.assertDirReaderIs(UTF8DirReaderFeature.reader)
+        self.assertDirReaderIs(
+            UTF8DirReaderFeature.module.UTF8DirReader)
 
     def test_force_walkdirs_utf8_fs_ascii(self):
         self.requireFeature(UTF8DirReaderFeature)
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
         osutils._fs_enc = 'US-ASCII'
-        self.assertDirReaderIs(UTF8DirReaderFeature.reader)
+        self.assertDirReaderIs(
+            UTF8DirReaderFeature.module.UTF8DirReader)
 
     def test_force_walkdirs_utf8_fs_ANSI(self):
         self.requireFeature(UTF8DirReaderFeature)
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
         osutils._fs_enc = 'ANSI_X3.4-1968'
-        self.assertDirReaderIs(UTF8DirReaderFeature.reader)
+        self.assertDirReaderIs(
+            UTF8DirReaderFeature.module.UTF8DirReader)
 
     def test_force_walkdirs_utf8_fs_latin1(self):
         self._save_platform_info()
@@ -1245,7 +1248,7 @@
 
     def test_unicode_walkdirs(self):
         """Walkdirs should always return unicode paths."""
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         name0 = u'0file-\xb6'
         name1 = u'1dir-\u062c\u0648'
         name2 = u'2file-\u0633'
@@ -1288,7 +1291,7 @@
 
         The abspath portion might be in unicode or utf-8
         """
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         name0 = u'0file-\xb6'
         name1 = u'1dir-\u062c\u0648'
         name2 = u'2file-\u0633'
@@ -1349,7 +1352,7 @@
 
         The abspath portion should be in unicode
         """
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         # Use the unicode reader. TODO: split into driver-and-driven unit
         # tests.
         self._save_platform_info()
@@ -1396,7 +1399,7 @@
 
     def test__walkdirs_utf8_win32readdir(self):
         self.requireFeature(test__walkdirs_win32.win32_readdir_feature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         from bzrlib._walkdirs_win32 import Win32ReadDir
         self._save_platform_info()
         osutils._selected_dir_reader = Win32ReadDir()
@@ -1453,7 +1456,7 @@
     def test__walkdirs_utf_win32_find_file_stat_file(self):
         """make sure our Stat values are valid"""
         self.requireFeature(test__walkdirs_win32.win32_readdir_feature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         from bzrlib._walkdirs_win32 import Win32ReadDir
         name0u = u'0file-\xb6'
         name0 = name0u.encode('utf8')
@@ -1477,7 +1480,7 @@
     def test__walkdirs_utf_win32_find_file_stat_directory(self):
         """make sure our Stat values are valid"""
         self.requireFeature(test__walkdirs_win32.win32_readdir_feature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         from bzrlib._walkdirs_win32 import Win32ReadDir
         name0u = u'0dir-\u062c\u0648'
         name0 = name0u.encode('utf8')
@@ -1583,7 +1586,7 @@
         self.assertEqual(['c'], os.listdir('target/b'))
 
     def test_copy_tree_symlinks(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.build_tree(['source/'])
         os.symlink('a/generic/path', 'source/lnk')
         osutils.copy_tree('source', 'target')
@@ -1856,15 +1859,15 @@
         return filtered_dirblocks
 
     def test_walk_unicode_tree(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree, expected_dirblocks = self._get_unicode_tree()
         self.build_tree(tree)
         result = list(osutils._walkdirs_utf8('.'))
         self.assertEqual(expected_dirblocks, self._filter_out(result))
 
     def test_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         target = u'target\N{Euro Sign}'
         link_name = u'l\N{Euro Sign}nk'
         os.symlink(target, link_name)
@@ -1888,7 +1891,7 @@
     But prior python versions failed to properly encode the passed unicode
     string.
     """
-    _test_needs_features = [tests.SymlinkFeature, tests.UnicodeFilenameFeature]
+    _test_needs_features = [features.SymlinkFeature, features.UnicodeFilenameFeature]
 
     def setUp(self):
         super(tests.TestCaseInTempDir, self).setUp()

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2011-07-15 08:25:00 +0000
+++ bzrlib/tests/test_remote.py	2011-08-12 09:49:24 +0000
@@ -1183,6 +1183,28 @@
         client.add_expected_call(
             'Branch.last_revision_info', ('quack/',),
             'success', ('ok', '1', 'rev-tip'))
+        client.add_expected_call(
+            'Branch.get_config_file', ('quack/',),
+            'success', ('ok',), '')
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        branch = self.make_remote_branch(transport, client)
+        result = branch.heads_to_fetch()
+        self.assertFinished(client)
+        self.assertEqual((set(['rev-tip']), set()), result)
+
+    def test_uses_last_revision_info_and_tags_when_set(self):
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('quack/',),
+            'error', ('NotStacked',))
+        client.add_expected_call(
+            'Branch.last_revision_info', ('quack/',),
+            'success', ('ok', '1', 'rev-tip'))
+        client.add_expected_call(
+            'Branch.get_config_file', ('quack/',),
+            'success', ('ok',), 'branch.fetch_tags = True')
         # XXX: this will break if the default format's serialization of tags
         # changes, or if the RPC for fetching tags changes from get_tags_bytes.
         client.add_expected_call(
@@ -1213,7 +1235,7 @@
         self.assertFinished(client)
         self.assertEqual((set(['tip']), set(['tagged-1', 'tagged-2'])), result)
 
-    def test_backwards_compatible(self):
+    def make_branch_with_tags(self):
         self.setup_smart_server_with_call_log()
         # Make a branch with a single revision.
         builder = self.make_branch_builder('foo')
@@ -1225,6 +1247,12 @@
         # Add two tags to that branch
         branch.tags.set_tag('tag-1', 'rev-1')
         branch.tags.set_tag('tag-2', 'rev-2')
+        return branch
+
+    def test_backwards_compatible(self):
+        branch = self.make_branch_with_tags()
+        c = branch.get_config()
+        c.set_user_option('branch.fetch_tags', 'True')
         self.addCleanup(branch.lock_read().unlock)
         # Disable the heads_to_fetch verb
         verb = 'Branch.heads_to_fetch'
@@ -1233,7 +1261,23 @@
         result = branch.heads_to_fetch()
         self.assertEqual((set(['tip']), set(['rev-1', 'rev-2'])), result)
         self.assertEqual(
-            ['Branch.last_revision_info', 'Branch.get_tags_bytes'],
+            ['Branch.last_revision_info', 'Branch.get_config_file',
+             'Branch.get_tags_bytes'],
+            [call.call.method for call in self.hpss_calls])
+
+    def test_backwards_compatible_no_tags(self):
+        branch = self.make_branch_with_tags()
+        c = branch.get_config()
+        c.set_user_option('branch.fetch_tags', 'False')
+        self.addCleanup(branch.lock_read().unlock)
+        # Disable the heads_to_fetch verb
+        verb = 'Branch.heads_to_fetch'
+        self.disable_verb(verb)
+        self.reset_smart_call_log()
+        result = branch.heads_to_fetch()
+        self.assertEqual((set(['tip']), set()), result)
+        self.assertEqual(
+            ['Branch.last_revision_info', 'Branch.get_config_file'],
             [call.call.method for call in self.hpss_calls])
 
 
@@ -3408,21 +3452,46 @@
         self.assertTrue(len(self.hpss_calls) > 1)
 
 
-class TestUpdateBoundBranch(tests.TestCaseWithTransport):
-
-    def test_bug_786980(self):
+class TestUpdateBoundBranchWithModifiedBoundLocation(
+    tests.TestCaseWithTransport):
+    """Ensure correct handling of bound_location modifications.
+
+    This is tested against a smart server as http://pad.lv/786980 was about a
+    ReadOnlyError (write attempt during a read-only transaction) which can only
+    happen in this context.
+    """
+
+    def setUp(self):
+        super(TestUpdateBoundBranchWithModifiedBoundLocation, self).setUp()
         self.transport_server = test_server.SmartTCPServer_for_testing
-        wt = self.make_branch_and_tree('master')
-        checkout = wt.branch.create_checkout('checkout')
-        wt.commit('add stuff')
-        last_revid = wt.commit('even more stuff')
-        bound_location = checkout.branch.get_bound_location()
+
+    def make_master_and_checkout(self, master_name, checkout_name):
+        # Create the master branch and its associated checkout
+        self.master = self.make_branch_and_tree(master_name)
+        self.checkout = self.master.branch.create_checkout(checkout_name)
+        # Modify the master branch so there is something to update
+        self.master.commit('add stuff')
+        self.last_revid = self.master.commit('even more stuff')
+        self.bound_location = self.checkout.branch.get_bound_location()
+
+    def assertUpdateSucceeds(self, new_location):
+        self.checkout.branch.set_bound_location(new_location)
+        self.checkout.update()
+        self.assertEquals(self.last_revid, self.checkout.last_revision())
+
+    def test_without_final_slash(self):
+        self.make_master_and_checkout('master', 'checkout')
         # For unclear reasons some users have a bound_location without a final
         # '/', simulate that by forcing such a value
-        self.assertEndsWith(bound_location, '/')
-        new_location = bound_location.rstrip('/')
-        checkout.branch.set_bound_location(new_location)
-        # bug 786980 was raising ReadOnlyError: A write attempt was made in a
-        # read only transaction during the update()
-        checkout.update()
-        self.assertEquals(last_revid, checkout.last_revision())
+        self.assertEndsWith(self.bound_location, '/')
+        self.assertUpdateSucceeds(self.bound_location.rstrip('/'))
+
+    def test_plus_sign(self):
+        self.make_master_and_checkout('+master', 'checkout')
+        self.assertUpdateSucceeds(self.bound_location.replace('%2B', '+', 1))
+
+    def test_tilda(self):
+        # Embed ~ in the middle of the path just to avoid any $HOME
+        # interpretation
+        self.make_master_and_checkout('mas~ter', 'checkout')
+        self.assertUpdateSucceeds(self.bound_location.replace('%2E', '~', 1))

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2011-07-26 07:30:06 +0000
+++ bzrlib/tests/test_selftest.py	2011-08-06 03:52:34 +0000
@@ -29,6 +29,7 @@
 from testtools import (
     ExtendedToOriginalDecorator,
     MultiTestResult,
+    __version__ as testtools_version,
     )
 from testtools.content import Content
 from testtools.content_type import ContentType
@@ -93,19 +94,10 @@
             DocTestMatches(u"...a test message\n", doctest.ELLIPSIS))
 
 
-class TestUnicodeFilename(tests.TestCase):
-
-    def test_probe_passes(self):
-        """UnicodeFilename._probe passes."""
-        # We can't test much more than that because the behaviour depends
-        # on the platform.
-        tests.UnicodeFilename._probe()
-
-
 class TestTreeShape(tests.TestCaseInTempDir):
 
     def test_unicode_paths(self):
-        self.requireFeature(tests.UnicodeFilename)
+        self.requireFeature(features.UnicodeFilenameFeature)
 
         filename = u'hell\u00d8'
         self.build_tree_contents([(filename, 'contents of hello')])
@@ -721,7 +713,7 @@
 class TestProfileResult(tests.TestCase):
 
     def test_profiles_tests(self):
-        self.requireFeature(test_lsprof.LSProfFeature)
+        self.requireFeature(features.lsprof_feature)
         terminal = testtools.testresult.doubles.ExtendedTestResult()
         result = tests.ProfileResult(terminal)
         class Sample(tests.TestCase):
@@ -782,7 +774,7 @@
 
     def test_lsprofiling(self):
         """Verbose test result prints lsprof statistics from test cases."""
-        self.requireFeature(test_lsprof.LSProfFeature)
+        self.requireFeature(features.lsprof_feature)
         result_stream = StringIO()
         result = bzrlib.tests.VerboseTestResult(
             result_stream,
@@ -833,7 +825,7 @@
         self.assertEndsWith(sio.getvalue(), "OK    50002ms\n")
 
     def test_known_failure(self):
-        """A KnownFailure being raised should trigger several result actions."""
+        """Using knownFailure should trigger several result actions."""
         class InstrumentedTestResult(tests.ExtendedTestResult):
             def stopTestRun(self): pass
             def report_tests_starting(self): pass
@@ -842,7 +834,7 @@
         result = InstrumentedTestResult(None, None, None, None)
         class Test(tests.TestCase):
             def test_function(self):
-                raise tests.KnownFailure('failed!')
+                self.knownFailure('failed!')
         test = Test("test_function")
         test.run(result)
         # it should invoke 'report_known_failure'.
@@ -864,23 +856,13 @@
             descriptions=0,
             verbosity=2,
             )
-        test = self.get_passing_test()
-        result.startTest(test)
-        prefix = len(result_stream.getvalue())
-        # the err parameter has the shape:
-        # (class, exception object, traceback)
-        # KnownFailures dont get their tracebacks shown though, so we
-        # can skip that.
-        err = (tests.KnownFailure, tests.KnownFailure('foo'), None)
-        result.report_known_failure(test, err)
-        output = result_stream.getvalue()[prefix:]
-        lines = output.splitlines()
-        self.assertContainsRe(lines[0], r'XFAIL *\d+ms$')
-        if sys.version_info > (2, 7):
-            self.expectFailure("_ExpectedFailure on 2.7 loses the message",
-                self.assertNotEqual, lines[1], '    ')
-        self.assertEqual(lines[1], '    foo')
-        self.assertEqual(2, len(lines))
+        _get_test("test_xfail").run(result)
+        self.assertContainsRe(result_stream.getvalue(),
+            "\n\\S+\\.test_xfail\\s+XFAIL\\s+\\d+ms\n"
+            "\\s*(?:Text attachment: )?reason"
+            "(?:\n-+\n|: {{{)"
+            "this_fails"
+            "(?:\n-+\n|}}}\n)")
 
     def get_passing_test(self):
         """Return a test object that can't be run usefully."""
@@ -897,7 +879,7 @@
                 self._call = test, feature
         result = InstrumentedTestResult(None, None, None, None)
         test = SampleTestCase('_test_pass')
-        feature = tests.Feature()
+        feature = features.Feature()
         result.startTest(test)
         result.addNotSupported(test, feature)
         # it should invoke 'report_unsupported'.
@@ -922,7 +904,7 @@
             verbosity=2,
             )
         test = self.get_passing_test()
-        feature = tests.Feature()
+        feature = features.Feature()
         result.startTest(test)
         prefix = len(result_stream.getvalue())
         result.report_unsupported(test, feature)
@@ -941,7 +923,7 @@
             def addNotSupported(self, test, feature):
                 self._call = test, feature
         result = InstrumentedTestResult(None, None, None, None)
-        feature = tests.Feature()
+        feature = features.Feature()
         class Test(tests.TestCase):
             def test_function(self):
                 raise tests.UnavailableFeature(feature)
@@ -966,9 +948,8 @@
     def test_strict_with_known_failure(self):
         result = bzrlib.tests.TextTestResult(self._log_file, descriptions=0,
                                              verbosity=1)
-        test = self.get_passing_test()
-        err = (tests.KnownFailure, tests.KnownFailure('foo'), None)
-        result.addExpectedFailure(test, err)
+        test = _get_test("test_xfail")
+        test.run(result)
         self.assertFalse(result.wasStrictlySuccessful())
         self.assertEqual(None, result._extractBenchmarkTime(test))
 
@@ -1006,15 +987,6 @@
         self.assertEquals(2, result.count)
 
 
-class TestUnicodeFilenameFeature(tests.TestCase):
-
-    def test_probe_passes(self):
-        """UnicodeFilenameFeature._probe passes."""
-        # We can't test much more than that because the behaviour depends
-        # on the platform.
-        tests.UnicodeFilenameFeature._probe()
-
-
 class TestRunner(tests.TestCase):
 
     def dummy_test(self):
@@ -1090,17 +1062,31 @@
                 self.expectFailure("No absolute truth", self.assertTrue, True)
         runner = tests.TextTestRunner(stream=StringIO())
         result = self.run_test_runner(runner, Test("test_truth"))
-        self.assertContainsRe(runner.stream.getvalue(),
-            "=+\n"
-            "FAIL: \\S+\.test_truth\n"
-            "-+\n"
-            "(?:.*\n)*"
-            "No absolute truth\n"
-            "(?:.*\n)*"
-            "-+\n"
-            "Ran 1 test in .*\n"
-            "\n"
-            "FAILED \\(failures=1\\)\n\\Z")
+        if testtools_version <= (0, 9, 11):
+            self.assertContainsRe(runner.stream.getvalue(),
+                "=+\n"
+                "FAIL: \\S+\.test_truth\n"
+                "-+\n"
+                "(?:.*\n)*"
+                "No absolute truth\n"
+                "(?:.*\n)*"
+                "-+\n"
+                "Ran 1 test in .*\n"
+                "\n"
+                "FAILED \\(failures=1\\)\n\\Z")
+        else:
+            self.assertContainsRe(runner.stream.getvalue(),
+                "=+\n"
+                "FAIL: \\S+\.test_truth\n"
+                "-+\n"
+                "Empty attachments:\n"
+                "  log\n"
+                "\n"
+                "reason: {{{No absolute truth}}}\n"
+                "-+\n"
+                "Ran 1 test in .*\n"
+                "\n"
+                "FAILED \\(failures=1\\)\n\\Z")
 
     def test_result_decorator(self):
         # decorate results
@@ -1191,9 +1177,9 @@
 
     def test_unsupported_features_listed(self):
         """When unsupported features are encountered they are detailed."""
-        class Feature1(tests.Feature):
+        class Feature1(features.Feature):
             def _probe(self): return False
-        class Feature2(tests.Feature):
+        class Feature2(features.Feature):
             def _probe(self): return False
         # create sample tests
         test1 = SampleTestCase('_test_pass')
@@ -1265,11 +1251,14 @@
             lambda trace=False: "ascii")
         result = self.run_test_runner(tests.TextTestRunner(stream=out),
             FailureWithUnicode("test_log_unicode"))
-        self.assertContainsRe(out.getvalue(),
-            "Text attachment: log\n"
-            "-+\n"
-            "\d+\.\d+  \\\\u2606\n"
-            "-+\n")
+        if testtools_version > (0, 9, 11):
+            self.assertContainsRe(out.getvalue(), "log: {{{\d+\.\d+  \\\\u2606}}}")
+        else:
+            self.assertContainsRe(out.getvalue(),
+                "Text attachment: log\n"
+                "-+\n"
+                "\d+\.\d+  \\\\u2606\n"
+                "-+\n")
 
 
 class SampleTestCase(tests.TestCase):
@@ -1476,7 +1465,7 @@
 
         Each self.time() call is individually and separately profiled.
         """
-        self.requireFeature(test_lsprof.LSProfFeature)
+        self.requireFeature(features.lsprof_feature)
         # overrides the class member with an instance member so no cleanup
         # needed.
         self._gather_lsprof_in_benchmarks = True
@@ -1512,14 +1501,14 @@
 
     def test_requireFeature_available(self):
         """self.requireFeature(available) is a no-op."""
-        class Available(tests.Feature):
+        class Available(features.Feature):
             def _probe(self):return True
         feature = Available()
         self.requireFeature(feature)
 
     def test_requireFeature_unavailable(self):
         """self.requireFeature(unavailable) raises UnavailableFeature."""
-        class Unavailable(tests.Feature):
+        class Unavailable(features.Feature):
             def _probe(self):return False
         feature = Unavailable()
         self.assertRaises(tests.UnavailableFeature,
@@ -1697,7 +1686,7 @@
     return a + b
 
 
-class _MissingFeature(tests.Feature):
+class _MissingFeature(features.Feature):
     def _probe(self):
         return False
 missing_feature = _MissingFeature()
@@ -1754,14 +1743,16 @@
         result = self._run_test('test_fail')
         self.assertEqual(1, len(result.failures))
         result_content = result.failures[0][1]
-        self.assertContainsRe(result_content, 'Text attachment: log')
+        if testtools_version < (0, 9, 12):
+            self.assertContainsRe(result_content, 'Text attachment: log')
         self.assertContainsRe(result_content, 'this was a failing test')
 
     def test_error_has_log(self):
         result = self._run_test('test_error')
         self.assertEqual(1, len(result.errors))
         result_content = result.errors[0][1]
-        self.assertContainsRe(result_content, 'Text attachment: log')
+        if testtools_version < (0, 9, 12):
+            self.assertContainsRe(result_content, 'Text attachment: log')
         self.assertContainsRe(result_content, 'this test errored')
 
     def test_skip_has_no_log(self):
@@ -2049,7 +2040,7 @@
         self.assertLength(2, output.readlines())
 
     def test_lsprof_tests(self):
-        self.requireFeature(test_lsprof.LSProfFeature)
+        self.requireFeature(features.lsprof_feature)
         results = []
         class Test(object):
             def __call__(test, result):
@@ -2619,80 +2610,6 @@
         self.assertEqual('bzr: interrupted\n', result[1])
 
 
-class TestFeature(tests.TestCase):
-
-    def test_caching(self):
-        """Feature._probe is called by the feature at most once."""
-        class InstrumentedFeature(tests.Feature):
-            def __init__(self):
-                super(InstrumentedFeature, self).__init__()
-                self.calls = []
-            def _probe(self):
-                self.calls.append('_probe')
-                return False
-        feature = InstrumentedFeature()
-        feature.available()
-        self.assertEqual(['_probe'], feature.calls)
-        feature.available()
-        self.assertEqual(['_probe'], feature.calls)
-
-    def test_named_str(self):
-        """Feature.__str__ should thunk to feature_name()."""
-        class NamedFeature(tests.Feature):
-            def feature_name(self):
-                return 'symlinks'
-        feature = NamedFeature()
-        self.assertEqual('symlinks', str(feature))
-
-    def test_default_str(self):
-        """Feature.__str__ should default to __class__.__name__."""
-        class NamedFeature(tests.Feature):
-            pass
-        feature = NamedFeature()
-        self.assertEqual('NamedFeature', str(feature))
-
-
-class TestUnavailableFeature(tests.TestCase):
-
-    def test_access_feature(self):
-        feature = tests.Feature()
-        exception = tests.UnavailableFeature(feature)
-        self.assertIs(feature, exception.args[0])
-
-
-simple_thunk_feature = tests._CompatabilityThunkFeature(
-    deprecated_in((2, 1, 0)),
-    'bzrlib.tests.test_selftest',
-    'simple_thunk_feature','UnicodeFilename',
-    replacement_module='bzrlib.tests'
-    )
-
-class Test_CompatibilityFeature(tests.TestCase):
-
-    def test_does_thunk(self):
-        res = self.callDeprecated(
-            ['bzrlib.tests.test_selftest.simple_thunk_feature was deprecated'
-             ' in version 2.1.0. Use bzrlib.tests.UnicodeFilename instead.'],
-            simple_thunk_feature.available)
-        self.assertEqual(tests.UnicodeFilename.available(), res)
-
-
-class TestModuleAvailableFeature(tests.TestCase):
-
-    def test_available_module(self):
-        feature = tests.ModuleAvailableFeature('bzrlib.tests')
-        self.assertEqual('bzrlib.tests', feature.module_name)
-        self.assertEqual('bzrlib.tests', str(feature))
-        self.assertTrue(feature.available())
-        self.assertIs(tests, feature.module)
-
-    def test_unavailable_module(self):
-        feature = tests.ModuleAvailableFeature('bzrlib.no_such_module_exists')
-        self.assertEqual('bzrlib.no_such_module_exists', str(feature))
-        self.assertFalse(feature.available())
-        self.assertIs(None, feature.module)
-
-
 class TestSelftestFiltering(tests.TestCase):
 
     def setUp(self):

=== modified file 'bzrlib/tests/test_shelf.py'
--- bzrlib/tests/test_shelf.py	2011-06-19 02:24:39 +0000
+++ bzrlib/tests/test_shelf.py	2011-07-08 23:01:39 +0000
@@ -25,6 +25,9 @@
     transform,
     workingtree,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 
 EMPTY_SHELF = ("Bazaar pack format 1 (introduced in 0.18)\n"
@@ -214,7 +217,7 @@
 
     def _test_shelve_symlink_creation(self, link_name, link_target,
                                       shelve_change=False):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -241,7 +244,7 @@
         self._test_shelve_symlink_creation('foo', 'bar')
 
     def test_shelve_unicode_symlink_creation(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_shelve_symlink_creation(u'fo\N{Euro Sign}o',
                                            u'b\N{Euro Sign}ar')
 
@@ -251,7 +254,7 @@
     def _test_shelve_symlink_target_change(self, link_name,
                                            old_target, new_target,
                                            shelve_change=False):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -282,7 +285,7 @@
         self._test_shelve_symlink_target_change('foo', 'bar', 'baz')
 
     def test_shelve_unicode_symlink_target_change(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_shelve_symlink_target_change(
             u'fo\N{Euro Sign}o', u'b\N{Euro Sign}ar', u'b\N{Euro Sign}az')
 

=== modified file 'bzrlib/tests/test_shelf_ui.py'
--- bzrlib/tests/test_shelf_ui.py	2011-07-07 09:34:06 +0000
+++ bzrlib/tests/test_shelf_ui.py	2011-07-08 23:01:39 +0000
@@ -27,6 +27,9 @@
     tests,
 )
 from bzrlib.tests import script
+from bzrlib.tests import (
+    features,
+    )
 
 
 class ExpectShelver(shelf_ui.Shelver):
@@ -208,7 +211,7 @@
         shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
 
     def test_shelve_modify_target(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.create_shelvable_tree()
         os.symlink('bar', 'tree/baz')
         tree.add('baz', 'baz-id')
@@ -448,7 +451,7 @@
         shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
 
     def test_shelve_modify_target(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.create_shelvable_tree()
         os.symlink('bar', 'tree/baz')
         tree.add('baz', 'baz-id')

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2011-05-26 08:05:45 +0000
+++ bzrlib/tests/test_smart_transport.py	2011-06-14 01:26:41 +0000
@@ -41,6 +41,7 @@
         vfs,
 )
 from bzrlib.tests import (
+    features,
     test_smart,
     test_server,
     )
@@ -82,7 +83,7 @@
         return 'pipes', (self.vendor.read_from, self.vendor.write_to)
 
 
-class _InvalidHostnameFeature(tests.Feature):
+class _InvalidHostnameFeature(features.Feature):
     """Does 'non_existent.invalid' fail to resolve?
 
     RFC 2606 states that .invalid is reserved for invalid domain names, and

=== modified file 'bzrlib/tests/test_strace.py'
--- bzrlib/tests/test_strace.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_strace.py	2011-08-04 00:17:53 +0000
@@ -22,12 +22,15 @@
     strace,
     tests,
     )
-from bzrlib.strace import StraceFeature, strace_detailed, StraceResult
+from bzrlib.strace import strace_detailed, StraceResult
+from bzrlib.tests.features import (
+    strace_feature,
+    )
 
 
 class TestStrace(tests.TestCaseWithTransport):
 
-    _test_needs_features = [StraceFeature]
+    _test_needs_features = [strace_feature]
 
     def setUp(self):
         # NB: see http://pad.lv/626679 and
@@ -47,7 +50,7 @@
         # restriction.
         active = threading.activeCount()
         if active > 1: # There is always the main thread at least
-            raise tests.KnownFailure(
+            self.knownFailure(
                 '%d active threads, bug #103133 needs to be fixed.' % active)
 
     def strace_detailed_or_skip(self, *args, **kwargs):

=== modified file 'bzrlib/tests/test_tag.py'
--- bzrlib/tests/test_tag.py	2010-11-18 00:43:57 +0000
+++ bzrlib/tests/test_tag.py	2011-08-06 07:32:51 +0000
@@ -26,7 +26,6 @@
     DisabledTags,
     )
 from bzrlib.tests import (
-    KnownFailure,
     TestCase,
     TestCaseWithTransport,
     )
@@ -168,7 +167,7 @@
         child.update()
         # and deletion of tags should also propagate
         master.tags.delete_tag('foo')
-        raise KnownFailure("tag deletion does not propagate: "
+        self.knownFailure("tag deletion does not propagate: "
             "https://bugs.launchpad.net/bzr/+bug/138802")
         self.assertRaises(errors.NoSuchTag,
             child.tags.lookup_tag, 'foo')

=== modified file 'bzrlib/tests/test_testament.py'
--- bzrlib/tests/test_testament.py	2011-05-11 14:26:41 +0000
+++ bzrlib/tests/test_testament.py	2011-06-14 01:26:41 +0000
@@ -21,13 +21,16 @@
 import os
 
 from bzrlib import osutils
-from bzrlib.tests import SymlinkFeature, TestCaseWithTransport
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.testament import (
     Testament,
     StrictTestament,
     StrictTestament3,
     )
 from bzrlib.transform import TreeTransform
+from bzrlib.tests.features import (
+    SymlinkFeature,
+    )
 
 
 class TestamentSetup(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2011-07-06 20:52:00 +0000
+++ bzrlib/tests/test_transform.py	2011-07-21 06:46:34 +0000
@@ -62,11 +62,13 @@
 from bzrlib.merge import Merge3Merger, Merger
 from bzrlib.tests import (
     features,
+    TestCaseInTempDir,
+    TestSkipped,
+    )
+from bzrlib.tests.features import (
     HardlinkFeature,
     SymlinkFeature,
-    TestCaseInTempDir,
-    TestSkipped,
-)
+    )
 from bzrlib.transform import (
     build_tree,
     create_from_tree,
@@ -321,6 +323,14 @@
         new_trans_id = transform.new_directory('', ROOT_PARENT, 'alt-root-id')
         self.assertRaises(ValueError, transform.fixup_new_roots)
 
+    def test_fixup_new_roots_permits_empty_tree(self):
+        transform, root = self.get_transform()
+        transform.delete_contents(root)
+        transform.unversion_file(root)
+        transform.fixup_new_roots()
+        self.assertIs(None, transform.final_kind(root))
+        self.assertIs(None, transform.final_file_id(root))
+
     def test_apply_retains_root_directory(self):
         # Do not attempt to delete the physical root directory, because that
         # is impossible.
@@ -332,6 +342,13 @@
                                   transform.apply)
         self.assertContainsRe('TransformRenameFailed not raised', str(e))
 
+    def test_apply_retains_file_id(self):
+        transform, root = self.get_transform()
+        old_root_id = transform.tree_file_id(root)
+        transform.unversion_file(root)
+        transform.apply()
+        self.assertEqual(old_root_id, self.wt.get_root_id())
+
     def test_hardlink(self):
         self.requireFeature(HardlinkFeature)
         transform, root = self.get_transform()
@@ -764,7 +781,7 @@
                             'wizard2', 'behind_curtain')
 
     def test_symlinks_unicode(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_symlinks(u'\N{Euro Sign}wizard',
                             u'wizard-targ\N{Euro Sign}t',
                             u'\N{Euro Sign}wizard2',
@@ -2209,8 +2226,8 @@
         self.assertEqualStat(source_stat, target_stat)
 
     def test_case_insensitive_build_tree_inventory(self):
-        if (tests.CaseInsensitiveFilesystemFeature.available()
-            or tests.CaseInsCasePresFilenameFeature.available()):
+        if (features.CaseInsensitiveFilesystemFeature.available()
+            or features.CaseInsCasePresFilenameFeature.available()):
             raise tests.UnavailableFeature('Fully case sensitive filesystem')
         source = self.make_branch_and_tree('source')
         self.build_tree(['source/file', 'source/FILE'])
@@ -3329,7 +3346,7 @@
         self.assertEqual('contents', rev2_tree.get_file_text('file_id'))
 
     def test_ascii_limbo_paths(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         branch = self.make_branch('any')
         tree = branch.repository.revision_tree(_mod_revision.NULL_REVISION)
         tt = TransformPreview(tree)
@@ -3352,7 +3369,7 @@
 
 class TestSerializeTransform(tests.TestCaseWithTransport):
 
-    _test_needs_features = [tests.UnicodeFilenameFeature]
+    _test_needs_features = [features.UnicodeFilenameFeature]
 
     def get_preview(self, tree=None):
         if tree is None:
@@ -3433,13 +3450,13 @@
         return self.make_records(attribs, contents)
 
     def test_serialize_symlink_creation(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tt = self.get_preview()
         tt.new_symlink(u'foo\u1234', tt.root, u'bar\u1234')
         self.assertSerializesTo(self.symlink_creation_records(), tt)
 
     def test_deserialize_symlink_creation(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tt = self.get_preview()
         tt.deserialize(iter(self.symlink_creation_records()))
         abspath = tt._limbo_name('new-1')

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2011-07-05 05:32:18 +0000
+++ bzrlib/tests/test_transport.py	2011-08-08 12:44:51 +0000
@@ -28,15 +28,18 @@
     transport,
     urlutils,
     )
+from bzrlib.directory_service import directories
 from bzrlib.transport import (
     chroot,
     fakenfs,
     http,
     local,
+    location_to_url,
     memory,
     pathfilter,
     readonly,
     )
+import bzrlib.transport.trace
 from bzrlib.tests import (
     features,
     test_server,
@@ -49,83 +52,72 @@
 class TestTransport(tests.TestCase):
     """Test the non transport-concrete class functionality."""
 
-    # FIXME: These tests should use addCleanup() and/or overrideAttr() instead
-    # of try/finally -- vila 20100205
-
     def test__get_set_protocol_handlers(self):
         handlers = transport._get_protocol_handlers()
-        self.assertNotEqual([], handlers.keys( ))
-        try:
-            transport._clear_protocol_handlers()
-            self.assertEqual([], transport._get_protocol_handlers().keys())
-        finally:
-            transport._set_protocol_handlers(handlers)
+        self.assertNotEqual([], handlers.keys())
+        transport._clear_protocol_handlers()
+        self.addCleanup(transport._set_protocol_handlers, handlers)
+        self.assertEqual([], transport._get_protocol_handlers().keys())
 
     def test_get_transport_modules(self):
         handlers = transport._get_protocol_handlers()
+        self.addCleanup(transport._set_protocol_handlers, handlers)
         # don't pollute the current handlers
         transport._clear_protocol_handlers()
+
         class SampleHandler(object):
             """I exist, isnt that enough?"""
-        try:
-            transport._clear_protocol_handlers()
-            transport.register_transport_proto('foo')
-            transport.register_lazy_transport('foo',
-                                              'bzrlib.tests.test_transport',
-                                              'TestTransport.SampleHandler')
-            transport.register_transport_proto('bar')
-            transport.register_lazy_transport('bar',
-                                              'bzrlib.tests.test_transport',
-                                              'TestTransport.SampleHandler')
-            self.assertEqual([SampleHandler.__module__,
-                              'bzrlib.transport.chroot',
-                              'bzrlib.transport.pathfilter'],
-                             transport._get_transport_modules())
-        finally:
-            transport._set_protocol_handlers(handlers)
+        transport._clear_protocol_handlers()
+        transport.register_transport_proto('foo')
+        transport.register_lazy_transport('foo',
+                                            'bzrlib.tests.test_transport',
+                                            'TestTransport.SampleHandler')
+        transport.register_transport_proto('bar')
+        transport.register_lazy_transport('bar',
+                                            'bzrlib.tests.test_transport',
+                                            'TestTransport.SampleHandler')
+        self.assertEqual([SampleHandler.__module__,
+                            'bzrlib.transport.chroot',
+                            'bzrlib.transport.pathfilter'],
+                            transport._get_transport_modules())
 
     def test_transport_dependency(self):
         """Transport with missing dependency causes no error"""
         saved_handlers = transport._get_protocol_handlers()
+        self.addCleanup(transport._set_protocol_handlers, saved_handlers)
         # don't pollute the current handlers
         transport._clear_protocol_handlers()
+        transport.register_transport_proto('foo')
+        transport.register_lazy_transport(
+            'foo', 'bzrlib.tests.test_transport', 'BadTransportHandler')
         try:
-            transport.register_transport_proto('foo')
-            transport.register_lazy_transport(
-                'foo', 'bzrlib.tests.test_transport', 'BadTransportHandler')
-            try:
-                transport.get_transport('foo://fooserver/foo')
-            except errors.UnsupportedProtocol, e:
-                e_str = str(e)
-                self.assertEquals('Unsupported protocol'
-                                  ' for url "foo://fooserver/foo":'
-                                  ' Unable to import library "some_lib":'
-                                  ' testing missing dependency', str(e))
-            else:
-                self.fail('Did not raise UnsupportedProtocol')
-        finally:
-            # restore original values
-            transport._set_protocol_handlers(saved_handlers)
+            transport.get_transport_from_url('foo://fooserver/foo')
+        except errors.UnsupportedProtocol, e:
+            e_str = str(e)
+            self.assertEquals('Unsupported protocol'
+                                ' for url "foo://fooserver/foo":'
+                                ' Unable to import library "some_lib":'
+                                ' testing missing dependency', str(e))
+        else:
+            self.fail('Did not raise UnsupportedProtocol')
 
     def test_transport_fallback(self):
         """Transport with missing dependency causes no error"""
         saved_handlers = transport._get_protocol_handlers()
-        try:
-            transport._clear_protocol_handlers()
-            transport.register_transport_proto('foo')
-            transport.register_lazy_transport(
-                'foo', 'bzrlib.tests.test_transport', 'BackupTransportHandler')
-            transport.register_lazy_transport(
-                'foo', 'bzrlib.tests.test_transport', 'BadTransportHandler')
-            t = transport.get_transport('foo://fooserver/foo')
-            self.assertTrue(isinstance(t, BackupTransportHandler))
-        finally:
-            transport._set_protocol_handlers(saved_handlers)
+        self.addCleanup(transport._set_protocol_handlers, saved_handlers)
+        transport._clear_protocol_handlers()
+        transport.register_transport_proto('foo')
+        transport.register_lazy_transport(
+            'foo', 'bzrlib.tests.test_transport', 'BackupTransportHandler')
+        transport.register_lazy_transport(
+            'foo', 'bzrlib.tests.test_transport', 'BadTransportHandler')
+        t = transport.get_transport_from_url('foo://fooserver/foo')
+        self.assertTrue(isinstance(t, BackupTransportHandler))
 
     def test_ssh_hints(self):
         """Transport ssh:// should raise an error pointing out bzr+ssh://"""
         try:
-            transport.get_transport('ssh://fooserver/foo')
+            transport.get_transport_from_url('ssh://fooserver/foo')
         except errors.UnsupportedProtocol, e:
             e_str = str(e)
             self.assertEquals('Unsupported protocol'
@@ -219,33 +211,32 @@
 
     def test_coalesce_fudge(self):
         self.check([(10, 30, [(0, 10), (20, 10)]),
-                    (100, 10, [(0, 10),]),
+                    (100, 10, [(0, 10)]),
                    ], [(10, 10), (30, 10), (100, 10)],
-                   fudge=10
-                  )
+                   fudge=10)
+
     def test_coalesce_max_size(self):
         self.check([(10, 20, [(0, 10), (10, 10)]),
                     (30, 50, [(0, 50)]),
                     # If one range is above max_size, it gets its own coalesced
                     # offset
-                    (100, 80, [(0, 80),]),],
+                    (100, 80, [(0, 80)]),],
                    [(10, 10), (20, 10), (30, 50), (100, 80)],
-                   max_size=50
-                  )
+                   max_size=50)
 
     def test_coalesce_no_max_size(self):
-        self.check([(10, 170, [(0, 10), (10, 10), (20, 50), (70, 100)]),],
+        self.check([(10, 170, [(0, 10), (10, 10), (20, 50), (70, 100)])],
                    [(10, 10), (20, 10), (30, 50), (80, 100)],
                   )
 
     def test_coalesce_default_limit(self):
         # By default we use a 100MB max size.
-        ten_mb = 10*1024*1024
-        self.check([(0, 10*ten_mb, [(i*ten_mb, ten_mb) for i in range(10)]),
+        ten_mb = 10 * 1024 * 1024
+        self.check([(0, 10 * ten_mb, [(i * ten_mb, ten_mb) for i in range(10)]),
                     (10*ten_mb, ten_mb, [(0, ten_mb)])],
                    [(i*ten_mb, ten_mb) for i in range(11)])
-        self.check([(0, 11*ten_mb, [(i*ten_mb, ten_mb) for i in range(11)]),],
-                   [(i*ten_mb, ten_mb) for i in range(11)],
+        self.check([(0, 11 * ten_mb, [(i * ten_mb, ten_mb) for i in range(11)])],
+                   [(i * ten_mb, ten_mb) for i in range(11)],
                    max_size=1*1024*1024*1024)
 
 
@@ -256,7 +247,7 @@
         server.start_server()
         url = server.get_url()
         self.assertTrue(url in transport.transport_list_registry)
-        t = transport.get_transport(url)
+        t = transport.get_transport_from_url(url)
         del t
         server.stop_server()
         self.assertFalse(url in transport.transport_list_registry)
@@ -377,9 +368,9 @@
     def test_abspath(self):
         # The abspath is always relative to the chroot_url.
         server = chroot.ChrootServer(
-            transport.get_transport('memory:///foo/bar/'))
+            transport.get_transport_from_url('memory:///foo/bar/'))
         self.start_server(server)
-        t = transport.get_transport(server.get_url())
+        t = transport.get_transport_from_url(server.get_url())
         self.assertEqual(server.get_url(), t.abspath('/'))
 
         subdir_t = t.clone('subdir')
@@ -387,9 +378,9 @@
 
     def test_clone(self):
         server = chroot.ChrootServer(
-            transport.get_transport('memory:///foo/bar/'))
+            transport.get_transport_from_url('memory:///foo/bar/'))
         self.start_server(server)
-        t = transport.get_transport(server.get_url())
+        t = transport.get_transport_from_url(server.get_url())
         # relpath from root and root path are the same
         relpath_cloned = t.clone('foo')
         abspath_cloned = t.clone('/foo')
@@ -404,13 +395,13 @@
         This is so that it is not possible to escape a chroot by doing::
             url = chroot_transport.base
             parent_url = urlutils.join(url, '..')
-            new_t = transport.get_transport(parent_url)
+            new_t = transport.get_transport_from_url(parent_url)
         """
         server = chroot.ChrootServer(
-            transport.get_transport('memory:///path/subpath'))
+            transport.get_transport_from_url('memory:///path/subpath'))
         self.start_server(server)
-        t = transport.get_transport(server.get_url())
-        new_t = transport.get_transport(t.base)
+        t = transport.get_transport_from_url(server.get_url())
+        new_t = transport.get_transport_from_url(t.base)
         self.assertEqual(t.server, new_t.server)
         self.assertEqual(t.base, new_t.base)
 
@@ -421,11 +412,12 @@
         This is so that it is not possible to escape a chroot by doing::
             url = chroot_transport.base
             parent_url = urlutils.join(url, '..')
-            new_t = transport.get_transport(parent_url)
+            new_t = transport.get_transport_from_url(parent_url)
         """
-        server = chroot.ChrootServer(transport.get_transport('memory:///path/'))
+        server = chroot.ChrootServer(
+            transport.get_transport_from_url('memory:///path/'))
         self.start_server(server)
-        t = transport.get_transport(server.get_url())
+        t = transport.get_transport_from_url(server.get_url())
         self.assertRaises(
             errors.InvalidURLJoin, urlutils.join, t.base, '..')
 
@@ -441,11 +433,9 @@
         backing_transport = memory.MemoryTransport()
         server = chroot.ChrootServer(backing_transport)
         server.start_server()
-        try:
-            self.assertTrue(server.scheme
-                            in transport._get_protocol_handlers().keys())
-        finally:
-            server.stop_server()
+        self.addCleanup(server.stop_server)
+        self.assertTrue(server.scheme
+                        in transport._get_protocol_handlers().keys())
 
     def test_stop_server(self):
         backing_transport = memory.MemoryTransport()
@@ -459,10 +449,8 @@
         backing_transport = memory.MemoryTransport()
         server = chroot.ChrootServer(backing_transport)
         server.start_server()
-        try:
-            self.assertEqual('chroot-%d:///' % id(server), server.get_url())
-        finally:
-            server.stop_server()
+        self.addCleanup(server.stop_server)
+        self.assertEqual('chroot-%d:///' % id(server), server.get_url())
 
 
 class PathFilteringDecoratorTransportTest(tests.TestCase):
@@ -471,10 +459,10 @@
     def test_abspath(self):
         # The abspath is always relative to the base of the backing transport.
         server = pathfilter.PathFilteringServer(
-            transport.get_transport('memory:///foo/bar/'),
+            transport.get_transport_from_url('memory:///foo/bar/'),
             lambda x: x)
         server.start_server()
-        t = transport.get_transport(server.get_url())
+        t = transport.get_transport_from_url(server.get_url())
         self.assertEqual(server.get_url(), t.abspath('/'))
 
         subdir_t = t.clone('subdir')
@@ -483,16 +471,16 @@
 
     def make_pf_transport(self, filter_func=None):
         """Make a PathFilteringTransport backed by a MemoryTransport.
-        
+
         :param filter_func: by default this will be a no-op function.  Use this
             parameter to override it."""
         if filter_func is None:
             filter_func = lambda x: x
         server = pathfilter.PathFilteringServer(
-            transport.get_transport('memory:///foo/bar/'), filter_func)
+            transport.get_transport_from_url('memory:///foo/bar/'), filter_func)
         server.start_server()
         self.addCleanup(server.stop_server)
-        return transport.get_transport(server.get_url())
+        return transport.get_transport_from_url(server.get_url())
 
     def test__filter(self):
         # _filter (with an identity func as filter_func) always returns
@@ -511,6 +499,7 @@
 
     def test_filter_invocation(self):
         filter_log = []
+
         def filter(path):
             filter_log.append(path)
             return path
@@ -541,10 +530,10 @@
         otherwise) the filtering by doing::
             url = filtered_transport.base
             parent_url = urlutils.join(url, '..')
-            new_t = transport.get_transport(parent_url)
+            new_t = transport.get_transport_from_url(parent_url)
         """
         t = self.make_pf_transport()
-        new_t = transport.get_transport(t.base)
+        new_t = transport.get_transport_from_url(t.base)
         self.assertEqual(t.server, new_t.server)
         self.assertEqual(t.base, new_t.base)
 
@@ -563,7 +552,7 @@
         # connect to '.' via http which is not listable
         server = HttpServer()
         self.start_server(server)
-        t = transport.get_transport('readonly+' + server.get_url())
+        t = transport.get_transport_from_url('readonly+' + server.get_url())
         self.assertIsInstance(t, readonly.ReadonlyTransportDecorator)
         self.assertEqual(False, t.listable())
         self.assertEqual(True, t.is_readonly())
@@ -602,7 +591,7 @@
         # the url should be decorated appropriately
         self.assertStartsWith(server.get_url(), 'fakenfs+')
         # and we should be able to get a transport for it
-        t = transport.get_transport(server.get_url())
+        t = transport.get_transport_from_url(server.get_url())
         # which must be a FakeNFSTransportDecorator instance.
         self.assertIsInstance(t, fakenfs.FakeNFSTransportDecorator)
 
@@ -685,7 +674,7 @@
         base_url = self._server.get_url()
         url = self._adjust_url(base_url, relpath)
         # try getting the transport via the regular interface:
-        t = transport.get_transport(url)
+        t = transport.get_transport_from_url(url)
         # vila--20070607 if the following are commented out the test suite
         # still pass. Is this really still needed or was it a forgotten
         # temporary fix ?
@@ -696,6 +685,34 @@
         return t
 
 
+class TestTransportFromPath(tests.TestCaseInTempDir):
+
+    def test_with_path(self):
+        t = transport.get_transport_from_path(self.test_dir)
+        self.assertIsInstance(t, local.LocalTransport)
+        self.assertEquals(t.base.rstrip("/"),
+            urlutils.local_path_to_url(self.test_dir))
+
+    def test_with_url(self):
+        t = transport.get_transport_from_path("file:")
+        self.assertIsInstance(t, local.LocalTransport)
+        self.assertEquals(t.base.rstrip("/"),
+            urlutils.local_path_to_url(os.path.join(self.test_dir, "file:")))
+
+
+class TestTransportFromUrl(tests.TestCaseInTempDir):
+
+    def test_with_path(self):
+        self.assertRaises(errors.InvalidURL, transport.get_transport_from_url,
+            self.test_dir)
+
+    def test_with_url(self):
+        url = urlutils.local_path_to_url(self.test_dir)
+        t = transport.get_transport_from_url(url)
+        self.assertIsInstance(t, local.LocalTransport)
+        self.assertEquals(t.base.rstrip("/"), url)
+
+
 class TestLocalTransports(tests.TestCase):
 
     def test_get_transport_from_abspath(self):
@@ -731,6 +748,10 @@
         We can't easily observe the external effect but we can at least see
         it's called.
         """
+        sentinel = object()
+        fdatasync = getattr(os, 'fdatasync', sentinel)
+        if fdatasync is sentinel:
+            raise tests.TestNotApplicable('fdatasync not supported')
         t = self.get_transport('.')
         calls = self.recordCalls(os, 'fdatasync')
         w = t.open_write_stream('out')
@@ -796,7 +817,8 @@
     def test_relpath(self):
         t = transport.ConnectedTransport('sftp://user@host.com/abs/path')
 
-        self.assertEquals(t.relpath('sftp://user@host.com/abs/path/sub'), 'sub')
+        self.assertEquals(t.relpath('sftp://user@host.com/abs/path/sub'),
+            'sub')
         self.assertRaises(errors.PathNotChild, t.relpath,
                           'http://user@host.com/abs/path/sub')
         self.assertRaises(errors.PathNotChild, t.relpath,
@@ -846,39 +868,40 @@
 
     def test_reuse_same_transport(self):
         possible_transports = []
-        t1 = transport.get_transport('http://foo/',
+        t1 = transport.get_transport_from_url('http://foo/',
                                      possible_transports=possible_transports)
         self.assertEqual([t1], possible_transports)
-        t2 = transport.get_transport('http://foo/',
+        t2 = transport.get_transport_from_url('http://foo/',
                                      possible_transports=[t1])
         self.assertIs(t1, t2)
 
         # Also check that final '/' are handled correctly
-        t3 = transport.get_transport('http://foo/path/')
-        t4 = transport.get_transport('http://foo/path',
+        t3 = transport.get_transport_from_url('http://foo/path/')
+        t4 = transport.get_transport_from_url('http://foo/path',
                                      possible_transports=[t3])
         self.assertIs(t3, t4)
 
-        t5 = transport.get_transport('http://foo/path')
-        t6 = transport.get_transport('http://foo/path/',
+        t5 = transport.get_transport_from_url('http://foo/path')
+        t6 = transport.get_transport_from_url('http://foo/path/',
                                      possible_transports=[t5])
         self.assertIs(t5, t6)
 
     def test_don_t_reuse_different_transport(self):
-        t1 = transport.get_transport('http://foo/path')
-        t2 = transport.get_transport('http://bar/path',
+        t1 = transport.get_transport_from_url('http://foo/path')
+        t2 = transport.get_transport_from_url('http://bar/path',
                                      possible_transports=[t1])
         self.assertIsNot(t1, t2)
 
 
 class TestTransportTrace(tests.TestCase):
 
-    def test_get(self):
-        t = transport.get_transport('trace+memory://')
-        self.assertIsInstance(t, bzrlib.transport.trace.TransportTraceDecorator)
+    def test_decorator(self):
+        t = transport.get_transport_from_url('trace+memory://')
+        self.assertIsInstance(
+            t, bzrlib.transport.trace.TransportTraceDecorator)
 
     def test_clone_preserves_activity(self):
-        t = transport.get_transport('trace+memory://')
+        t = transport.get_transport_from_url('trace+memory://')
         t2 = t.clone('.')
         self.assertTrue(t is not t2)
         self.assertTrue(t._activity is t2._activity)
@@ -888,7 +911,7 @@
     # still won't cause a test failure when the top level Transport API
     # changes; so there is little return doing that.
     def test_get(self):
-        t = transport.get_transport('trace+memory:///')
+        t = transport.get_transport_from_url('trace+memory:///')
         t.put_bytes('foo', 'barish')
         t.get('foo')
         expected_result = []
@@ -900,7 +923,7 @@
         self.assertEqual(expected_result, t._activity)
 
     def test_readv(self):
-        t = transport.get_transport('trace+memory:///')
+        t = transport.get_transport_from_url('trace+memory:///')
         t.put_bytes('foo', 'barish')
         list(t.readv('foo', [(0, 1), (3, 2)],
                      adjust_for_latency=True, upper_limit=6))
@@ -916,7 +939,7 @@
 class TestSSHConnections(tests.TestCaseWithTransport):
 
     def test_bzr_connect_to_bzr_ssh(self):
-        """User acceptance that get_transport of a bzr+ssh:// behaves correctly.
+        """get_transport of a bzr+ssh:// behaves correctly.
 
         bzr+ssh:// should cause bzr to run a remote bzr smart server over SSH.
         """
@@ -938,6 +961,7 @@
         # SSH channel ourselves.  Surely this has already been implemented
         # elsewhere?
         started = []
+
         class StubSSHServer(stub_sftp.StubServer):
 
             test = self
@@ -949,8 +973,8 @@
                     stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
                 # XXX: horribly inefficient, not to mention ugly.
-                # Start a thread for each of stdin/out/err, and relay bytes from
-                # the subprocess to channel and vice versa.
+                # Start a thread for each of stdin/out/err, and relay bytes
+                # from the subprocess to channel and vice versa.
                 def ferry_bytes(read, write, close):
                     while True:
                         bytes = read(1)
@@ -1024,3 +1048,38 @@
         result = http.unhtml_roughly(fake_html)
         self.assertEquals(len(result), 1000)
         self.assertStartsWith(result, " something!")
+
+
+class SomeDirectory(object):
+
+    def look_up(self, name, url):
+        return "http://bar"
+
+
+class TestLocationToUrl(tests.TestCase):
+
+    def test_regular_url(self):
+        self.assertEquals("file://foo", location_to_url("file://foo"))
+
+    def test_directory(self):
+        directories.register("bar:", SomeDirectory, "Dummy directory")
+        self.addCleanup(directories.remove, "bar:")
+        self.assertEquals("http://bar", location_to_url("bar:"))
+
+    def test_unicode_url(self):
+        self.assertRaises(errors.InvalidURL, location_to_url,
+            "http://fo/\xc3\xaf".decode("utf-8"))
+
+    def test_unicode_path(self):
+        self.assertEquals("file:///foo/bar%C3%AF",
+            location_to_url("/foo/bar\xc3\xaf".decode("utf-8")))
+
+    def test_path(self):
+        self.assertEquals("file:///foo/bar", location_to_url("/foo/bar"))
+
+    def test_relative_file_url(self):
+        self.assertEquals(urlutils.local_path_to_url(".") + "/bar",
+            location_to_url("file:bar"))
+
+    def test_absolute_file_url(self):
+        self.assertEquals("file:///bar", location_to_url("file:/bar"))

=== modified file 'bzrlib/tests/test_treeshape.py'
--- bzrlib/tests/test_treeshape.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_treeshape.py	2011-06-14 01:26:41 +0000
@@ -19,6 +19,9 @@
 
 
 from bzrlib import tests
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestTreeShape(tests.TestCaseWithTransport):
@@ -35,7 +38,7 @@
         self.assertFileEqual('hello', '.bzr/README')
 
     def test_build_tree_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.build_tree_contents([('link@', 'target')])
         self.assertEqual('target',
             os.readlink('link'))

=== modified file 'bzrlib/tests/test_win32utils.py'
--- bzrlib/tests/test_win32utils.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_win32utils.py	2011-06-14 01:26:41 +0000
@@ -24,36 +24,21 @@
     win32utils,
     )
 from bzrlib.tests import (
-    Feature,
     TestCase,
     TestCaseInTempDir,
     TestSkipped,
-    UnicodeFilenameFeature,
     )
 from bzrlib.tests.features import backslashdir_feature
 from bzrlib.win32utils import glob_expand, get_app_path
-
-
-class _RequiredModuleFeature(Feature):
-
-    def __init__(self, mod_name):
-        self.mod_name = mod_name
-        super(_RequiredModuleFeature, self).__init__()
-
-    def _probe(self):
-        try:
-            __import__(self.mod_name)
-            return True
-        except ImportError:
-            return False
-
-    def feature_name(self):
-        return self.mod_name
-
-Win32RegistryFeature = _RequiredModuleFeature('_winreg')
-CtypesFeature = _RequiredModuleFeature('ctypes')
-Win32comShellFeature = _RequiredModuleFeature('win32com.shell')
-Win32ApiFeature = _RequiredModuleFeature('win32api') 
+from bzrlib.tests import (
+    features,
+    )
+
+
+Win32RegistryFeature = features.ModuleAvailableFeature('_winreg')
+CtypesFeature = features.ModuleAvailableFeature('ctypes')
+Win32comShellFeature = features.ModuleAvailableFeature('win32com.shell')
+Win32ApiFeature = features.ModuleAvailableFeature('win32api') 
 
 
 # Tests
@@ -78,7 +63,7 @@
                          'd/', 'd/d1', 'd/d2', 'd/e/', 'd/e/e1'])
 
     def build_unicode_tree(self):
-        self.requireFeature(UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self.build_tree([u'\u1234', u'\u1234\u1234', u'\u1235/',
                          u'\u1235/\u1235'])
 
@@ -263,14 +248,14 @@
 
     def test_unicode_dir(self):
         # we should handle unicode paths without errors
-        self.requireFeature(UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         os.mkdir(u'\u1234')
         win32utils.set_file_attr_hidden(u'\u1234')
 
     def test_dot_bzr_in_unicode_dir(self):
         # we should not raise traceback if we try to set hidden attribute
         # on .bzr directory below unicode path
-        self.requireFeature(UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         os.makedirs(u'\u1234\\.bzr')
         path = osutils.abspath(u'\u1234\\.bzr')
         win32utils.set_file_attr_hidden(path)

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2011-05-19 09:32:38 +0000
+++ bzrlib/trace.py	2011-07-15 14:13:32 +0000
@@ -71,11 +71,6 @@
 
 import bzrlib
 
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    deprecated_in,
-    )
-
 lazy_import(globals(), """
 from bzrlib import (
     debug,
@@ -130,24 +125,6 @@
     _bzr_logger.warning(*args, **kwargs)
 
 
-@deprecated_function(deprecated_in((2, 1, 0)))
-def info(*args, **kwargs):
-    """Deprecated: use trace.note instead."""
-    note(*args, **kwargs)
-
-
-@deprecated_function(deprecated_in((2, 1, 0)))
-def log_error(*args, **kwargs):
-    """Deprecated: use bzrlib.trace.show_error instead"""
-    _bzr_logger.error(*args, **kwargs)
-
-
-@deprecated_function(deprecated_in((2, 1, 0)))
-def error(*args, **kwargs):
-    """Deprecated: use bzrlib.trace.show_error instead"""
-    _bzr_logger.error(*args, **kwargs)
-
-
 def show_error(*args, **kwargs):
     """Show an error message to the user.
 

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2011-07-06 20:52:00 +0000
+++ bzrlib/transform.py	2011-08-05 13:34:40 +0000
@@ -231,11 +231,6 @@
         new_roots = [k for k, v in self._new_parent.iteritems() if v is
                      ROOT_PARENT]
         if len(new_roots) < 1:
-            if self.final_kind(self.root) is None:
-                self.cancel_deletion(self.root)
-            if self.final_file_id(self.root) is None:
-                self.version_file(self.tree_file_id(self.root),
-                                     self.root)
             return
         if len(new_roots) != 1:
             raise ValueError('A tree cannot have two roots!')
@@ -1753,6 +1748,8 @@
                 mover.apply_deletions()
         finally:
             child_pb.finished()
+        if self.final_file_id(self.root) is None:
+            inventory_delta = [e for e in inventory_delta if e[0] != '']
         self._tree.apply_inventory_delta(inventory_delta)
         self._apply_observed_sha1s()
         self._done = True
@@ -2261,6 +2258,18 @@
         else:
             return None
 
+    def get_file_verifier(self, file_id, path=None, stat_value=None):
+        trans_id = self._transform.trans_id_file_id(file_id)
+        kind = self._transform._new_contents.get(trans_id)
+        if kind is None:
+            return self._transform._tree.get_file_verifier(file_id)
+        if kind == 'file':
+            fileobj = self.get_file(file_id)
+            try:
+                return ("SHA1", sha_file(fileobj))
+            finally:
+                fileobj.close()
+
     def get_file_sha1(self, file_id, path=None, stat_value=None):
         trans_id = self._transform.trans_id_file_id(file_id)
         kind = self._transform._new_contents.get(trans_id)

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2011-07-27 05:05:15 +0000
+++ bzrlib/transport/__init__.py	2011-08-02 01:10:27 +0000
@@ -119,10 +119,6 @@
     def register_transport(self, key, help=None):
         self.register(key, [], help)
 
-    def set_default_transport(self, key=None):
-        """Return either 'key' or the default key if key is None"""
-        self._default_key = key
-
 
 transport_list_registry = TransportListRegistry()
 
@@ -1584,71 +1580,98 @@
         raise NotImplementedError(self.disconnect)
 
 
-def get_transport(base, possible_transports=None):
-    """Open a transport to access a URL or directory.
-
-    :param base: either a URL or a directory name.
-
-    :param transports: optional reusable transports list. If not None, created
-        transports will be added to the list.
-
-    :return: A new transport optionally sharing its connection with one of
-        possible_transports.
+def location_to_url(location):
+    """Determine a fully qualified URL from a location string.
+
+    This will try to interpret location as both a URL and a directory path. It
+    will also lookup the location in directories.
+
+    :param location: Unicode or byte string object with a location
+    :raise InvalidURL: If the location is already a URL, but not valid.
+    :return: Byte string with resulting URL
     """
-    if base is None:
-        base = '.'
-    last_err = None
+    if not isinstance(location, basestring):
+        raise AssertionError("location not a byte or unicode string")
     from bzrlib.directory_service import directories
-    base = directories.dereference(base)
-
-    def convert_path_to_url(base, error_str):
-        if urlutils.is_url(base):
-            # This looks like a URL, but we weren't able to
-            # instantiate it as such raise an appropriate error
-            # FIXME: we have a 'error_str' unused and we use last_err below
-            raise errors.UnsupportedProtocol(base, last_err)
-        # This doesn't look like a protocol, consider it a local path
-        new_base = urlutils.local_path_to_url(base)
-        # mutter('converting os path %r => url %s', base, new_base)
-        return new_base
+    location = directories.dereference(location)
 
     # Catch any URLs which are passing Unicode rather than ASCII
     try:
-        base = base.encode('ascii')
+        location = location.encode('ascii')
     except UnicodeError:
-        # Only local paths can be Unicode
-        base = convert_path_to_url(base,
-            'URLs must be properly escaped (protocol: %s)')
-
+        if urlutils.is_url(location):
+            raise errors.InvalidURL(path=location,
+                extra='URLs must be properly escaped')
+        location = urlutils.local_path_to_url(location)
+
+    if location.startswith("file:") and not location.startswith("file://"):
+        return urlutils.join(urlutils.local_path_to_url("."), location[5:])
+
+    if not urlutils.is_url(location):
+        return urlutils.local_path_to_url(location)
+
+    return location
+
+
+def get_transport_from_path(path, possible_transports=None):
+    """Open a transport for a local path.
+
+    :param path: Local path as byte or unicode string
+    :return: Transport object for path
+    """
+    return get_transport_from_url(urlutils.local_path_to_url(path),
+        possible_transports)
+
+
+def get_transport_from_url(url, possible_transports=None):
+    """Open a transport to access a URL.
+    
+    :param base: a URL
+    :param transports: optional reusable transports list. If not None, created
+        transports will be added to the list.
+
+    :return: A new transport optionally sharing its connection with one of
+        possible_transports.
+    """
     transport = None
     if possible_transports is not None:
         for t in possible_transports:
-            t_same_connection = t._reuse_for(base)
+            t_same_connection = t._reuse_for(url)
             if t_same_connection is not None:
                 # Add only new transports
                 if t_same_connection not in possible_transports:
                     possible_transports.append(t_same_connection)
                 return t_same_connection
 
+    last_err = None
     for proto, factory_list in transport_list_registry.items():
-        if proto is not None and base.startswith(proto):
-            transport, last_err = _try_transport_factories(base, factory_list)
+        if proto is not None and url.startswith(proto):
+            transport, last_err = _try_transport_factories(url, factory_list)
             if transport:
                 if possible_transports is not None:
                     if transport in possible_transports:
                         raise AssertionError()
                     possible_transports.append(transport)
                 return transport
-
-    # We tried all the different protocols, now try one last time
-    # as a local protocol
-    base = convert_path_to_url(base, 'Unsupported protocol: %s')
-
-    # The default handler is the filesystem handler, stored as protocol None
-    factory_list = transport_list_registry.get(None)
-    transport, last_err = _try_transport_factories(base, factory_list)
-
-    return transport
+    if not urlutils.is_url(url):
+        raise errors.InvalidURL(path=url)
+    raise errors.UnsupportedProtocol(url, last_err)
+
+
+def get_transport(base, possible_transports=None):
+    """Open a transport to access a URL or directory.
+
+    :param base: either a URL or a directory name.
+
+    :param transports: optional reusable transports list. If not None, created
+        transports will be added to the list.
+
+    :return: A new transport optionally sharing its connection with one of
+        possible_transports.
+    """
+    if base is None:
+        base = '.'
+    return get_transport_from_url(location_to_url(base), possible_transports)
 
 
 def _try_transport_factories(base, factory_list):
@@ -1723,7 +1746,6 @@
 register_transport_proto('file://',
             help="Access using the standard filesystem (default)")
 register_lazy_transport('file://', 'bzrlib.transport.local', 'LocalTransport')
-transport_list_registry.set_default_transport("file://")
 
 register_transport_proto('sftp://',
             help="Access using SFTP (most SSH servers provide SFTP).",

=== modified file 'bzrlib/transport/gio_transport.py'
--- bzrlib/transport/gio_transport.py	2011-05-27 10:02:53 +0000
+++ bzrlib/transport/gio_transport.py	2011-07-23 16:33:06 +0000
@@ -24,15 +24,10 @@
 supported by gio.
 """
 from cStringIO import StringIO
-import getpass
 import os
 import random
-import socket
 import stat
-import urllib
 import time
-import sys
-import getpass
 import urlparse
 
 from bzrlib import (
@@ -49,12 +44,11 @@
     deprecated_passed,
     warn,
     )
-from bzrlib.trace import mutter, warning
+from bzrlib.trace import mutter
 from bzrlib.transport import (
     FileStream,
     ConnectedTransport,
     _file_streams,
-    Server,
     )
 
 from bzrlib.tests.test_server import TestServer

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2011-06-07 12:25:45 +0000
+++ bzrlib/transport/http/_pycurl.py	2011-06-14 01:26:41 +0000
@@ -423,10 +423,10 @@
 
 def get_test_permutations():
     """Return the permutations to be used in testing."""
-    from bzrlib import tests
+    from bzrlib.tests import features
     from bzrlib.tests import http_server
     permutations = [(PyCurlTransport, http_server.HttpServer_PyCurl),]
-    if tests.HTTPSServerFeature.available():
+    if features.HTTPSServerFeature.available():
         from bzrlib.tests import (
             https_server,
             ssl_certs,

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2010-11-06 10:55:58 +0000
+++ bzrlib/transport/http/_urllib.py	2011-06-14 01:26:41 +0000
@@ -172,10 +172,12 @@
 
 def get_test_permutations():
     """Return the permutations to be used in testing."""
-    from bzrlib import tests
-    from bzrlib.tests import http_server
+    from bzrlib.tests import (
+        features,
+        http_server,
+        )
     permutations = [(HttpTransport_urllib, http_server.HttpServer_urllib),]
-    if tests.HTTPSServerFeature.available():
+    if features.HTTPSServerFeature.available():
         from bzrlib.tests import https_server
         permutations.append((HttpTransport_urllib,
                              https_server.HTTPSServer_urllib))

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2011-06-28 21:47:24 +0000
+++ bzrlib/transport/sftp.py	2011-07-23 16:33:38 +0000
@@ -337,10 +337,6 @@
     # up the request itself, rather than us having to worry about it
     _max_request_size = 32768
 
-    def __init__(self, base, _from_transport=None):
-        super(SFTPTransport, self).__init__(base,
-                                            _from_transport=_from_transport)
-
     def _remote_path(self, relpath):
         """Return the path to be passed along the sftp protocol for relpath.
 

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2011-06-19 02:24:39 +0000
+++ bzrlib/tree.py	2011-08-05 13:34:40 +0000
@@ -277,8 +277,11 @@
 
         :param file_id: The file_id of the file.
         :param path: The path of the file.
+
         If both file_id and path are supplied, an implementation may use
         either one.
+
+        :returns: A single byte string for the whole file.
         """
         my_file = self.get_file(file_id, path)
         try:
@@ -297,9 +300,27 @@
         """
         return osutils.split_lines(self.get_file_text(file_id, path))
 
+    def get_file_verifier(self, file_id, path=None, stat_value=None):
+        """Return a verifier for a file.
+
+        The default implementation returns a sha1.
+
+        :param file_id: The handle for this file.
+        :param path: The path that this file can be found at.
+            These must point to the same object.
+        :param stat_value: Optional stat value for the object
+        :return: Tuple with verifier name and verifier data
+        """
+        return ("SHA1", self.get_file_sha1(file_id, path=path,
+            stat_value=stat_value))
+
     def get_file_sha1(self, file_id, path=None, stat_value=None):
         """Return the SHA1 file for a file.
 
+        :note: callers should use get_file_verifier instead
+            where possible, as the underlying repository implementation may
+            have quicker access to a non-sha1 verifier.
+
         :param file_id: The handle for this file.
         :param path: The path that this file can be found at.
             These must point to the same object.
@@ -952,8 +973,8 @@
         if source_kind != target_kind:
             changed_content = True
         elif source_kind == 'file':
-            if (self.source.get_file_sha1(file_id, source_path, source_stat) !=
-                self.target.get_file_sha1(file_id, target_path, target_stat)):
+            if not self.file_content_matches(file_id, file_id, source_path,
+                    target_path, source_stat, target_stat):
                 changed_content = True
         elif source_kind == 'symlink':
             if (self.source.get_symlink_target(file_id) !=
@@ -1272,6 +1293,40 @@
                     changed_file_ids.add(result[0])
                     yield result
 
+    @needs_read_lock
+    def file_content_matches(self, source_file_id, target_file_id,
+            source_path=None, target_path=None, source_stat=None, target_stat=None):
+        """Check if two files are the same in the source and target trees.
+
+        This only checks that the contents of the files are the same,
+        it does not touch anything else.
+
+        :param source_file_id: File id of the file in the source tree
+        :param target_file_id: File id of the file in the target tree
+        :param source_path: Path of the file in the source tree
+        :param target_path: Path of the file in the target tree
+        :param source_stat: Optional stat value of the file in the source tree
+        :param target_stat: Optional stat value of the file in the target tree
+        :return: Boolean indicating whether the files have the same contents
+        """
+        source_verifier_kind, source_verifier_data = self.source.get_file_verifier(
+            source_file_id, source_path, source_stat)
+        target_verifier_kind, target_verifier_data = self.target.get_file_verifier(
+            target_file_id, target_path, target_stat)
+        if source_verifier_kind == target_verifier_kind:
+            return (source_verifier_data == target_verifier_data)
+        # Fall back to SHA1 for now
+        if source_verifier_kind != "SHA1":
+            source_sha1 = self.source.get_file_sha1(source_file_id,
+                    source_path, source_stat)
+        else:
+            source_sha1 = source_verifier_data
+        if target_verifier_kind != "SHA1":
+            target_sha1 = self.target.get_file_sha1(target_file_id,
+                    target_path, target_stat)
+        else:
+            target_sha1 = target_verifier_data
+        return (source_sha1 == target_sha1)
 
 InterTree.register_optimiser(InterTree)
 

=== modified file 'bzrlib/ui/__init__.py'
--- bzrlib/ui/__init__.py	2011-06-10 07:01:07 +0000
+++ bzrlib/ui/__init__.py	2011-08-08 15:57:57 +0000
@@ -47,6 +47,7 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
+    config,
     osutils,
     progress,
     trace,
@@ -145,6 +146,10 @@
             "This may take some time. Upgrade the repositories to the "
             "same format for better performance."
             ),
+        experimental_format_fetch=("Fetching into experimental format "
+            "%(to_format)s.\n"
+            "This format may be unreliable or change in the future "
+            "without an upgrade path.\n"),
         deprecated_command=(
             "The command 'bzr %(deprecated_name)s' "
             "has been deprecated in bzr %(deprecated_in_version)s. "
@@ -243,9 +248,7 @@
         """
         # XXX: is the caller supposed to close the resulting object?
         if encoding is None:
-            from bzrlib import config
-            encoding = config.GlobalConfig().get_user_option(
-                'output_encoding')
+            encoding = config.GlobalStack().get('output_encoding')
         if encoding is None:
             encoding = osutils.get_terminal_encoding(trace=True)
         if encoding_type is None:
@@ -408,22 +411,6 @@
         """Show a warning to the user."""
         raise NotImplementedError(self.show_warning)
 
-    def warn_cross_format_fetch(self, from_format, to_format):
-        """Warn about a potentially slow cross-format transfer.
-        
-        This is deprecated in favor of show_user_warning, but retained for api
-        compatibility in 2.0 and 2.1.
-        """
-        self.show_user_warning('cross_format_fetch', from_format=from_format,
-            to_format=to_format)
-
-    def warn_experimental_format_fetch(self, inter):
-        """Warn about fetching into experimental repository formats."""
-        if inter.target._format.experimental:
-            trace.warning("Fetching into experimental format %s.\n"
-                "This format may be unreliable or change in the future "
-                "without an upgrade path.\n" % (inter.target._format,))
-
 
 class NoninteractiveUIFactory(UIFactory):
     """Base class for UIs with no user."""

=== modified file 'bzrlib/vf_repository.py'
--- bzrlib/vf_repository.py	2011-06-28 17:25:26 +0000
+++ bzrlib/vf_repository.py	2011-08-02 11:18:43 +0000
@@ -1180,6 +1180,18 @@
                 'sha1 mismatch: %s has sha1 %s expected %s referenced by %s' %
                 (record.key, sha1, item_data[1], item_data[2]))
 
+    @needs_read_lock
+    def _eliminate_revisions_not_present(self, revision_ids):
+        """Check every revision id in revision_ids to see if we have it.
+
+        Returns a set of the present revisions.
+        """
+        result = []
+        graph = self.get_graph()
+        parent_map = graph.get_parent_map(revision_ids)
+        # The old API returned a list, should this actually be a set?
+        return parent_map.keys()
+
     def __init__(self, _format, a_bzrdir, control_files):
         """Instantiate a VersionedFileRepository.
 
@@ -2483,7 +2495,10 @@
                             content is copied.
         :return: None.
         """
-        ui.ui_factory.warn_experimental_format_fetch(self)
+        if self.target._format.experimental:
+            ui.ui_factory.show_user_warning('experimental_format_fetch',
+                from_format=self.source._format,
+                to_format=self.target._format)
         from bzrlib.fetch import RepoFetcher
         # See <https://launchpad.net/bugs/456077> asking for a warning here
         if self.source._format.network_name() != self.target._format.network_name():
@@ -2928,7 +2943,10 @@
             revision_ids = fetch_spec.get_keys()
         else:
             revision_ids = None
-        ui.ui_factory.warn_experimental_format_fetch(self)
+        if self.source._format.experimental:
+            ui.ui_factory.show_user_warning('experimental_format_fetch',
+                from_format=self.source._format,
+                to_format=self.target._format)
         if (not self.source.supports_rich_root()
             and self.target.supports_rich_root()):
             self._converting_to_rich_root = True

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2011-06-28 17:25:26 +0000
+++ bzrlib/workingtree.py	2011-08-04 13:30:30 +0000
@@ -595,10 +595,6 @@
             else:
                 return None
 
-    def get_file_sha1(self, file_id, path=None, stat_value=None):
-        # FIXME: Shouldn't this be in Tree?
-        raise NotImplementedError(self.get_file_sha1)
-
     @needs_tree_write_lock
     def _gather_kinds(self, files, kinds):
         """See MutableTree._gather_kinds."""

=== modified file 'doc/developers/configuration.txt'
--- doc/developers/configuration.txt	2011-05-18 08:47:16 +0000
+++ doc/developers/configuration.txt	2011-08-09 09:27:46 +0000
@@ -19,8 +19,9 @@
 * default: the default value that Stack.get() should return if no
   value can be found for the option.
 
-Since plugins should be able to lazily register options, the associated help
-is not part of the object but provided at registration time.
+* help: a doc string describing the option, the first line should be a
+  summary and can be followed by a blank line and a more detailed
+  explanation.
 
 Sections
 --------

=== modified file 'doc/developers/releasing.txt'
--- doc/developers/releasing.txt	2011-05-26 20:42:13 +0000
+++ doc/developers/releasing.txt	2011-08-04 11:46:39 +0000
@@ -2,79 +2,141 @@
 ################
 
 This document describes the processes for making and announcing a Bazaar
-release, and managing the release process.  This is just one phase of
-the `overall development cycle
-<http://doc.bazaar.canonical.com/developers/cycle.html>`_, (go re-read
-this document to ensure it hasn't been updated) but it's the most
-complex part.  This document gives a checklist you can follow from start
-to end in one go.
+release, and managing the release process.  This is just one phase of the
+`overall development cycle
+<http://doc.bazaar.canonical.com/developers/cycle.html>`_, (go re-read this
+document to ensure it hasn't been updated since you last read it) but it's
+the most complex part.
+
+If you're doing your first release you can follow this document and read
+each step explanation. It's also a good practice to read it for any release
+to ensure you don't miss a step and to update it as the release process
+evolves.
 
 If you're helping the Release Manager (RM) for one reason or another, you
 may notice that he didn't follow that document scrupulously. He may have
 good reasons to do that but he may also have missed some parts.
 
-Follow the document yourself and don't hesitate to create the missing
-milestones for example (we tend to forget these ones a lot).
-
 .. contents::
 
 
 Preconditions
 =============
 
+#. PQM access rights (or you won't be able to land any change)
+
 #. Download the pqm plugin and install it into your ``~/.bazaar/plugins``::
 
      bzr branch lp:bzr-pqm ~/.bazaar/plugins/pqm
 
+#. Alternatively, you can download and install ``lp:hydrazine`` (the main
+   difference is that hydrazine requires the branch to land to be hosted on
+   launchpad).
+
+What do we release
+==================
+
+In this document, we're talking about source releases only, packages and
+installers are built from this but we won't talk about them here.
+
+Every release is part of a series, ``bzr-2.4.1`` is part of series ``2.4``.
+
+We do two different kind of releases: the betas releases and the stable
+releases for a given series.
+
+For a given series, releases will be done to deliver new versions of bzr to
+different kinds of users:
+
+#. beta releases: named ``x.ybn`` where ``x.y`` is the series and ``n``
+   starts at 1 and is incremented. These releases are targeted to beta
+   testers who don't want to run from source but are interested in features
+   or improvements.
+
+#. stable releases: name ``x.y.z`` where ``x.y.`` is the series and ``z``
+   starts at 1 and is incremented. These releases are targeted at people
+   that want bugfixes only and no new features.
+
+
+Differences in the release process between beta and stable release will be
+mentioned when needed.
 
 When do we relase ?
 ===================
 
-As of October 2010, we mantain four series.  Concurrently releasing them
-all at the same time makes it harder to shorten the delay between the
-source availability and the package building longer than necessary (we
-delay the official announcement until most of our users can install the new
-release).
+As of July 2011, we maintain four series (and one that is about to be EOLed).
+Concurrently releasing them all at the same time makes it harder to shorten
+the delay between the source availability and the package building longer
+than necessary (we delay the official announcement until most of our users
+can install the new release).
 
 In order to continue to do time-based releases, we need to plan the
 releases by series to minimize the collisions. In the end, it's the Release
 Manager call to decide whether he prefers to do all releases at once
 though, so the rules presented here are a conservative approach.
 
-We want to respect the following rules::
-
- #. as much as possible releases should not disturb development, and
-    ongoing development should not disturb releases,
-
- #. the most recent development series should release once a month during
-    the beta period (see `Development cycles <cycle.html>`_ for more
-    details),
-
- #. the most recent stable series should release every other month (based
-    on the amount of bug fixes, this can be shorter or longer depending on
-    the bugs importance),
-
- #. previous series should relesase on a regular basis without interfering
-    with the most recent series with a decreasing order of priority (again
-    this should be based on bugs importance and user feedback),
-
- #. the death of a series should be planned ahead of time. 6 months should
-    give enough time to our users to migrate to a more recent series. This
-    doesn't mean we will make a release at the end of the series, just that
-    before the end date we _could_ possibly put out another release if
-    there was a sufficiently important fix.  Beyond that date, we won't
-    even land changes on that branch (unless something causes a miraculous
-    resurrection.)
-
- #. there should not be more than 2 releases in the same week (but the
-    Release Manager is free to ignore this (get in touch with packagers
-    though),
-
- #. the series are aligned with Ubuntu releases for convenience since we
-    create a new series every 6 months. This means that we support the
-    stable series for 18 months. Note that we also propose the most recent
-    stable series via the ppa, so whether we keep supporting LTS directly
-    or via the ppa is still an open question.
+We want to respect the following rules:
+
+#. as much as possible releases should not disturb development, and
+   ongoing development should not disturb releases,
+
+#. the most recent development series should release once a month during
+   the beta period (see `Development cycles <cycle.html>`_ for more
+   details),
+
+#. the most recent stable series should release every other month (based
+   on the amount of bug fixes, this can be shorter or longer depending on
+   the bugs importance),
+
+#. previous series should release on a regular basis without interfering
+   with the most recent series with a decreasing order of priority (again
+   this should be based on bugs importance and user feedback),
+
+#. the death of a series should be planned ahead of time. 6 months should
+   give enough time to our users to migrate to a more recent series. This
+   doesn't mean we will make a release at the end of the series, just that
+   before the end date we *could* possibly put out another release if
+   there was a sufficiently important fix.  Beyond that date, we won't
+   even land changes on that branch (unless something causes a miraculous
+   resurrection.)
+
+#. there should not be more than 2 releases in the same week (but the
+   Release Manager is free to ignore this (get in touch with packagers
+   though),
+
+#. the series are aligned with Ubuntu releases for convenience since we
+   create a new series every 6 months. This means that we support the
+   stable series for 18 months. Note that we also propose the most recent
+   stable series via the stable PPA but that the SRU processs allow us to
+   reach a wider audience.
+
+At the start of a series cycle
+==============================
+
+To start a new series cycle:
+
+#. Create a new series ``x.y`` at <https://launchpad.net/bzr/+addseries>.
+
+#. Add milestones at <https://launchpad.net/bzr/x.y/+addmilestone> to that
+   series for the beta releases and the stable series mentioning their
+   expected dates. Only the milestone associated to the next release in
+   this series should be left active to avoid clutter when targeting bugs.
+
+#. If you made a new series, you will need to create a new pqm-controlled
+   branch for this release series. This branch will be used only from the
+   first non-beta release onwards. It needs to be created by a Canonical
+   sysadmin (ask the core devs for instructions or to do it for you).
+
+#. Start a new release-notes file::
+
+       cd doc/en/release-notes
+       cp series-template.txt bzr-x.y.txt  # e.g. bzr-2.3.txt
+       bzr add bzr-x.y.txt
+
+#. Start a new whats-new file::
+
+       cd doc/en/whats-new
+       cp template.txt bzr-x.y.txt  # e.g. bzr-2.6.txt
+       bzr add bzr-x.y.txt
 
 
 At the start of a release cycle
@@ -82,45 +144,32 @@
 
 To start a new release cycle:
 
-#. If this is the first release for a given *x.y* then create a new
-   series at <https://launchpad.net/bzr/+addseries>. There is one series
-   for every *x.y* release.
-
-#. If you made a new series, create a new pqm-controlled branch for this
-   release series, by asking a Canonical sysadmin.  This branch means that
-   from the first release beta or candidate onwards, general development
-   continues on the trunk, and only specifically-targeted fixes go into
-   the release branch.
-
-#. If you made a new series, add milestones at
-   <https://launchpad.net/bzr/x.y/+addmilestone> to that series for
-   the beta release, release candidate and the final release, and their
-   expected dates.
-
-#. Create a new milestone <https://launchpad.net/bzr/x.y/+addmilestone>
-   and add information about this release.  We will not use it yet, but it
-   will be available for targeting or nominating bugs.
-
 #. Send mail to the list with the key dates, who will be the release
    manager, and the main themes or targeted bugs.  Ask people to nominate
    objectives, or point out any high-risk things that are best done early,
    or that interact with other changes. This is called the metronome mail
    and is described in `Development cycles <cycle.html>`_.
 
-#. Make a local branch for preparing this release.  (Only for the first
-   release in a series, otherwise you should already have a branch.) ::
-
-     bzr branch trunk prepare-1.14
+#. Make a local branch to prepare the release::
+
+        bzr branch lp:bzr/x.y x.y-dev
+
+   If you're doing your first beta release, branch from trunk::
+
+       bzr branch lp:bzr x.y-dev
+
+   Note that you will generally reuse the same branch for all releases in a
+   given series.
 
 #. Configure pqm-submit for this branch, with a section like this (where
-   x.y is the version to release). **Or use hydrazine for easy use**
-   ``~/.bazaar/locations.conf``::
+   ``x.y`` is the series for your release). **Or use hydrazine for easier
+   setup** ``~/.bazaar/locations.conf``::
 
-	[/home/mbp/bzr/prepare-x.y]
+	[/home/mbp/bzr/x.y-dev]
 	pqm_email = Canonical PQM <pqm@bazaar-vcs.org>
 	submit_branch = http://bazaar.launchpad.net/~bzr-pqm/bzr/x.y
 	parent_branch = http://bazaar.launchpad.net/~bzr-pqm/bzr/x.y
-	public_branch = http://bazaar.example.com/prepare-x.y
+	public_branch = http://bazaar.example.com/x.y-dev
 	submit_to = bazaar@lists.canonical.com
 	smtp_server = mail.example.com:25
 
@@ -132,18 +181,17 @@
    
        version_info = (x, y, z, 'dev', 0)
    
-#. If you made a new series, then start a new release-notes file::
-
-       cd doc/en/release-notes
-       cp series-template.txt bzr-x.y.txt  # e.g. bzr-2.3.txt
-       bzr add bzr-x.y.txt
-
 #. Add a new section at the top of the current release notes (in
    ``doc/en/release-notes``) about the new release, including its version
    number and the headings from ``release-template.txt``.
 
 #. Update the "What's New" documents in ``doc/en/whats-new``.
 
+#. Make sure a milestone exists for your release and that it is active,
+   <https://launchpad.net/bzr/x.y> lists the existing milestones,
+   <https://launchpad.net/bzr/x.y/x.y.z/+edit> allows you to toggle the
+   active flag.
+
 #. Commit this and send it to PQM.
 
 
@@ -161,7 +209,7 @@
 #. In the release branch, update  ``version_info`` in ``./bzrlib/__init__.py``.
    Make sure the corresponding milestone exists.
    Double check that ./bzr ``_script_version`` matches ``version_info``. Check
-   the output of ``bzr --version``.
+   the output of ``./bzr --version``.
 
    For beta releases use::
 
@@ -171,10 +219,6 @@
 
        version_info = (2, 1, 0, 'beta', 1)
 
-   For release candidates use::
-
-       version_info = (2, 0, 1, 'candidate', SERIAL)
-
    For stable releases use::
 
        version_info = (2, 1, 2, 'final', 0)
@@ -187,7 +231,7 @@
 
    See *2.1.1* or similar for an example of what this looks like.
 
-#. Add a summary of the release into the "What's New" document.
+#. Add or check the summary of the release into the "What's New" document.
 
 #. To check that all bugs mentioned in the release notes are actually
    marked as closed in Launchpad, you can run
@@ -195,10 +239,12 @@
 
      ./tools/check-newsbugs.py doc/en/release-notes/bzr-x.y.txt
 
-   As of 2011-05-26, only a few false positives remain in the older
-   series. Don't let this slow you down too much. This script accepts
-   options you may find useful, use ``./tools/check-newsbugs.py`` to display
-   its usage.
+   As of 2011-07-18, all bugs mentioned in the output of the script requires
+   some sort of intervention (either changing the status if it's not 'Fix
+   Released' or setting a different milestone if the bug hasn't been
+   fixed). A few false positives may remain in the older series, don't let
+   this slow you down too much. This script accepts options you may find
+   useful, use ``./tools/check-newsbugs.py`` to display its usage.
 
 #. Commit these changes to the release branch, using a command like::
 
@@ -276,10 +322,10 @@
 
      bzr tag bzr-2.3.1
 
-#. Push those changes to a bzr repository that is public and accessible on
-   the Internet. PQM will pull from this repository when it attempts to merge
-   your changes. Then submit those changes to PQM for merge into the
-   appropriate release branch::
+#. Push those changes to a bzr branch that is public and accessible on the
+   Internet. PQM will pull from this branch when it attempts to merge your
+   changes. Then submit those changes to PQM for merge into the appropriate
+   release branch::
 
      bzr push
      bzr pqm-submit -m "(vila) Release 2.3.1 (Vincent Ladeuil)"
@@ -320,7 +366,7 @@
 Publishing the source tarball
 -----------------------------
 
-#. Go to the relevant series page in Launchpad.
+#. Go to the relevant <https://launchpad.net/bzr/x.y> series page in Launchpad.
 
 #. Create a release of the milestone, and upload the source tarball and
    the GPG signature.  Or, if you prefer, use the
@@ -331,6 +377,56 @@
    <https://bugs.launchpad.net/launchpad/+bug/586445>
 
 
+Kick off the next cycle
+-----------------------
+
+From that point, there is no possible return, the tarball has been uploaded
+so you can relax a bit.
+
+You're still holding a "social" lock on the launchpad branch though. Until
+your start the next cycle, nobody should land anything on this branch. If
+they do, they either targeted the wrong branch or didn't update the news
+file correctly, so the sooner the branch is opened again, the better.
+
+This matters more for ``lp:bzr`` than for ``lp:bzr/x.y``, ``lp:bzr`` should
+always be open for landing, so you should do `At the start of a release
+cycle`_ as soon as possible (i.e. update the version number in ``bzr`` and
+``bzrlib/__init__``, create/update the news files and create/update the
+milestone for the next relase).
+
+You may also need to do `At the start of a series cycle`_ if you're starting
+a new series.
+
+A word of caution: the instructions above works well for all releases but
+there is one special case that requires a bit more care: when you release
+the *last* beta for a given ``x.y`` series (from trunk aka lp:bzr), you need
+to setup *two* branches for the next cycle:
+
+#. ``lp:bzr`` needs to be opened for the next *series* ``x.(y+1)``
+
+#. ``lp:bzr/x.y`` needs to be opened for the next *release* ``x.y.0`` in the
+   series. Since this is first real use of ``lp:bzr/x.y``, this is also the
+   deadline for the PQM branch to be created.
+
+Both are important as ``lp:bzr`` should remain open so any change can be
+landed, ``lp:bzr/x.y`` on the other hand should be ready to receive bug
+fixes.
+
+``lp:bzr`` is generally more important as the bug fixes on ``lp:bzr/x.y``
+won't be released sooner than a month from now whereas people may already
+been waiting to land on ``lp:bzr``.
+
+In a nutshell:
+
+#. Create or update the ``x.y`` PQM branch based on whatever
+   revision you want to release
+
+#. Open ``lp:bzr`` for ``x.(y+1)``
+
+#. Release ``x.y.0`` from ``lp:bzr/x.y``
+
+#. Open ``lp:bzr/x.y`` for bug fixes
+
 Announcing the source freeze
 ----------------------------
 
@@ -341,13 +437,6 @@
    for platform maintainers and plugin authors to update their code.  This
    is done before the general public announcement of the release.
 
-
-Kick off the next cycle
------------------------
-
-#. To let developers work on the next release, do
-   `At the start of a release cycle` now.
-
 #. Pause for a few days.
 
 
@@ -378,13 +467,14 @@
 
 #. Make an announcement mail.
 
-   For release candidates or beta releases, this is sent to the ``bazaar``
-   list only to inform plugin authors and package or installer managers.
+   For beta releases, this is sent to the ``bazaar@lists.canonical.com``
+   list only to inform plugin authors and people responsible for building
+   packages or installers.
 
    Once the installers are available, the mail can be sent to the
    ``bazaar-announce`` list too.
 
-   For final releases, it should also be cc'd to ``info-gnu@gnu.org``,
+   For stable releases, it should also be cc'd to ``info-gnu@gnu.org``,
    ``python-announce-list@python.org``, ``bug-directory@gnu.org``.
 
    In all cases, it is good to set ``Reply-To: bazaar@lists.canonical.com``,
@@ -413,17 +503,13 @@
 
 #. Make an announcement through <https://launchpad.net/bzr/+announce>
 
-#. Update the IRC channel topic. Use the ``/topic`` command to do this,
-   ensuring the new topic text keeps the project name, web site link, etc.
-
 #. Announce on http://freshmeat.net/projects/bzr/
 
-   This should be done for beta releases, release candidates and final
-   releases. If you do not have a Freshmeat account yet, ask one of the
-   existing admins.
+   This should be done for beta releases and stable releases. If you do not
+   have a Freshmeat account yet, ask one of the existing admins.
 
    The purpose here is to point users to the latest stable release while still
-   publishing announcements for development releases.
+   publishing announcements for beta releases.
 
    There are several kinds of modifications that could be done there via the
    ``Administration`` box in the lower right area of the page:
@@ -432,32 +518,31 @@
      ``Links`` box are edited. This should rarely change except for the URLs
      related to the latest stable release.
 
-   * New announcement: When doing a release (beta, candidates, final), put the
-     summary of the release (you can't embed URLs there, the moderation staff
-     remove them). Users can still access the releases notes via the ``Release
-     Notes`` URL in the ``Links`` box in the upper right area of the
-     page. When doing the first stable release in a series, delete the
-     ``Unstable installers`` <https://launchpad.net/bzr/x.y/x.ybn> and
-     ``Unstable source tarball``
+   * New announcement: When doing a release, put the summary of the release
+     (you can't embed URLs there, the moderation staff remove them). Users
+     can still access the releases notes via the ``Release Notes`` URL in
+     the ``Links`` box in the upper right area of the page. When doing the
+     first stable release in a series, delete the ``Unstable installers``
+     <https://launchpad.net/bzr/x.y/x.ybn> and ``Unstable source tarball``
      <http://launchpad.net/bzr/x.y/x.ybn/+download/bzr-x.ybn.tar.gz>
-     links. Conversely, when creating the first beta in a development series,
-     create these links again. Check all links when doing other kinds of
-     release.
+     links. Conversely, when creating the first beta in a development
+     series, create these links again. Check all links when doing other
+     kinds of release.
 
    * Set direct download: When releasing a new stable release, this should
      point to the corresponding launchpad page:
      <https://launchpad.net/bzr/x.y/x.y.z/>
 
 #. Update `<http://en.wikipedia.org/wiki/Bazaar_(software)>`_ -- this should
-   be done for final releases but not for beta releases or Release Candidates.
+   be done for the stable and beta releases.
 
 #. Update the python package index: <http://pypi.python.org/pypi/bzr> - best
    done by running ::
 
        python setup.py register
 
-   Remember to check the results afterwards -- this should be done for
-   final releases but not for beta releases or Release Candidates.
+   Remember to check the results afterward -- this should be done for
+   stable releases but not for beta releases.
 
    To be able to register the release you must create an account on
    <http://pypi.python.org/pypi> and have one of the existing owners of
@@ -467,10 +552,9 @@
 Merging the released code back to trunk
 ---------------------------------------
 
-Merge the release branch back into the trunk.  The
-``doc/en/release-notes`` changes should be merged into the right place
-because each release series has its own release-notes file, but
-double-check.
+Merge the release branch back into the trunk.  The ``doc/en/release-notes``
+changes should be merged into the right place because each release series
+has its own release-notes file, but double-check.
 
 If it's not already done, advance the version number in ``bzr`` and
 ``bzrlib/__init__.py``.  Submit this back into pqm for bzr.dev.
@@ -479,24 +563,21 @@
 created the corresponding milestone to ensure the continuity in bug
 targeting or nominating. Depending on the change, you may even have to
 create a new series (if your change the major or minor release number), in
-that case go to `At the start of a release cycle` and follow the instructions from there.
+that case go to `At the start of a series cycle`_ and follow the
+instructions from there.
 
-You should also merge (not pull) the release branch into
-``lp:~bzr/bzr/current``, so that branch contains the current released code
-at any time.
 
 Releases until the final one
 ----------------------------
 
-Congratulations - you have made your first release.  Have a beer
-or fruit juice - it's on the house! If it was a beta, or
-candidate, you're not finished yet. Another beta or candidate or
-hopefully a final release is still to come.
+Congratulations - you have made your first release.  Have a beer or fruit
+juice - it's on the house! If it was a beta, you're not finished
+yet. Another beta or hopefully a stable release is still to come.
 
-The process is the same as for the first release. Goto `Doing a
-particular release`_ and follow the instructions again. Some details change
-between beta, candidate and final releases, but they should be
-documented. If the instructions aren't clear enough, please fix them.
+The process is the same as for the first release. Goto `Doing a particular
+release`_ and follow the instructions again. Some details change between
+beta and stable releases, but they should be documented. If the instructions
+aren't clear enough, please fix them.
 
 
 Getting the release into Ubuntu
@@ -515,10 +596,15 @@
 the `SRU (Stable Release Updates)
 <https://wiki.ubuntu.com/StableReleaseUpdates>`__ process.   
 
-As of September 2010, bzr has applied to the technical board to be added
-to the `MicroReleaseExceptions
+Since September 2010, bzr has received approval by the technical
+board for the `MicroReleaseExceptions
 <https://wiki.ubuntu.com/StableReleaseUpdates/MicroReleaseExceptions>`__
-category so that whole bugfix releases can more easily be approved.
+category so that whole bugfix releases can more easily be
+approved.
+
+Progress on these realeases is tracked on the `SRU wiki
+<http://wiki.bazaar.canonical.com/UbuntuStableReleaseUpdates>`_
+page.
 
 **After making a bzr stable-release release, nominate the most serious bug
 for the appropriate Ubuntu release and subscribe the `ubuntu-sru` team.**
@@ -535,9 +621,10 @@
    button and select the right Ubuntu release. As of September 2010, this
    means:
 
+  * ``oneiric`` for the 2.4 series,
+  * ``natty`` for the 2.3 series,
   * ``maverick`` for the 2.2 series,
   * ``lucid`` for the 2.1 series,
-  * ``karmic`` for the 2.0 series.
 
  * Subscribe the ``~ubuntu-sru`` team to the bug.
 

=== modified file 'doc/developers/testing.txt'
--- doc/developers/testing.txt	2011-07-05 05:16:33 +0000
+++ doc/developers/testing.txt	2011-08-04 00:28:09 +0000
@@ -1089,9 +1089,8 @@
 
     tc qdisc add dev lo root handle 1: prio
     tc qdisc add dev lo parent 1:3 handle 30: netem delay 500ms 
-    tc qdisc add dev lo parent 30:1 handle 40: prio
-    tc filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip dport 4155 0xffff flowid 1:3 handle 800::800
-    tc filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip sport 4155 0xffff flowid 1:3 handle 800::801
+    tc filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip dport 4155 0xffff flowid 1:3 
+    tc filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip sport 4155 0xffff flowid 1:3 
 
 and to remove this::
 

=== modified file 'doc/en/mini-tutorial/index.txt'
--- doc/en/mini-tutorial/index.txt	2011-06-28 16:03:09 +0000
+++ doc/en/mini-tutorial/index.txt	2011-07-01 22:01:47 +0000
@@ -56,8 +56,12 @@
 
 Let's suppose we want to store a new project under Bazaar.  First, we'll
 make a *repository directory* to hold all our work related to this
-project.  We can then have multiple branch directories under here, and
-they'll all store the committed history in the repository.
+project, where developers can create branches to test development of
+specific features or, more generally, modifications to the working file
+set.  
+
+After creating the repository, change to that directory, and create the
+project's main trunk branch.
 
 ::
 
@@ -67,9 +71,26 @@
   cd trunk
 
 
+Adding files
+============
+
+Now that we have the trunk, we need to move to that directory and
+create some example files for the first version of that project.  Create
+a file ``test1.txt`` using a text editor (like emacs, nano, or notepad),
+and save it.  Then we'll "add" the file, which tells bzr we want it to
+track changes::
+
+  bzr add test1.txt
+
+and then commit, which saves a snapshot of all versioned files::
+
+  bzr commit -m "Added first line of text"
+
+
 Making changes to your files
 ============================
 
+
 Let's change a file and commit that change to your branch.
 
 Edit ``test1.txt`` in your favourite editor, then check what have you done::

=== modified file 'doc/en/release-notes/bzr-2.4.txt'
--- doc/en/release-notes/bzr-2.4.txt	2011-08-10 09:38:09 +0000
+++ doc/en/release-notes/bzr-2.4.txt	2011-08-12 09:49:24 +0000
@@ -5,10 +5,78 @@
 .. toctree::
    :maxdepth: 1
 
+bzr 2.4.1
+#########
+
+:2.4.1: NOT RELEASED YET
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+
 bzr 2.4.0
 #########
 
-:2.4.0: NOT RELEASED YET
+:2.4.0: 2011-08-11
+
+This release marks the start of a new long-term-stable series. From here, we
+will only make bugfix releases on the 2.4 series (2.4.1, etc, and support it
+until February 2013), while 2.5 will become our new development series.
+
+This is a bugfix and polish release over the 2.3 series, with a large number
+of bugs fixed (>150 for the 2.4 series alone), and some performance
+improvements. Support for python 2.4 and 2.5 has been dropped, many large
+working tree operations have been optimized as well as some stacked branches
+operations.
+
+Only bugfixes from other stables series have been included since 2.4b5 so
+all known fixed bugs are included here.
+
+Users are encouraged to upgrade from the other stable series.
+
 
 External Compatibility Breaks
 *****************************
@@ -61,6 +129,14 @@
 
   (John Arbash Meinel, #609187, #812928)
 
+* Cope with not all Python versions having a ``clear`` method on
+  ``TestCase._type_equality_funcs``.
+  (Martin [gz], Jelmer Vernooij, #809048)
+
+* Fetching tags when fetching the tip revision of a branch is now
+  controlled by the config setting ``branch.fetch_tags``. The behavior has
+  been reverted to 2.3's not-fetching tagged revisions by default.
+  (John Arbash Meinel, #771184)
 
 * The fix for bug #513709 caused us to open a new connection when
   switching a lightweight checkout that was pointing at a bound branch.
@@ -68,9 +144,6 @@
   avoiding an extra SSH connection, etc.
   (John Arbash Meinel, #812285)
 
-* Cope with not all Python versions having a ``clear`` method on
-  ``TestCase._type_equality_funcs``. (#809048, Martin [gz], Jelmer
-  Vernooij)
 
 Documentation
 *************
@@ -116,7 +189,7 @@
 :2.4b5: 2011-07-07
 
 This is the fifth (and last) beta of the 2.4 series leading to
-2.4.0 release in Auguest 2011. Beta releases are suitable for
+2.4.0 release in August 2011. Beta releases are suitable for
 everyday use but may cause some incompatibilities with plugins.
 
 This release includes all bug fixed in previous series known at
@@ -301,7 +374,7 @@
 * Reports the original error when an InvalidHttpResponse exception is
   encountered to facilitate debug. (Vincent Ladeuil, #788530)
 
-* Reports a non-existant file error when trying to merge in a file
+* Reports a non-existent file error when trying to merge in a file
   that does not exist. (Jonathan Riddell, #330063)
 
 * ``UIFactory.prompt``, ``UIFactory.get_username``,
@@ -383,7 +456,7 @@
   (Vincent Ladeuil, #787942)
 
 * Re-target ``bb.test_merge.TestMerge.test_merge_reversed_revision_range``
-  and rewrite it as a parameterized test to avoid unrelated failures.
+  and rewrite it as a parametrized test to avoid unrelated failures.
   (Vincent Ladeuil, #795456)
 
 * Show log file contents from subprocesses started by
@@ -391,7 +464,7 @@
   strange hangs and failures involving subprocesses.  (Andrew Bennetts)
 
 * Skip ``utextwrap`` tests when ``sphinx`` breaks text_wrap by an hostile
-  monkeypatch to textwrap.TextWrapper.wordsep_re.
+  monkey-patch to textwrap.TextWrapper.wordsep_re.
   (Vincent Ladeuil, #785098)
 
 * Multiple ``selftest --exclude`` options are now combined instead of
@@ -1080,7 +1153,7 @@
   by catching them so they can be re-raised in the controlling thread. It's
   available in the ``bzrlib.cethread`` module.  (Vincent Ladeuil)
 
-* Correctly propogate malloc failures from diff-delta.c code as MemoryError
+* Correctly propagate malloc failures from diff-delta.c code as MemoryError
   so OOM conditions during groupcompress are clearly reported. This entailed a
   change to several function signatures. (Martin [gz], #633336)
 

=== added file 'doc/en/release-notes/bzr-2.5.txt'
--- doc/en/release-notes/bzr-2.5.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-2.5.txt	2011-08-09 17:04:46 +0000
@@ -0,0 +1,180 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 2.5b1
+#########
+
+:2.5b1: NOT RELEASED YET
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+* Accessing a packaging branch on Launchpad (eg, ``lp:ubuntu/bzr``) now
+  checks to see if the most recent published source package version for
+  that project is present in the branch tags. This should help developers
+  trust whether the packaging branch is up-to-date and can be used for new
+  changes. The level of verbosity is controlled by the config item
+  ``launchpad.packaging_verbosity``. It can be set to one of
+
+  off
+    disable all checks
+
+
+  minimal
+    only display if the branch is out-of-date
+
+  short
+    also display single-line up-to-date and missing,
+
+
+  all
+    (default) display multi-line content for all states
+
+
+  (John Arbash Meinel, #609187, #812928)
+
+* Add a config option gpg_signing_key for setting which GPG key should
+  be used to sign commits. Also default to using the gpg user identity
+  which matches user_email() as set by whoami. (Jonathan Riddell,
+  #68501)
+
+* bzr log -m now matches message, author, committer and bugs instead
+  of just matching the message.  --message keeps its original meaning,
+  while --match-message, --match-author, --match-committer and
+  --match-bugs match each of those fields.
+
+* Relative local paths can now be specified in URL syntax by using the
+  "file:" prefix.  (Jelmer Vernooij)
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* A call to CHKInventory's filter-method will not result in a
+  DuplicateFileId error, if you move a subfolder and change a file in
+  that subfolder.
+  (Bastian Bowe, #809901)
+
+* Decode ``BZR_HOME`` with fs encoding on posix platforms to avoid unicode
+  errors.  (Vincent Ladeuil, #822571)
+
+* Fix i18n use when no environment variables are set. (Jelmer Vernooij, #810701)
+
+* TreeTransformBase.fixup_new_roots no longer forces trees to have a root, so
+  operations that use it, like merge, can now create trees without a root.
+  (Aaron Bentley)
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+* Release instructions refreshed. (Vincent Ladeuil)
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+* New methods ``get_transport_from_path`` and ``get_transport_from_url``
+  have been added that only support opening from a path or a URL,
+  unlike ``get_transport``. (Jelmer Vernooij)
+
+* New method ``Tree.get_file_verifier`` which allows tree implementations
+  to return non-sha1 checksums to verify files.
+  (Jelmer Vernooij, #720831)
+
+* New method ``InterTree.file_content_matches`` which checks that
+  two files in different trees have the same contents.
+  (Jelmer Vernooij)
+
+* New registry ``OptionRegistry`` specialized for configuration options.
+  (Vincent Ladeuil)
+
+* Remove ``AtomicFile.closed`` which has been deprecated in bzr 0.10.
+  (Vincent Ladeuil)
+
+* Remove ``commands._builtin_commands``, ``commands.shlex_split_unicode``,
+  ``Command._maybe_expand_globs`` and ``Command.run_direct`` deprecated in
+  2.10 and 2.2.0. (Vincent Ladeuil)
+
+* Remove ``diff.get_trees_and_branches_to_diff`` deprecated in 2.2.0.
+
+* Remove ``log.calculate_view_revisions``, ``log._filter_revision_range``,
+  ``log.get_view_revisions`` which have been deprecated in bzr 2.1.0. Also
+  remove ``log.show_one_log`` which was never properly deprecated but wasn't
+  used and is easy to inline if needed. (Vincent Ladeuil)
+
+* Remove ``trace.info``, ``trace.error`` and ``trace.show_log_error``
+  deprecated in 2.1.0. (Vincent Ladeuil)
+
+* Remove ``TransportListRegistry.set_default_transport``, as the concept of
+  a default transport is currently unused. (Jelmer Vernooij)
+
+* Remove ``UIFactory.warn_cross_format_fetch`` and
+  ``UIFactory.warn_experimental_format_fetch`` in favor of
+  ``UIFactory.show_user_warning``. (Jelmer Vernooij)
+
+* There is a new class `ContentFilterTree` that provides a facade for 
+  content filtering.  The `filtered` parameter to `export` is deprecated 
+  in favor of passing a filtered tree, and the specific exporter plugins
+  no longer support it.
+  (Martin Pool)
+
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+* `BranchBuilder.build_snapshot` now supports a "flush" action.  This
+  cleanly and reliably allows tests using `BranchBuilder` to construct
+  branches that e.g. rename files out of a directory and unversion that
+  directory in the same revision.  Previously some changes were impossible
+  due to the order that `build_snapshot` performs its actions.
+  (Andrew Bennetts)
+
+* Don't require ``os.fdatasync`` to be defined on all supported OSes
+  (BSD-based OSes don't define it).  (Vincent Ladeuil, #822649)
+
+* Fix compatibility with testtools 0.9.12. (Jelmer Vernooij, #815423)
+
+* `TestCaseWithMemoryTransport` is faster now: `_check_safety_net` now
+  just compares the bytes in the dirstate file to its pristine state,
+  rather than opening the WorkingTree and calling ``last_revision()``.
+  This reduces the overall test suite time by about 10% on my laptop.
+  (Andrew Bennetts)
+
+* Update `TestCase.knownFailure` to the testtools way of handling expected
+  failures to resolve Python 2.7 incompatibility. (Martin [gz], #607400)
+
+..
+   vim: tw=74 ft=rst ff=unix

=== modified file 'doc/en/user-guide/gpg_signatures.txt'
--- doc/en/user-guide/gpg_signatures.txt	2011-06-22 15:55:13 +0000
+++ doc/en/user-guide/gpg_signatures.txt	2011-07-11 10:53:46 +0000
@@ -53,6 +53,14 @@
 
 ``re-sign`` is also useful to change an existing signature.
 
+By default Bazaar will tell GnuPG to use a key with the same user
+identity as the one set with ``whoami``.  To override this set
+``gpg_signing_key`` in bazaar.conf or locations.conf.
+
+  ``gpg_signing_key=DD4D5088``
+
+  ``gpg_signing_key=amy@example.com``
+
 Verifying Commits
 -----------------
 
@@ -78,6 +86,6 @@
 Work in Progress
 ----------------
 
-There is still a number of digital signature related features which are hoped
-to be added to Bazaar soon.  These include verificiation in logs, qbzr
+There is still a number of digital signature related features which
+are hoped to be added to Bazaar soon.  These include bzr explorer
 integration and setting branches to require signatures.

=== added file 'doc/en/whats-new/template.txt'
--- doc/en/whats-new/template.txt	1970-01-01 00:00:00 +0000
+++ doc/en/whats-new/template.txt	2011-07-18 15:58:50 +0000
@@ -0,0 +1,26 @@
+*************************
+What's New in Bazaar x.y?
+*************************
+
+Bazaar x.y is still under development, and will be released in Month Year.
+This document accumulates a high level summary of what's changed.  See the
+:doc:`../release-notes/index` for a full list.
+
+<topics of interest here>
+
+Further information
+*******************
+
+For more detailed information on the changes made, see the the
+:doc:`../release-notes/index` for:
+
+* the interim bzr `milestones <https://launchpad.net/bzr/x.y>`_
+* the plugins you use.
+
+For a summary of changes made in earlier releases, see:
+
+* :doc:`whats-new-in-2.1`
+* :doc:`whats-new-in-2.2`
+* :doc:`whats-new-in-2.3`
+<intermediate series here>
+* :doc:`whats-new-in-x.(y-1)`

=== modified file 'doc/en/whats-new/whats-new-in-2.4.txt'
--- doc/en/whats-new/whats-new-in-2.4.txt	2011-07-15 09:23:14 +0000
+++ doc/en/whats-new/whats-new-in-2.4.txt	2011-08-12 09:49:24 +0000
@@ -1,8 +1,12 @@
-***********************************
-What's New in Bazaar 2.4 (Oronsay)?
-***********************************
-
-Bazaar 2.4 is still under development, and will be released in August 2011.
+**********************************
+What's New in Bazaar 2.4 (Oronsay)
+**********************************
+
+Bazaar 2.4 has been released on the 8th of August 2011 and marks the start
+of a new long-term-stable series. From here, we will only make bugfix
+releases on the 2.4 series (2.4.1, etc, and support it until February 2013),
+while 2.5 will become our new development series.
+
 This document accumulates a high level summary of what's changed.  See the
 :doc:`../release-notes/index` for a full list.
 
@@ -38,10 +42,11 @@
 ***************************
 
 When tags are copied from a branch, the associated revisions are now copied
-too.  Previously operations like branching, merging or pulling might have
-copied new tags visible in ``bzr tags``, but not copied the revisions.  Now
-revisions from tags will always be present, so that operations like ``bzr
-log -r tag:foo`` will always work.
+too if the config entry ``branch.fetch_tags`` is set to True.  Operations
+like branching, merging or pulling will still always copy new tags visible
+in ``bzr tags``. When the config is set, it will now also copy the
+revisions and their ancestry.  This way tagged revisions will always be
+present, so that operations like ``bzr log -r tag:foo`` will always work.
 
 Deprecated command synonyms
 ***************************
@@ -58,10 +63,10 @@
 Configuration files
 *******************
 
-Option values can now refer to other options in the same configuration file by
-enclosing them in curly brackets (``{option}``). This is an opt-in feature
-during the beta period controlled by the ``bzr.config.expand`` option that
-should be declared in ``bazaar.conf`` and no other file.
+Option values can now refer to other options in the same configuration file
+by enclosing them in curly brackets (``{option}``). This is an opt-in
+feature controlled by the ``bzr.config.expand`` option that should be
+declared in ``bazaar.conf`` and no other file.
 
 Changelog merge plugin
 **********************
@@ -118,7 +123,7 @@
 
 The ``selftest --exclude`` option can now be specified multiple times and
 the tests that match any of the specified patterns will be excluded. Only
-the last specified patetrn was previously taken into account.
+the last specified pattern was previously taken into account.
 
 Digital Signature Verification
 ******************************

=== added file 'doc/en/whats-new/whats-new-in-2.5.txt'
--- doc/en/whats-new/whats-new-in-2.5.txt	1970-01-01 00:00:00 +0000
+++ doc/en/whats-new/whats-new-in-2.5.txt	2011-07-11 07:49:59 +0000
@@ -0,0 +1,36 @@
+*************************
+What's New in Bazaar 2.5?
+*************************
+
+Bazaar 2.5 is still under development, and will be released in February 2012.
+This document accumulates a high level summary of what's changed.  See the
+:doc:`../release-notes/index` for a full list.
+
+Users are encouraged to upgrade from the other stable series.  This
+document outlines the improvements in Bazaar 2.5 vs Bazaar 2.4. As well as
+summarizing improvements made to the core product, it highlights
+enhancements within the broader Bazaar world of potential interest to
+those upgrading.
+
+Bazaar 2.5.0 is fully compatible both locally and on the network with 2.0,
+2.1, 2.2, 2.3 and 2.4, and can read and write repositories generated by all
+previous versions.
+
+Further information
+*******************
+
+For more detailed information on the changes made, see the the
+:doc:`../release-notes/index` for:
+
+* the interim bzr `milestones <https://launchpad.net/bzr/2.5>`_
+* the plugins you use.
+
+For a summary of changes made in earlier releases, see:
+
+* :doc:`whats-new-in-2.1`
+* :doc:`whats-new-in-2.2`
+* :doc:`whats-new-in-2.3`
+* :doc:`whats-new-in-2.4`
+
+..
+   vim: tw=74 ft=rst ff=unix

